-- MySQL dump 10.11
--
-- Host: localhost    Database: baldfury_sswp01
-- ------------------------------------------------------
-- Server version	5.0.75-community-log

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES utf8 */;
/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;
/*!40103 SET TIME_ZONE='+00:00' */;
/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;

--
-- Table structure for table `comments`
--

DROP TABLE IF EXISTS `comments`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `comments` (
  `comment_ID` bigint(20) unsigned NOT NULL auto_increment,
  `comment_post_ID` int(11) NOT NULL default '0',
  `comment_author` tinytext NOT NULL,
  `comment_author_email` varchar(100) NOT NULL default '',
  `comment_author_url` varchar(200) NOT NULL default '',
  `comment_author_IP` varchar(100) NOT NULL default '',
  `comment_date` datetime NOT NULL default '0000-00-00 00:00:00',
  `comment_date_gmt` datetime NOT NULL default '0000-00-00 00:00:00',
  `comment_content` text NOT NULL,
  `comment_karma` int(11) NOT NULL default '0',
  `comment_approved` varchar(20) NOT NULL default '1',
  `comment_agent` varchar(255) NOT NULL default '',
  `comment_type` varchar(20) NOT NULL default '',
  `comment_parent` bigint(20) NOT NULL default '0',
  `user_id` bigint(20) NOT NULL default '0',
  PRIMARY KEY  (`comment_ID`),
  KEY `comment_approved` (`comment_approved`),
  KEY `comment_post_ID` (`comment_post_ID`),
  KEY `comment_approved_date_gmt` (`comment_approved`,`comment_date_gmt`),
  KEY `comment_date_gmt` (`comment_date_gmt`)
) ENGINE=MyISAM AUTO_INCREMENT=7 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `comments`
--

LOCK TABLES `comments` WRITE;
/*!40000 ALTER TABLE `comments` DISABLE KEYS */;
INSERT INTO `comments` VALUES (1,1,'Mr WordPress','','http://wordpress.org/','','2008-07-14 22:33:23','2008-07-15 04:33:23','Hi, this is a comment.<br />To delete a comment, just log in and view the post&#039;s comments. There you will have the option to edit or delete them.',0,'1','','',0,0),(2,1,'Isosorbide Mononitrate','guest19811@mail.ru','http://www.eva-pharmacy.com','92.243.166.98','2008-09-29 16:39:29','2008-09-29 22:39:29','Ran in front locomotive, which arrived late.',0,'0','Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 4.0) Opera 7.0 [en]','',0,0),(3,1,'ED Trial Pack','grdgereg@gmail.com','http://www.eva-pharmacy.com','92.243.166.98','2008-10-02 07:17:51','2008-10-02 13:17:51','Suggest argue, because only in a dispute born truth.',0,'0','Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Crazy Browser 1.0.5)','',0,0),(4,1,'AnnHie','anaenvoixliex@mail.ru','http://www.elcocn.com/','194.165.42.81','2008-10-20 11:55:04','2008-10-20 17:55:04','Hello.\r\nIt\'s better to put source link\r\n\r\nThanks. Yours Anne!',0,'0','Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322; FDM)','',0,0),(5,1,'Bolivo','juimmerturo@mail.ru','http://www.contuser.com/','194.165.42.81','2008-10-27 07:10:46','2008-10-27 13:10:46','Hello.\r\nFucking hard thinking posts\r\n\r\nGood Luck!\r\nAndy.',0,'0','Mozilla/4.61 [en] (X11; U; ) - BrowseX (2.0.0 Windows)','',0,0),(6,1,'Inga','grdgereg@gmail.com','http://filmnew.ru','92.243.166.98','2008-11-05 19:43:56','2008-11-06 01:43:56','Impressa:) or as a Portuguese, vpechatlilso!',0,'0','Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; FunWebProducts; MRA 4.6 (build 01425); .NET CLR 1.1.4322; .NET CLR 2.0.50727)','',0,0);
/*!40000 ALTER TABLE `comments` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `links`
--

DROP TABLE IF EXISTS `links`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `links` (
  `link_id` bigint(20) NOT NULL auto_increment,
  `link_url` varchar(255) NOT NULL default '',
  `link_name` varchar(255) NOT NULL default '',
  `link_image` varchar(255) NOT NULL default '',
  `link_target` varchar(25) NOT NULL default '',
  `link_category` bigint(20) NOT NULL default '0',
  `link_description` varchar(255) NOT NULL default '',
  `link_visible` varchar(20) NOT NULL default 'Y',
  `link_owner` int(11) NOT NULL default '1',
  `link_rating` int(11) NOT NULL default '0',
  `link_updated` datetime NOT NULL default '0000-00-00 00:00:00',
  `link_rel` varchar(255) NOT NULL default '',
  `link_notes` mediumtext NOT NULL,
  `link_rss` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`link_id`),
  KEY `link_category` (`link_category`),
  KEY `link_visible` (`link_visible`)
) ENGINE=MyISAM AUTO_INCREMENT=8 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `links`
--

LOCK TABLES `links` WRITE;
/*!40000 ALTER TABLE `links` DISABLE KEYS */;
INSERT INTO `links` VALUES (1,'http://codex.wordpress.org/','Documentation','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(2,'http://wordpress.org/development/','Development Blog','','',0,'','Y',1,0,'0000-00-00 00:00:00','','','http://wordpress.org/development/feed/'),(3,'http://wordpress.org/extend/ideas/','Suggest Ideas','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(4,'http://wordpress.org/support/','Support Forum','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(5,'http://wordpress.org/extend/plugins/','Plugins','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(6,'http://wordpress.org/extend/themes/','Themes','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(7,'http://planet.wordpress.org/','WordPress Planet','','',0,'','Y',1,0,'0000-00-00 00:00:00','','','');
/*!40000 ALTER TABLE `links` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `options`
--

DROP TABLE IF EXISTS `options`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `options` (
  `option_id` bigint(20) NOT NULL auto_increment,
  `blog_id` int(11) NOT NULL default '0',
  `option_name` varchar(64) NOT NULL default '',
  `option_value` longtext NOT NULL,
  `autoload` varchar(20) NOT NULL default 'yes',
  PRIMARY KEY  (`option_id`,`blog_id`,`option_name`),
  KEY `option_name` (`option_name`)
) ENGINE=MyISAM AUTO_INCREMENT=100 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `options`
--

LOCK TABLES `options` WRITE;
/*!40000 ALTER TABLE `options` DISABLE KEYS */;
INSERT INTO `options` VALUES (1,0,'siteurl','http://www.garff.org/','yes'),(2,0,'blogname','The World According to Garff','yes'),(3,0,'blogdescription','Just another WordPress weblog','yes'),(4,0,'users_can_register','0','yes'),(5,0,'admin_email','you@yourdomain.com','yes'),(6,0,'start_of_week','1','yes'),(7,0,'use_balanceTags','','yes'),(8,0,'use_smilies','1','yes'),(9,0,'require_name_email','1','yes'),(10,0,'comments_notify','1','yes'),(11,0,'posts_per_rss','10','yes'),(12,0,'rss_excerpt_length','50','yes'),(13,0,'rss_use_excerpt','0','yes'),(14,0,'mailserver_url','mail.example.com','yes'),(15,0,'mailserver_login','login@example.com','yes'),(16,0,'mailserver_pass','password','yes'),(17,0,'mailserver_port','110','yes'),(18,0,'default_category','1','yes'),(19,0,'default_comment_status','open','yes'),(20,0,'default_ping_status','open','yes'),(21,0,'default_pingback_flag','1','yes'),(22,0,'default_post_edit_rows','10','yes'),(23,0,'posts_per_page','10','yes'),(24,0,'what_to_show','posts','yes'),(25,0,'date_format','F j, Y','yes'),(26,0,'time_format','g:i a','yes'),(27,0,'links_updated_date_format','F j, Y g:i a','yes'),(28,0,'links_recently_updated_prepend','<em>','yes'),(29,0,'links_recently_updated_append','</em>','yes'),(30,0,'links_recently_updated_time','120','yes'),(31,0,'comment_moderation','0','yes'),(32,0,'moderation_notify','1','yes'),(33,0,'permalink_structure','','yes'),(34,0,'gzipcompression','0','yes'),(35,0,'hack_file','0','yes'),(36,0,'blog_charset','UTF-8','yes'),(37,0,'moderation_keys','','no'),(38,0,'active_plugins','','yes'),(39,0,'home','http://www.garff.org/','yes'),(40,0,'category_base','','yes'),(41,0,'ping_sites','http://rpc.pingomatic.com/','yes'),(42,0,'advanced_edit','0','yes'),(43,0,'comment_max_links','2','yes'),(44,0,'gmt_offset','-6','yes'),(45,0,'default_email_category','1','yes'),(46,0,'recently_edited','','no'),(47,0,'use_linksupdate','0','yes'),(48,0,'template','default','yes'),(49,0,'stylesheet','default','yes'),(50,0,'comment_whitelist','1','yes'),(51,0,'page_uris','','yes'),(52,0,'blacklist_keys','','no'),(53,0,'comment_registration','0','yes'),(54,0,'rss_language','en','yes'),(55,0,'html_type','text/html','yes'),(56,0,'use_trackback','0','yes'),(57,0,'default_role','subscriber','yes'),(58,0,'db_version','8204','yes'),(59,0,'uploads_use_yearmonth_folders','1','yes'),(60,0,'upload_path','/home/baldfury/public_html/garff/wp-content/uploads','yes'),(61,0,'secret','E1wTthshzrvNk9UB4XLIb66byWxCbP3TCTAMqLRCjIh7nwC5CzrFUK1qPEMTwGbg','yes'),(62,0,'blog_public','1','yes'),(63,0,'default_link_category','2','yes'),(64,0,'show_on_front','posts','yes'),(65,0,'tag_base','','yes'),(66,0,'show_avatars','1','yes'),(67,0,'avatar_rating','G','yes'),(68,0,'upload_url_path','','yes'),(69,0,'thumbnail_size_w','150','yes'),(70,0,'thumbnail_size_h','150','yes'),(71,0,'thumbnail_crop','1','yes'),(72,0,'medium_size_w','300','yes'),(73,0,'medium_size_h','300','yes'),(74,0,'avatar_default','mystery','yes'),(75,0,'enable_app','','yes'),(76,0,'enable_xmlrpc','1','yes'),(77,0,'user_roles','a:5:{s:13:\"administrator\";a:2:{s:4:\"name\";s:23:\"Administrator|User role\";s:12:\"capabilities\";a:51:{s:13:\"switch_themes\";b:1;s:11:\"edit_themes\";b:1;s:16:\"activate_plugins\";b:1;s:12:\"edit_plugins\";b:1;s:10:\"edit_users\";b:1;s:10:\"edit_files\";b:1;s:14:\"manage_options\";b:1;s:17:\"moderate_comments\";b:1;s:17:\"manage_categories\";b:1;s:12:\"manage_links\";b:1;s:12:\"upload_files\";b:1;s:6:\"import\";b:1;s:15:\"unfiltered_html\";b:1;s:10:\"edit_posts\";b:1;s:17:\"edit_others_posts\";b:1;s:20:\"edit_published_posts\";b:1;s:13:\"publish_posts\";b:1;s:10:\"edit_pages\";b:1;s:4:\"read\";b:1;s:8:\"level_10\";b:1;s:7:\"level_9\";b:1;s:7:\"level_8\";b:1;s:7:\"level_7\";b:1;s:7:\"level_6\";b:1;s:7:\"level_5\";b:1;s:7:\"level_4\";b:1;s:7:\"level_3\";b:1;s:7:\"level_2\";b:1;s:7:\"level_1\";b:1;s:7:\"level_0\";b:1;s:17:\"edit_others_pages\";b:1;s:20:\"edit_published_pages\";b:1;s:13:\"publish_pages\";b:1;s:12:\"delete_pages\";b:1;s:19:\"delete_others_pages\";b:1;s:22:\"delete_published_pages\";b:1;s:12:\"delete_posts\";b:1;s:19:\"delete_others_posts\";b:1;s:22:\"delete_published_posts\";b:1;s:20:\"delete_private_posts\";b:1;s:18:\"edit_private_posts\";b:1;s:18:\"read_private_posts\";b:1;s:20:\"delete_private_pages\";b:1;s:18:\"edit_private_pages\";b:1;s:18:\"read_private_pages\";b:1;s:12:\"delete_users\";b:1;s:12:\"create_users\";b:1;s:17:\"unfiltered_upload\";b:1;s:14:\"edit_dashboard\";b:1;s:14:\"update_plugins\";b:1;s:14:\"delete_plugins\";b:1;}}s:6:\"editor\";a:2:{s:4:\"name\";s:16:\"Editor|User role\";s:12:\"capabilities\";a:34:{s:17:\"moderate_comments\";b:1;s:17:\"manage_categories\";b:1;s:12:\"manage_links\";b:1;s:12:\"upload_files\";b:1;s:15:\"unfiltered_html\";b:1;s:10:\"edit_posts\";b:1;s:17:\"edit_others_posts\";b:1;s:20:\"edit_published_posts\";b:1;s:13:\"publish_posts\";b:1;s:10:\"edit_pages\";b:1;s:4:\"read\";b:1;s:7:\"level_7\";b:1;s:7:\"level_6\";b:1;s:7:\"level_5\";b:1;s:7:\"level_4\";b:1;s:7:\"level_3\";b:1;s:7:\"level_2\";b:1;s:7:\"level_1\";b:1;s:7:\"level_0\";b:1;s:17:\"edit_others_pages\";b:1;s:20:\"edit_published_pages\";b:1;s:13:\"publish_pages\";b:1;s:12:\"delete_pages\";b:1;s:19:\"delete_others_pages\";b:1;s:22:\"delete_published_pages\";b:1;s:12:\"delete_posts\";b:1;s:19:\"delete_others_posts\";b:1;s:22:\"delete_published_posts\";b:1;s:20:\"delete_private_posts\";b:1;s:18:\"edit_private_posts\";b:1;s:18:\"read_private_posts\";b:1;s:20:\"delete_private_pages\";b:1;s:18:\"edit_private_pages\";b:1;s:18:\"read_private_pages\";b:1;}}s:6:\"author\";a:2:{s:4:\"name\";s:16:\"Author|User role\";s:12:\"capabilities\";a:10:{s:12:\"upload_files\";b:1;s:10:\"edit_posts\";b:1;s:20:\"edit_published_posts\";b:1;s:13:\"publish_posts\";b:1;s:4:\"read\";b:1;s:7:\"level_2\";b:1;s:7:\"level_1\";b:1;s:7:\"level_0\";b:1;s:12:\"delete_posts\";b:1;s:22:\"delete_published_posts\";b:1;}}s:11:\"contributor\";a:2:{s:4:\"name\";s:21:\"Contributor|User role\";s:12:\"capabilities\";a:5:{s:10:\"edit_posts\";b:1;s:4:\"read\";b:1;s:7:\"level_1\";b:1;s:7:\"level_0\";b:1;s:12:\"delete_posts\";b:1;}}s:10:\"subscriber\";a:2:{s:4:\"name\";s:20:\"Subscriber|User role\";s:12:\"capabilities\";a:2:{s:4:\"read\";b:1;s:7:\"level_0\";b:1;}}}','yes'),(78,0,'update_core','O:8:\"stdClass\":5:{s:12:\"last_checked\";i:1235759150;s:15:\"version_checked\";s:5:\"2.6.1\";s:8:\"response\";s:7:\"upgrade\";s:3:\"url\";s:30:\"http://wordpress.org/download/\";s:7:\"current\";s:5:\"2.7.1\";}','yes'),(79,0,'update_plugins','O:8:\"stdClass\":3:{s:12:\"last_checked\";i:1235759150;s:7:\"checked\";a:2:{s:19:\"akismet/akismet.php\";s:5:\"2.1.6\";s:9:\"hello.php\";s:3:\"1.5\";}s:8:\"response\";a:1:{s:19:\"akismet/akismet.php\";O:8:\"stdClass\":5:{s:2:\"id\";s:2:\"15\";s:4:\"slug\";s:7:\"akismet\";s:11:\"new_version\";s:5:\"2.2.3\";s:3:\"url\";s:44:\"http://wordpress.org/extend/plugins/akismet/\";s:7:\"package\";s:49:\"http://downloads.wordpress.org/plugin/akismet.zip\";}}}','yes'),(80,0,'auth_salt','4Ryq%7WCn2e1','yes'),(81,0,'logged_in_salt','NT09Ud*di6R7','yes'),(82,0,'dashboard_widget_options','a:3:{s:24:\"dashboard_incoming_links\";a:5:{s:4:\"home\";s:20:\"http://www.garff.org\";s:4:\"link\";s:102:\"http://blogsearch.google.com/blogsearch?hl=en&scoring=d&partner=wordpress&q=link:http://www.garff.org/\";s:3:\"url\";s:135:\"http://blogsearch.google.com/blogsearch_feeds?hl=en&scoring=d&ie=utf-8&num=10&output=rss&partner=wordpress&q=link:http://www.garff.org/\";s:5:\"items\";i:5;s:9:\"show_date\";i:0;}s:17:\"dashboard_primary\";a:7:{s:4:\"link\";s:33:\"http://wordpress.org/development/\";s:3:\"url\";s:38:\"http://wordpress.org/development/feed/\";s:5:\"title\";s:26:\"WordPress Development Blog\";s:5:\"items\";i:2;s:12:\"show_summary\";i:1;s:11:\"show_author\";i:0;s:9:\"show_date\";i:1;}s:19:\"dashboard_secondary\";a:4:{s:4:\"link\";s:28:\"http://planet.wordpress.org/\";s:3:\"url\";s:33:\"http://planet.wordpress.org/feed/\";s:5:\"title\";s:20:\"Other WordPress News\";s:5:\"items\";i:15;}}','yes'),(83,0,'current_theme','WordPress Default','yes'),(84,0,'sidebars_widgets','a:1:{s:13:\"array_version\";i:3;}','yes'),(85,0,'rss_cb7939b78822598324486bb696a27b33','O:9:\"magpierss\":17:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:0:{}s:7:\"channel\";a:4:{s:5:\"title\";s:47:\"link:http://www.garff.org/ - Google Blog Search\";s:4:\"link\";s:105:\"http://blogsearch.google.com/blogsearch?hl=en&scoring=d&ie=ISO-8859-1&num=10&q=link:http://www.garff.org/\";s:11:\"description\";s:81:\"Your search - <b>link:http://www.garff.org/</b> - did not match any documents.   \";s:7:\"tagline\";s:81:\"Your search - <b>link:http://www.garff.org/</b> - did not match any documents.   \";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}}','no'),(86,0,'rss_cb7939b78822598324486bb696a27b33_ts','1217943918','no'),(87,0,'rss_867bd5c64f85878d03a060509cd2f92c','O:9:\"magpierss\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:50:{i:0;a:7:{s:5:\"title\";s:155:\"WordPress Podcast: Episode 44: WordPress 2.6 released, iPhone app, XML-RPC controversy, theme repository, and an exclusive interview with Woopra developers\";s:4:\"guid\";s:29:\"http://wp-community.org/?p=80\";s:4:\"link\";s:46:\"http://wp-community.org/2008/08/04/episode-44/\";s:11:\"description\";s:546:\"<p>Yes, everyone, I&#8217;m aware that I&#8217;m a month late in publishing this episode.</p>\n<p>Here&#8217;s the deal: The entire episode is fully edited, it just needs to be segmented and uploaded to Podango Show Builder Lite for assembly. Then, the show notes need to be written. When all that is done, it&#8217;ll be published in place of this notice.</p>\n<p>Fair warning: The entire episode clocks in at over 1 hour 20 minutes because we had so much material to slog through. Rest assured, as soon as I am able, I&#8217;ll be posting it.</p>\";s:7:\"pubdate\";s:31:\"Tue, 05 Aug 2008 05:27:48 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:17:\"Charles Stricklin\";}s:7:\"summary\";s:546:\"<p>Yes, everyone, I&#8217;m aware that I&#8217;m a month late in publishing this episode.</p>\n<p>Here&#8217;s the deal: The entire episode is fully edited, it just needs to be segmented and uploaded to Podango Show Builder Lite for assembly. Then, the show notes need to be written. When all that is done, it&#8217;ll be published in place of this notice.</p>\n<p>Fair warning: The entire episode clocks in at over 1 hour 20 minutes because we had so much material to slog through. Rest assured, as soon as I am able, I&#8217;ll be posting it.</p>\";}i:1;a:7:{s:5:\"title\";s:40:\"Ryan Boren: iPhone 2.0.1 with WP App 1.1\";s:4:\"guid\";s:62:\"http://boren.nu/archives/2008/08/04/iphone-201-with-wp-app-11/\";s:4:\"link\";s:62:\"http://boren.nu/archives/2008/08/04/iphone-201-with-wp-app-11/\";s:11:\"description\";s:341:\"<p>I&#8217;m trying out the iPhone 2.0.1 firmware along with WP App 1.1.  Both were released today.  2.0.1 seems to fix the keyboard lag that was coaxing expletives out of me.  WP 1.1 is solid so far.  We&#8217;ve been working on extending the WP XML-RPC API to allow the iPhone App (and other XML-RPC clients) to do some cool new stuff.</p>\";s:7:\"pubdate\";s:31:\"Tue, 05 Aug 2008 03:15:10 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ryan\";}s:7:\"summary\";s:341:\"<p>I&#8217;m trying out the iPhone 2.0.1 firmware along with WP App 1.1.  Both were released today.  2.0.1 seems to fix the keyboard lag that was coaxing expletives out of me.  WP 1.1 is solid so far.  We&#8217;ve been working on extending the WP XML-RPC API to allow the iPhone App (and other XML-RPC clients) to do some cool new stuff.</p>\";}i:2;a:7:{s:5:\"title\";s:23:\"Matt: Community Tagging\";s:4:\"guid\";s:41:\"http://ma.tt/2008/08/community-tagging-2/\";s:4:\"link\";s:41:\"http://ma.tt/2008/08/community-tagging-2/\";s:11:\"description\";s:411:\"<p><a href=\"http://wordpress.org/extend/plugins/matts-community-tags/\">Matt&#8217;s Community Tags</a>. This is the VERY BETA plugin I&#8217;m using for the community tagging on my photos, which allows people to submit tags which then go into a moderation queue to be approved or modified by an admin. Not recommended for general use yet, just getting it out there since a lot of people have asked about it.</p>\";s:7:\"pubdate\";s:31:\"Mon, 04 Aug 2008 23:20:50 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:411:\"<p><a href=\"http://wordpress.org/extend/plugins/matts-community-tags/\">Matt&#8217;s Community Tags</a>. This is the VERY BETA plugin I&#8217;m using for the community tagging on my photos, which allows people to submit tags which then go into a moderation queue to be approved or modified by an admin. Not recommended for general use yet, just getting it out there since a lot of people have asked about it.</p>\";}i:3;a:7:{s:5:\"title\";s:59:\"Weblog Tools Collection: WordPress Theme Releases for 08/04\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3859\";s:4:\"link\";s:87:\"http://weblogtoolscollection.com/archives/2008/08/04/wordpress-theme-releases-for-0804/\";s:11:\"description\";s:1809:\"<p><a href=\"http://daus.trala.la/2008/08/02/refreshed-wordpress-theme/\"><strong>Refreshed</strong></a></p>\n<p><img alt=\"refreshed\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/refreshed.jpg\" border=\"0\" width=\"200\" height=\"114\" /></p>\n<p>Refreshed is a one column, white-spaced, and clean design with a lifestream feature outside the regular post stream. So your lifestream will not junks to any RSS reader.</p>\n<p><a href=\"http://www.codescheme.net/2008/08/04/free-wordpress-theme-ringbinder/\"><strong>RingBinder</strong></a></p>\n<p><a href=\"http://demo.codescheme.net/?wptheme=ringbinder\"><img alt=\"ringbinder\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/ringbinder.gif\" border=\"0\" width=\"146\" height=\"114\" /></a></p>\n<p>RingBinder is a 2 column fixed-width, free WordPress theme, with righthand sidebar, enabled for widgets - the name is reasonably self-explanatory.</p>\n<p><a href=\"http://www.simplywp.net/2008/08/04/entrecard-theme-is-here/\"><strong>Entrecard</strong></a></p>\n<p><img alt=\"entrecard_theme\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/entrecard-theme.png\" border=\"0\" width=\"188\" height=\"114\" /></p>\n<p>4 columns, widget ready, right sidebar, login menu at the header</p>\n<p><a href=\"http://easternwest.com/simplifi-dark-brown-wordpress-theme/\"><strong>Simplifi Dark (brown)</strong></a></p>\n<p><img alt=\"simplifidarkbrown\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/simplifidarkbrown.png\" border=\"0\" width=\"200\" height=\"97\" /></p>\n<p>Dark brown, two column theme</p>\n<p><a href=\"http://www.azmoney.co.uk/wordpress-themes/vista\"><strong>Vista</strong></a></p>\n<p><img alt=\"vista\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/vista.png\" border=\"0\" width=\"152\" height=\"114\" /></p>\n<p>Three column, widget and gravatar ready, SEO optimized theme</p>\";s:7:\"pubdate\";s:31:\"Mon, 04 Aug 2008 20:19:59 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:1809:\"<p><a href=\"http://daus.trala.la/2008/08/02/refreshed-wordpress-theme/\"><strong>Refreshed</strong></a></p>\n<p><img alt=\"refreshed\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/refreshed.jpg\" border=\"0\" width=\"200\" height=\"114\" /></p>\n<p>Refreshed is a one column, white-spaced, and clean design with a lifestream feature outside the regular post stream. So your lifestream will not junks to any RSS reader.</p>\n<p><a href=\"http://www.codescheme.net/2008/08/04/free-wordpress-theme-ringbinder/\"><strong>RingBinder</strong></a></p>\n<p><a href=\"http://demo.codescheme.net/?wptheme=ringbinder\"><img alt=\"ringbinder\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/ringbinder.gif\" border=\"0\" width=\"146\" height=\"114\" /></a></p>\n<p>RingBinder is a 2 column fixed-width, free WordPress theme, with righthand sidebar, enabled for widgets - the name is reasonably self-explanatory.</p>\n<p><a href=\"http://www.simplywp.net/2008/08/04/entrecard-theme-is-here/\"><strong>Entrecard</strong></a></p>\n<p><img alt=\"entrecard_theme\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/entrecard-theme.png\" border=\"0\" width=\"188\" height=\"114\" /></p>\n<p>4 columns, widget ready, right sidebar, login menu at the header</p>\n<p><a href=\"http://easternwest.com/simplifi-dark-brown-wordpress-theme/\"><strong>Simplifi Dark (brown)</strong></a></p>\n<p><img alt=\"simplifidarkbrown\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/simplifidarkbrown.png\" border=\"0\" width=\"200\" height=\"97\" /></p>\n<p>Dark brown, two column theme</p>\n<p><a href=\"http://www.azmoney.co.uk/wordpress-themes/vista\"><strong>Vista</strong></a></p>\n<p><img alt=\"vista\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/vista.png\" border=\"0\" width=\"152\" height=\"114\" /></p>\n<p>Three column, widget and gravatar ready, SEO optimized theme</p>\";}i:4;a:7:{s:5:\"title\";s:28:\"Matt: Random Redirect Plugin\";s:4:\"guid\";s:46:\"http://ma.tt/2008/08/random-redirect-plugin-2/\";s:4:\"link\";s:46:\"http://ma.tt/2008/08/random-redirect-plugin-2/\";s:11:\"description\";s:145:\"<p><a href=\"http://wordpress.org/extend/plugins/random-redirect/\">I just updated the  Random Redirect plugin</a>, with two extra parameters.</p>\";s:7:\"pubdate\";s:31:\"Mon, 04 Aug 2008 18:25:36 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:145:\"<p><a href=\"http://wordpress.org/extend/plugins/random-redirect/\">I just updated the  Random Redirect plugin</a>, with two extra parameters.</p>\";}i:5;a:7:{s:5:\"title\";s:60:\"Weblog Tools Collection: WordPress Plugin Releases for 08/03\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3851\";s:4:\"link\";s:88:\"http://weblogtoolscollection.com/archives/2008/08/03/wordpress-plugin-releases-for-0803/\";s:11:\"description\";s:2700:\"<p><a href=\"http://www.viper007bond.com/wordpress-plugins/jquery-lightbox-for-native-galleries/\"><strong>jQuery Lightbox for Native Galleries</strong></a></p>\n<p>Makes the native WordPress galleries introduced in WordPress 2.5 use <a href=\"http://plugins.jquery.com/project/jquerylightbox_bal\">jQuery Lightbox by balupton</a> to display the fullsize images.</p>\n<p><a href=\"http://www.viper007bond.com/wordpress-plugins/disable-theme-preview/\"><strong>Disable Theme Preview</strong></a></p>\n<p>Disables the cool but sometimes annoying theme preview feature added in WordPress 2.6. You’ll still be able to preview themes by clicking a link located below the theme’s description.</p>\n<p><a href=\"http://www.viper007bond.com/wordpress-plugins/direct-image-urls-for-galleries/\"><strong>Direct Image URLs For Galleries</strong></a></p>\n<p>Modifies thumbnails to link to fullsize image instead of the page containing the medium thumbnail.</p>\n<p><a href=\"http://wordpress.org/extend/plugins/statrix/\"><strong>Statrix</strong></a></p>\n<p>Statrix plugin collects data about your blog visits. It creates charts and lists of the full data history in real-time, which you can browse, filter and export immediately after plugin activation. The plugin keeps the full visits history, so you can make reports about any period of time you want.</p>\n<p><a href=\"http://www.basilv.com/psd/software#VersionInformation\"><strong>Version Information</strong></a></p>\n<p>This plugin displays version and configuration information for WordPress, PHP, MySQL, and the web server. A summary is displayed in the Site Admin dashboard and more details are displayed under a Version Information page under the Site Admin Plugins menu.</p>\n<p><a href=\"http://www.theblog.ca/wordpress-collaboration-emails\"><strong>Peter&#8217;s Collaboration Emails</strong></a></p>\n<p>Enhance the &#8220;Submit for Review&#8221; feature for Contributor users in WordPress 2.3 and above. This plugin enables automatic e-mails to the relevant users when posts are pending and when they are approved.</p>\n<p><a href=\"http://www.theblog.ca/anti-spam\"><strong>Peter&#8217;s Custom Anti-Spam Image</strong></a></p>\n<p>This differs from the other image plugins in that it creates an anti-spam jpg using random words of your choice (adding a fun element) and it does not require visitors to have cookies or JavaScript enabled. It also generates audio files for visually impaired visitors.</p>\n<p><a href=\"http://scribu.net/projects/extra-feed-links.html\"><strong>Extra Feed Links</strong></a></p>\n<p>Adds links to the specific feed for each page type in Wordpress. Ex: Comments Feed for single posts and pages; Category Feed for category pages etc.</p>\";s:7:\"pubdate\";s:31:\"Sun, 03 Aug 2008 19:03:42 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:2700:\"<p><a href=\"http://www.viper007bond.com/wordpress-plugins/jquery-lightbox-for-native-galleries/\"><strong>jQuery Lightbox for Native Galleries</strong></a></p>\n<p>Makes the native WordPress galleries introduced in WordPress 2.5 use <a href=\"http://plugins.jquery.com/project/jquerylightbox_bal\">jQuery Lightbox by balupton</a> to display the fullsize images.</p>\n<p><a href=\"http://www.viper007bond.com/wordpress-plugins/disable-theme-preview/\"><strong>Disable Theme Preview</strong></a></p>\n<p>Disables the cool but sometimes annoying theme preview feature added in WordPress 2.6. You’ll still be able to preview themes by clicking a link located below the theme’s description.</p>\n<p><a href=\"http://www.viper007bond.com/wordpress-plugins/direct-image-urls-for-galleries/\"><strong>Direct Image URLs For Galleries</strong></a></p>\n<p>Modifies thumbnails to link to fullsize image instead of the page containing the medium thumbnail.</p>\n<p><a href=\"http://wordpress.org/extend/plugins/statrix/\"><strong>Statrix</strong></a></p>\n<p>Statrix plugin collects data about your blog visits. It creates charts and lists of the full data history in real-time, which you can browse, filter and export immediately after plugin activation. The plugin keeps the full visits history, so you can make reports about any period of time you want.</p>\n<p><a href=\"http://www.basilv.com/psd/software#VersionInformation\"><strong>Version Information</strong></a></p>\n<p>This plugin displays version and configuration information for WordPress, PHP, MySQL, and the web server. A summary is displayed in the Site Admin dashboard and more details are displayed under a Version Information page under the Site Admin Plugins menu.</p>\n<p><a href=\"http://www.theblog.ca/wordpress-collaboration-emails\"><strong>Peter&#8217;s Collaboration Emails</strong></a></p>\n<p>Enhance the &#8220;Submit for Review&#8221; feature for Contributor users in WordPress 2.3 and above. This plugin enables automatic e-mails to the relevant users when posts are pending and when they are approved.</p>\n<p><a href=\"http://www.theblog.ca/anti-spam\"><strong>Peter&#8217;s Custom Anti-Spam Image</strong></a></p>\n<p>This differs from the other image plugins in that it creates an anti-spam jpg using random words of your choice (adding a fun element) and it does not require visitors to have cookies or JavaScript enabled. It also generates audio files for visually impaired visitors.</p>\n<p><a href=\"http://scribu.net/projects/extra-feed-links.html\"><strong>Extra Feed Links</strong></a></p>\n<p>Adds links to the specific feed for each page type in Wordpress. Ex: Comments Feed for single posts and pages; Category Feed for category pages etc.</p>\";}i:6;a:7:{s:5:\"title\";s:61:\"Weblog Tools Collection: WordPress.com announces 24/7 support\";s:4:\"guid\";s:88:\"http://weblogtoolscollection.com/archives/2008/08/03/wordpresscom-announces-247-support/\";s:4:\"link\";s:88:\"http://weblogtoolscollection.com/archives/2008/08/03/wordpresscom-announces-247-support/\";s:11:\"description\";s:1111:\"<p>WordPress.com has <a href=\"http://en.blog.wordpress.com/2008/08/02/announcing-24-7-support/\" target=\"_blank\">announced 24/7 support</a> where people from across the world can ask their support questions without having to worry about time differences. The WordPress.com engineers will cater to all the questions sent using the <a href=\"http://wordpress.com/contact-support/\" target=\"_blank\">contact form</a> and reply within 24 hours or lesser.</p>\n<blockquote><p>Not long ago, our team was online 9 a.m to 4 p.m. PDT to answer your questions and keep your blog running smoothly. Soon after, we brought it up to 24-hour support, five days and week, so those across the country and across the globe wouldn’t have to wait for their answers. </p>\n<p>The time has come for help without delay. We hereby announce <a href=\"http://wordpress.com/features/support/\" target=\"_blank\">24-7 support on WordPress.com</a>.</p>\n</blockquote>\n<p>Remember to use the <a href=\"http://wordpress.com/contact-support/\" target=\"_blank\">contact form</a> to send your questions to the WordPress.com team, to get a quicker reply.</p>\";s:7:\"pubdate\";s:31:\"Sun, 03 Aug 2008 17:16:25 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:1111:\"<p>WordPress.com has <a href=\"http://en.blog.wordpress.com/2008/08/02/announcing-24-7-support/\" target=\"_blank\">announced 24/7 support</a> where people from across the world can ask their support questions without having to worry about time differences. The WordPress.com engineers will cater to all the questions sent using the <a href=\"http://wordpress.com/contact-support/\" target=\"_blank\">contact form</a> and reply within 24 hours or lesser.</p>\n<blockquote><p>Not long ago, our team was online 9 a.m to 4 p.m. PDT to answer your questions and keep your blog running smoothly. Soon after, we brought it up to 24-hour support, five days and week, so those across the country and across the globe wouldn’t have to wait for their answers. </p>\n<p>The time has come for help without delay. We hereby announce <a href=\"http://wordpress.com/features/support/\" target=\"_blank\">24-7 support on WordPress.com</a>.</p>\n</blockquote>\n<p>Remember to use the <a href=\"http://wordpress.com/contact-support/\" target=\"_blank\">contact form</a> to send your questions to the WordPress.com team, to get a quicker reply.</p>\";}i:7;a:7:{s:5:\"title\";s:26:\"Matt: Theme Video Tutorial\";s:4:\"guid\";s:42:\"http://ma.tt/2008/08/theme-video-tutorial/\";s:4:\"link\";s:42:\"http://ma.tt/2008/08/theme-video-tutorial/\";s:11:\"description\";s:192:\"<p><a href=\"http://css-tricks.com/designing-for-wordpress-complete-series-downloads/\">A series of videos, Designing for WordPress, that walks you through all steps of creating a theme</a>.</p>\";s:7:\"pubdate\";s:31:\"Sun, 03 Aug 2008 01:35:19 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:192:\"<p><a href=\"http://css-tricks.com/designing-for-wordpress-complete-series-downloads/\">A series of videos, Designing for WordPress, that walks you through all steps of creating a theme</a>.</p>\";}i:8;a:7:{s:5:\"title\";s:61:\"Weblog Tools Collection: WordPress Facebook Community / Group\";s:4:\"guid\";s:88:\"http://weblogtoolscollection.com/archives/2008/08/02/wordpress-facebook-community-group/\";s:4:\"link\";s:88:\"http://weblogtoolscollection.com/archives/2008/08/02/wordpress-facebook-community-group/\";s:11:\"description\";s:220:\"<p>If you are a Facebook user, you may want to <a href=\"http://www.facebook.com/pages/WordPress/6427302910\" target=\"_blank\">add this group</a> to your favorites to get good inputs from Facebook users about WordPress.</p>\";s:7:\"pubdate\";s:31:\"Sat, 02 Aug 2008 20:11:56 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:220:\"<p>If you are a Facebook user, you may want to <a href=\"http://www.facebook.com/pages/WordPress/6427302910\" target=\"_blank\">add this group</a> to your favorites to get good inputs from Facebook users about WordPress.</p>\";}i:9;a:7:{s:5:\"title\";s:59:\"Weblog Tools Collection: WordPress Theme Releases for 08/02\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3846\";s:4:\"link\";s:87:\"http://weblogtoolscollection.com/archives/2008/08/02/wordpress-theme-releases-for-0802/\";s:11:\"description\";s:1792:\"<p><a href=\"http://www.ofimdavarzea.com/wp/free-wordpress-theme-not-so-serious/\"><strong>[Not] So Serious</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/08/not-so-serious1.jpg\"><img alt=\"not_so_serious1\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/not-so-serious1-thumb.jpg\" width=\"204\" border=\"0\" height=\"142\" /></a></p>\n<p>Three column, fixed widget, gravatar and adsense ready theme</p>\n<p><a href=\"http://www.teckitech.com/08/wordpress-theme-web-20-pinky/\"><strong>Web 2.0 Pinky</strong></a></p>\n<p><a href=\"http://ourfather.co.cc/index.php?wptheme=Web+2.0+Pinky\"><img alt=\"Web2Pinky\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/web2pinky.jpg\" width=\"191\" border=\"0\" height=\"204\" /></a></p>\n<p>Has a great use of colors and neatly centered in the browser. Gorgeous web 2.0 looks and can edit the top sidebar if you know some coding. It support widget and can easily modify colors and other thing if you have some basic knowledge in HTML and CSS.</p>\n<p><a href=\"http://www.magznetwork.com/indomagz-premium-wordpress-theme\"><strong>Indomagz</strong></a></p>\n<p><a href=\"http://www.demo.magznetwork.com/\"><img alt=\"indomagz-2-preview\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/indomagz-2-preview.jpg\" width=\"204\" border=\"0\" height=\"176\" /></a></p>\n<p>Simple, two column, highly customizable theme. <em>Credit Removal requires $20</em>.</p>\n<p><a href=\"http://www.simplywp.net/2008/08/01/its-friday-its-theme-day/\"><strong>Simplest Blue Theme</strong></a></p>\n<p><a href=\"http://simplywp.net/demo/index.php?wptheme=simplest_blue\"><img alt=\"simplest_blue\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/simplest-blue.png\" width=\"204\" border=\"0\" height=\"130\" /></a></p>\n<p>3 columns theme and widget ready, blue and white theme</p>\";s:7:\"pubdate\";s:31:\"Sat, 02 Aug 2008 19:36:21 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:1792:\"<p><a href=\"http://www.ofimdavarzea.com/wp/free-wordpress-theme-not-so-serious/\"><strong>[Not] So Serious</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/08/not-so-serious1.jpg\"><img alt=\"not_so_serious1\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/not-so-serious1-thumb.jpg\" width=\"204\" border=\"0\" height=\"142\" /></a></p>\n<p>Three column, fixed widget, gravatar and adsense ready theme</p>\n<p><a href=\"http://www.teckitech.com/08/wordpress-theme-web-20-pinky/\"><strong>Web 2.0 Pinky</strong></a></p>\n<p><a href=\"http://ourfather.co.cc/index.php?wptheme=Web+2.0+Pinky\"><img alt=\"Web2Pinky\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/web2pinky.jpg\" width=\"191\" border=\"0\" height=\"204\" /></a></p>\n<p>Has a great use of colors and neatly centered in the browser. Gorgeous web 2.0 looks and can edit the top sidebar if you know some coding. It support widget and can easily modify colors and other thing if you have some basic knowledge in HTML and CSS.</p>\n<p><a href=\"http://www.magznetwork.com/indomagz-premium-wordpress-theme\"><strong>Indomagz</strong></a></p>\n<p><a href=\"http://www.demo.magznetwork.com/\"><img alt=\"indomagz-2-preview\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/indomagz-2-preview.jpg\" width=\"204\" border=\"0\" height=\"176\" /></a></p>\n<p>Simple, two column, highly customizable theme. <em>Credit Removal requires $20</em>.</p>\n<p><a href=\"http://www.simplywp.net/2008/08/01/its-friday-its-theme-day/\"><strong>Simplest Blue Theme</strong></a></p>\n<p><a href=\"http://simplywp.net/demo/index.php?wptheme=simplest_blue\"><img alt=\"simplest_blue\" src=\"http://weblogtoolscollection.com/b2-img/2008/08/simplest-blue.png\" width=\"204\" border=\"0\" height=\"130\" /></a></p>\n<p>3 columns theme and widget ready, blue and white theme</p>\";}i:10;a:7:{s:5:\"title\";s:21:\"Matt: Free Jon Miller\";s:4:\"guid\";s:37:\"http://ma.tt/2008/08/free-jon-miller/\";s:4:\"link\";s:37:\"http://ma.tt/2008/08/free-jon-miller/\";s:11:\"description\";s:172:\"<p><a href=\"http://kara.allthingsd.com/20080801/boomtown-plea-to-jeff-bewkes-free-jon-miller/\">BoomTown Plea to Jeff Bewkes: Free Jon Miller!</a> &#8212; Kara Swisher.</p>\";s:7:\"pubdate\";s:31:\"Sat, 02 Aug 2008 17:44:06 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:172:\"<p><a href=\"http://kara.allthingsd.com/20080801/boomtown-plea-to-jeff-bewkes-free-jon-miller/\">BoomTown Plea to Jeff Bewkes: Free Jon Miller!</a> &#8212; Kara Swisher.</p>\";}i:11;a:7:{s:5:\"title\";s:29:\"Matt: Free Software Usability\";s:4:\"guid\";s:45:\"http://ma.tt/2008/08/free-software-usability/\";s:4:\"link\";s:45:\"http://ma.tt/2008/08/free-software-usability/\";s:11:\"description\";s:266:\"<p><a href=\"http://mpt.net.nz/archive/2008/08/01/free-software-usability\">Why Free Software is hard to use, and how to improve it</a>, by Matthew Paul Thomas. (The <a href=\"http://iammattthomas.com/\">other</a> Matt Thomas who used to be involved with WordPress.)</p>\";s:7:\"pubdate\";s:31:\"Sat, 02 Aug 2008 00:02:41 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:266:\"<p><a href=\"http://mpt.net.nz/archive/2008/08/01/free-software-usability\">Why Free Software is hard to use, and how to improve it</a>, by Matthew Paul Thomas. (The <a href=\"http://iammattthomas.com/\">other</a> Matt Thomas who used to be involved with WordPress.)</p>\";}i:12;a:7:{s:5:\"title\";s:30:\"Matt: Interference Robustness?\";s:4:\"guid\";s:45:\"http://ma.tt/2008/08/interference-robustness/\";s:4:\"link\";s:45:\"http://ma.tt/2008/08/interference-robustness/\";s:11:\"description\";s:165:\"<p><a href=\"http://www.macinstruct.com/node/213\">What is Interference Robustness?</a> In case you&#8217;ve ever wondered what that checkbox under Mac wifi means.</p>\";s:7:\"pubdate\";s:31:\"Fri, 01 Aug 2008 22:19:08 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:165:\"<p><a href=\"http://www.macinstruct.com/node/213\">What is Interference Robustness?</a> In case you&#8217;ve ever wondered what that checkbox under Mac wifi means.</p>\";}i:13;a:7:{s:5:\"title\";s:60:\"Weblog Tools Collection: WordPress Plugin Releases for 08/01\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3839\";s:4:\"link\";s:88:\"http://weblogtoolscollection.com/archives/2008/08/01/wordpress-plugin-releases-for-0801/\";s:11:\"description\";s:2085:\"<p><a href=\"http://striderweb.com/nerdaphernalia/features/wp-log-deprecated-calls/\"><strong>Log Deprecated Calls</strong></a></p>\n<p>The plugin on this page, when activated, puts an entry in your PHP log any time a deprecated function is called, or a deprecated file is included. The entry is far more useful than the default WordPress log notice - this one tells you the name of the function, the exact file that called it, the line the call was on, and what new function, if any, you should use instead.</p>\n<p><a href=\"http://www.sattes-faction.de/picasaview/\"><strong>PicasaView</strong></a></p>\n<p>With picasaView you can easily view your picasaweb-albums in your WordPress blog.</p>\n<p><a href=\"http://wp.gdragon.info/plugins/gd-plugin-core/\"><strong>GD Plugin Core</strong></a></p>\n<p>This is not actually plugin, but base class you can use to extend and create your own plugin. Along with the base class, you have a Wizard for creating plugins using base class.</p>\n<p><a href=\"http://www.mkyong.com/blog/digg-digg-wordpress-plugin/\"><strong>Digg Digg</strong></a></p>\n<p>Add Digg Button Into Wordpress Content with four prefix template</p>\n<p><a href=\"http://wpimagerotator.com/\"><strong>Header Image Rotator</strong></a></p>\n<p>Displays a new header image in your blog based on different time variables.</p>\n<p><a href=\"http://imidea.com/nofollow-links-page-wordpress-plugin/\"><strong>Nofollow Links Page</strong></a></p>\n<p>A simple plugin to create a special links page or blogroll page with embedded nofollow tag to prevent PageRank leak.</p>\n<p><a href=\"http://blog.avirtualhome.com/wordpress-plugins/avh-amazon/\"><strong>AVH Amazon</strong></a></p>\n<p>The AVH Amazon plugin give you the abillity to add multiple widgets which will display a random item from your Amazon wishlist, baby registry and/or wedding registry.</p>\n<p><a href=\"http://www.wpg2.org/\"><strong>WPG2</strong></a></p>\n<p>WPG2 is a Wordpress Plug-in that embeds Gallery2 within Wordpress to share photos, videos and any other Gallery2 content seamlessly into the Wordpress Sidebar and Blog entries.</p>\";s:7:\"pubdate\";s:31:\"Fri, 01 Aug 2008 18:51:38 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:2085:\"<p><a href=\"http://striderweb.com/nerdaphernalia/features/wp-log-deprecated-calls/\"><strong>Log Deprecated Calls</strong></a></p>\n<p>The plugin on this page, when activated, puts an entry in your PHP log any time a deprecated function is called, or a deprecated file is included. The entry is far more useful than the default WordPress log notice - this one tells you the name of the function, the exact file that called it, the line the call was on, and what new function, if any, you should use instead.</p>\n<p><a href=\"http://www.sattes-faction.de/picasaview/\"><strong>PicasaView</strong></a></p>\n<p>With picasaView you can easily view your picasaweb-albums in your WordPress blog.</p>\n<p><a href=\"http://wp.gdragon.info/plugins/gd-plugin-core/\"><strong>GD Plugin Core</strong></a></p>\n<p>This is not actually plugin, but base class you can use to extend and create your own plugin. Along with the base class, you have a Wizard for creating plugins using base class.</p>\n<p><a href=\"http://www.mkyong.com/blog/digg-digg-wordpress-plugin/\"><strong>Digg Digg</strong></a></p>\n<p>Add Digg Button Into Wordpress Content with four prefix template</p>\n<p><a href=\"http://wpimagerotator.com/\"><strong>Header Image Rotator</strong></a></p>\n<p>Displays a new header image in your blog based on different time variables.</p>\n<p><a href=\"http://imidea.com/nofollow-links-page-wordpress-plugin/\"><strong>Nofollow Links Page</strong></a></p>\n<p>A simple plugin to create a special links page or blogroll page with embedded nofollow tag to prevent PageRank leak.</p>\n<p><a href=\"http://blog.avirtualhome.com/wordpress-plugins/avh-amazon/\"><strong>AVH Amazon</strong></a></p>\n<p>The AVH Amazon plugin give you the abillity to add multiple widgets which will display a random item from your Amazon wishlist, baby registry and/or wedding registry.</p>\n<p><a href=\"http://www.wpg2.org/\"><strong>WPG2</strong></a></p>\n<p>WPG2 is a Wordpress Plug-in that embeds Gallery2 within Wordpress to share photos, videos and any other Gallery2 content seamlessly into the Wordpress Sidebar and Blog entries.</p>\";}i:14;a:7:{s:5:\"title\";s:59:\"Weblog Tools Collection: Features Planned for WordPress 2.7\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3832\";s:4:\"link\";s:80:\"http://weblogtoolscollection.com/archives/2008/07/31/features-plannwordpress-27/\";s:11:\"description\";s:5191:\"<p>It has been only a couple of weeks since WordPress 2.6 has released and there is already talk about a much more feature rich version in the making. WordPress 2.7 has several exciting features <em>planned/proposed</em> as discussed in informal meetings on IRC and the feature votes being posted on the Codex. Though there are very few hints hint of a release date, I expect it to be released by the end of 2008.</p>\n<p><strong>Here is a list of features that will be introduced into WordPress 2.7:</strong></p>\n<ul>\n<li><strong>Comments API</strong> – This according to me is the most exciting feature, it will allow developers to create offline integration of comments management, where users could use desktop clients to manage comments (moderate, edit, reply, delete, spam, unspam).</li>\n<li><strong>Keyboard shortcuts for comment moderation</strong> – This feature will help users to quickly moderate comments using keyboard shortucts, so you can expect key combinations like Ctrl + S to spam a comment, Ctrl + A to approve and so on.</li>\n<li><strong>Theme Update API</strong> – WordPress 2.5 and above have really made plugin management quite easy, you automatically receive notices when a new update to a plugin is available and can upgrade it without having to manually upload it to your FTP server. With this addition WordPress aims to bridge the gap between theme developers and the users for publishing updates to themes.</li>\n<li><strong>One Click Plugin Installs</strong> – Installing a plugin will be much easier with One click plugin installs. A plugin with similar features called <a href=\"http://anirudhsanjeev.org/oneclick-plugin/\" target=\"_blank\">One Click Plugin</a> was the winner of the Plugin Competition last year.</li>\n<li><strong>WordPress core updates</strong> – This is a feature many users have been waiting for, with core updates you can easily upgrade your WordPress versions. The <a href=\"http://techie-buzz.com/wordpress-plugins/wordpress-automatic-upgrade-12-release.html\" target=\"_blank\">WordPress Automatic Upgrade</a> plugin performs the same task.</li>\n<li><strong>Default Sitemaps</strong> – WordPress 2.7 will include a default sitemap creator, which will create Google XML sitemaps for your blog. <a href=\"http://www.arnebrachhold.de/projects/wordpress-plugins/google-xml-sitemaps-generator/\" target=\"_blank\">Google Sitemaps Generator</a> plugin is the best option available right now for creating XML sitemaps.</li>\n<li><strong>Admin Panel Comment Replies</strong> - This plugin will help administrators to reply to comments using the admin panel. There are several plugins which allow you to do so right now, including <a href=\"http://www.raproject.com/ajax-edit-comments/\">WP AJax Edit Comments</a>, <a href=\"http://techie-buzz.com/wordpress-plugins/better-comments-manager-just-got-better.html\">Better Comments Manager</a> and <a href=\"http://planetozh.com/blog/my-projects/absolute-comments-manager-instant-reply/\">Absolute Comments</a>.</li>\n<li><strong>Comment Threading</strong> – This feature will allow users to reply to other commentators in a threaded manner, where replied comments will display below the actual comment, WordPress already has the ability to scale to accomodate threaded comments, without requiring database changes. <a href=\"http://meidell.dk/archives/2004/09/04/nested-comments/\" target=\"_blank\">Brian’s Threaded Comments</a> is a does the job of allowing users to reply to comments.</li>\n<li><strong>Subscribe to Comments</strong> – The WordPress team is also planning to include a option to allow commentators to subscribe to follow up comments, but this is marked as a question, so it may not actully make it into WordPress 2.7. The <a href=\"http://txfx.net/code/wordpress/subscribe-to-comments/\" target=\"_blank\">Subscribe to Comments</a> plugin does the job to notify commentators on follow up comments.</li>\n<li><strong>Widgets for Dashboard and Write Box</strong> – This feature will allow users to re-arrange the widgets on the dashboard and write page to suit their preferences.</li>\n<li><strong>Batch Editing of Posts</strong> – This feature will allow users to batch edit their posts. Though there is not much information available about this right now, and I could not really find more from my research on the topic.</li>\n</ul>\n<p>With WordPress 2.7, we may see many popular plugins retire because of direct integration into WordPress core but we should have better functionality and easier management of daily blogging tasks from within the vanilla WordPress install.</p>\n<p>There are several other interesting features, you can take a look at them by visiting <a href=\"http://codex.wordpress.org/Version_2.7\" target=\"_blank\">WordPress codex</a>. Thanks to <a href=\"http://www.quickonlinetips.com/archives/2008/07/wordpress-27/\" target=\"_blank\">QuickOnlineTips</a> for drawing my attention to the new features.</p>\n<p>What features are you looking the WordPress team to integrate? Do let me know. It would be great to hear what your expectations are.</p>\n<p><strong>Dislaimer:</strong> WordPress Automatic Upgrade Plugin and Better Comments Manager plugin has been written by me.</p>\";s:7:\"pubdate\";s:31:\"Fri, 01 Aug 2008 15:00:06 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:5191:\"<p>It has been only a couple of weeks since WordPress 2.6 has released and there is already talk about a much more feature rich version in the making. WordPress 2.7 has several exciting features <em>planned/proposed</em> as discussed in informal meetings on IRC and the feature votes being posted on the Codex. Though there are very few hints hint of a release date, I expect it to be released by the end of 2008.</p>\n<p><strong>Here is a list of features that will be introduced into WordPress 2.7:</strong></p>\n<ul>\n<li><strong>Comments API</strong> – This according to me is the most exciting feature, it will allow developers to create offline integration of comments management, where users could use desktop clients to manage comments (moderate, edit, reply, delete, spam, unspam).</li>\n<li><strong>Keyboard shortcuts for comment moderation</strong> – This feature will help users to quickly moderate comments using keyboard shortucts, so you can expect key combinations like Ctrl + S to spam a comment, Ctrl + A to approve and so on.</li>\n<li><strong>Theme Update API</strong> – WordPress 2.5 and above have really made plugin management quite easy, you automatically receive notices when a new update to a plugin is available and can upgrade it without having to manually upload it to your FTP server. With this addition WordPress aims to bridge the gap between theme developers and the users for publishing updates to themes.</li>\n<li><strong>One Click Plugin Installs</strong> – Installing a plugin will be much easier with One click plugin installs. A plugin with similar features called <a href=\"http://anirudhsanjeev.org/oneclick-plugin/\" target=\"_blank\">One Click Plugin</a> was the winner of the Plugin Competition last year.</li>\n<li><strong>WordPress core updates</strong> – This is a feature many users have been waiting for, with core updates you can easily upgrade your WordPress versions. The <a href=\"http://techie-buzz.com/wordpress-plugins/wordpress-automatic-upgrade-12-release.html\" target=\"_blank\">WordPress Automatic Upgrade</a> plugin performs the same task.</li>\n<li><strong>Default Sitemaps</strong> – WordPress 2.7 will include a default sitemap creator, which will create Google XML sitemaps for your blog. <a href=\"http://www.arnebrachhold.de/projects/wordpress-plugins/google-xml-sitemaps-generator/\" target=\"_blank\">Google Sitemaps Generator</a> plugin is the best option available right now for creating XML sitemaps.</li>\n<li><strong>Admin Panel Comment Replies</strong> - This plugin will help administrators to reply to comments using the admin panel. There are several plugins which allow you to do so right now, including <a href=\"http://www.raproject.com/ajax-edit-comments/\">WP AJax Edit Comments</a>, <a href=\"http://techie-buzz.com/wordpress-plugins/better-comments-manager-just-got-better.html\">Better Comments Manager</a> and <a href=\"http://planetozh.com/blog/my-projects/absolute-comments-manager-instant-reply/\">Absolute Comments</a>.</li>\n<li><strong>Comment Threading</strong> – This feature will allow users to reply to other commentators in a threaded manner, where replied comments will display below the actual comment, WordPress already has the ability to scale to accomodate threaded comments, without requiring database changes. <a href=\"http://meidell.dk/archives/2004/09/04/nested-comments/\" target=\"_blank\">Brian’s Threaded Comments</a> is a does the job of allowing users to reply to comments.</li>\n<li><strong>Subscribe to Comments</strong> – The WordPress team is also planning to include a option to allow commentators to subscribe to follow up comments, but this is marked as a question, so it may not actully make it into WordPress 2.7. The <a href=\"http://txfx.net/code/wordpress/subscribe-to-comments/\" target=\"_blank\">Subscribe to Comments</a> plugin does the job to notify commentators on follow up comments.</li>\n<li><strong>Widgets for Dashboard and Write Box</strong> – This feature will allow users to re-arrange the widgets on the dashboard and write page to suit their preferences.</li>\n<li><strong>Batch Editing of Posts</strong> – This feature will allow users to batch edit their posts. Though there is not much information available about this right now, and I could not really find more from my research on the topic.</li>\n</ul>\n<p>With WordPress 2.7, we may see many popular plugins retire because of direct integration into WordPress core but we should have better functionality and easier management of daily blogging tasks from within the vanilla WordPress install.</p>\n<p>There are several other interesting features, you can take a look at them by visiting <a href=\"http://codex.wordpress.org/Version_2.7\" target=\"_blank\">WordPress codex</a>. Thanks to <a href=\"http://www.quickonlinetips.com/archives/2008/07/wordpress-27/\" target=\"_blank\">QuickOnlineTips</a> for drawing my attention to the new features.</p>\n<p>What features are you looking the WordPress team to integrate? Do let me know. It would be great to hear what your expectations are.</p>\n<p><strong>Dislaimer:</strong> WordPress Automatic Upgrade Plugin and Better Comments Manager plugin has been written by me.</p>\";}i:15;a:7:{s:5:\"title\";s:33:\"Andrew: PodPress in WordPress 2.6\";s:4:\"guid\";s:33:\"http://azaozz.wordpress.com/?p=43\";s:4:\"link\";s:64:\"http://azaozz.wordpress.com/2008/07/31/podpress-in-wordpress-26/\";s:11:\"description\";s:2478:\"<div class=\"snap_preview\"><br /><p>Yesterday had a quick look at <a href=\"http://wordpress.org/extend/plugins/podpress/\">podPress,</a> the premier plugin for <a href=\"http://en.wikipedia.org/wiki/Podcasting\">podcasting.</a> It has a small incompatibility with the new Post Revisions feature introduced in WordPress 2.6.</p>\n<p>PodPress seems to be in the middle of a large update. I&#8217;ve used the <a href=\"http://downloads.wordpress.org/plugin/podpress.experimental.zip\">&#8220;experimental&#8221;</a> version (8.9) of the plugin and emailed the proposed fix to the authors, however for the more impatient here is the patch:</p>\n<pre name=\"code\" class=\"php\">\n\nfunction post_edit($post_id) {\n	GLOBAL $post;\n	if($this-&gt;justposted) {\n		return;\n	}\n	$this-&gt;justposted = true;\n\n	if ( isset($_POST[&#039;post_ID&#039;]) &amp;&amp; (int) $_POST[&#039;post_ID&#039;] )\n		$post_id = (int) $_POST[&#039;post_ID&#039;];\n\n	$this-&gt;settings_item_save($post_id, $_POST);\n}\n</pre>\n<p>It goes in <code>podpress/podpress_admin_class.php,</code> around line 51. What it does is to make sure the custom post meta fields used by podPress are attached to the actual Post, not one of the revisions.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/azaozz.wordpress.com/43/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/azaozz.wordpress.com/43/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/azaozz.wordpress.com/43/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/azaozz.wordpress.com/43/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/azaozz.wordpress.com/43/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/azaozz.wordpress.com/43/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/azaozz.wordpress.com/43/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=azaozz.wordpress.com&blog=3380945&post=43&subd=azaozz&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 19:06:26 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Andrew Ozz\";}s:7:\"summary\";s:2478:\"<div class=\"snap_preview\"><br /><p>Yesterday had a quick look at <a href=\"http://wordpress.org/extend/plugins/podpress/\">podPress,</a> the premier plugin for <a href=\"http://en.wikipedia.org/wiki/Podcasting\">podcasting.</a> It has a small incompatibility with the new Post Revisions feature introduced in WordPress 2.6.</p>\n<p>PodPress seems to be in the middle of a large update. I&#8217;ve used the <a href=\"http://downloads.wordpress.org/plugin/podpress.experimental.zip\">&#8220;experimental&#8221;</a> version (8.9) of the plugin and emailed the proposed fix to the authors, however for the more impatient here is the patch:</p>\n<pre name=\"code\" class=\"php\">\n\nfunction post_edit($post_id) {\n	GLOBAL $post;\n	if($this-&gt;justposted) {\n		return;\n	}\n	$this-&gt;justposted = true;\n\n	if ( isset($_POST[&#039;post_ID&#039;]) &amp;&amp; (int) $_POST[&#039;post_ID&#039;] )\n		$post_id = (int) $_POST[&#039;post_ID&#039;];\n\n	$this-&gt;settings_item_save($post_id, $_POST);\n}\n</pre>\n<p>It goes in <code>podpress/podpress_admin_class.php,</code> around line 51. What it does is to make sure the custom post meta fields used by podPress are attached to the actual Post, not one of the revisions.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/azaozz.wordpress.com/43/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/azaozz.wordpress.com/43/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/azaozz.wordpress.com/43/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/azaozz.wordpress.com/43/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/azaozz.wordpress.com/43/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/azaozz.wordpress.com/43/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/azaozz.wordpress.com/43/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/azaozz.wordpress.com/43/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=azaozz.wordpress.com&blog=3380945&post=43&subd=azaozz&ref=&feed=1\" /></div>\";}i:16;a:7:{s:5:\"title\";s:59:\"Weblog Tools Collection: WordPress Theme Releases for 07/31\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3828\";s:4:\"link\";s:87:\"http://weblogtoolscollection.com/archives/2008/07/31/wordpress-theme-releases-for-0731/\";s:11:\"description\";s:3673:\"<p><a href=\"http://www.quakemedia.com/wordpress-themes/tremor\"><strong>Tremor</strong></a></p>\n<p><img width=\"189\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/tremorss1.png\" alt=\"tremorss\" height=\"125\" /></p>\n<p>This theme is a 3 column widget ready theme. It is XHTML Strict and a CSS 2.1 compliant and uses the author’s comment highlight hack.</p>\n<p><a href=\"http://www.giselejaquenod.com.ar/blog/2008/07/31/new-free-wordpress-theme-birdies-blog/\"><strong>Birdie&#8217;s Blog</strong></a></p>\n<p><img width=\"125\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/birdie1.jpg\" alt=\"birdie\" height=\"125\" /></p>\n<p>Two column, brown WordPress theme with a birdie calling out!</p>\n<p><a href=\"http://design.crushlabs.com/home/news/1-company-news/127-crush-magazine-wordpress-theme\"><strong>Crush</strong></a></p>\n<p><img width=\"134\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/crush1.png\" alt=\"Crush\" height=\"125\" /></p>\n<p>Crush is a simple three column theme with a single column sidebar and two columns for posts.</p>\n<p><a href=\"http://www.supersatellite.com/projects/grayplicity/\"><strong>Grayplicity</strong></a></p>\n<p><img width=\"188\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/grayplicity1.png\" alt=\"grayplicity\" height=\"125\" /></p>\n<p>Grayplicity is the public release version of the theme being used for SuperSatellite.com. The default setup is a one column, full width (split ~70%/30% content/sidebar) layout. Has support for several plugins as listed on the theme page.</p>\n<p><a href=\"http://easternwest.com/hellowest-wordpress-theme/\"><strong>HelloWest</strong></a></p>\n<p><img width=\"200\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/hellowest1.png\" alt=\"hellowest\" height=\"103\" /></p>\n<p>A dark, simple two column theme based on <a href=\"http://www.hellowiki.com\">HelloWiki’s</a> latest theme release.</p>\n<p><a href=\"http://www.dailyblogtips.com/bluesensation-wordpress-theme-released/\"><strong>BlueSensation</strong></a></p>\n<p><img width=\"163\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/bluesensation1.png\" alt=\"bluesensation\" height=\"125\" /></p>\n<p>Clean and minimalist theme with 2 columns, and ready to display 125&#215;125 ad blocks</p>\n<p><a href=\"http://www.longren.org/wordpress/unwakeable/\"><strong>Unwakeable</strong></a></p>\n<p><img width=\"200\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/unwakeable1.jpg\" alt=\"unwakeable\" height=\"118\" /></p>\n<p>A two column theme based on of <a href=\"http://www.getk2.com/\">K2</a> and <a href=\"http://openswitch.org/unsleepable\">Unsleepable</a>.</p>\n<p><a href=\"http://wordpress.org/extend/themes/1024px\"><strong>1024px</strong></a></p>\n<p>A light and extremely simple theme with 2 columns. It is based on Andreas&#8217; <a href=\"http://andreasviklund.com/templates/1024px/\">1024px XHTML/CSS template</a>. Updated to support tags, category descriptions, gravatars and more. Validates as XHTML 1.0 Strict. Public domain.</p>\n<p><a href=\"http://www.blogchemistry.com/2008/07/28/free-wordpress-theme-alibi/\"><strong>Alibi</strong></a></p>\n<p><img width=\"191\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/wordpress-theme-alibi1.png\" alt=\"wordpress-theme-alibi\" height=\"125\" /></p>\n<p>Alibi is a 2 column fixed-width, free WordPress theme, with righthand sidebar, enabled for widgets.</p>\n<p><a href=\"http://www.simplywp.net/2008/07/25/releasing-first-free-wp-theme-starlite-diva/\"><strong>Starlite Diva</strong></a></p>\n<p><img width=\"200\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/starlite-diva1.jpg\" alt=\"starlite_diva\" height=\"108\" /></p>\n<p>3 column, with pinkie violet colors, widget-ready sidebar. Tested on WordPress 2.5.</p>\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 17:35:50 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:3673:\"<p><a href=\"http://www.quakemedia.com/wordpress-themes/tremor\"><strong>Tremor</strong></a></p>\n<p><img width=\"189\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/tremorss1.png\" alt=\"tremorss\" height=\"125\" /></p>\n<p>This theme is a 3 column widget ready theme. It is XHTML Strict and a CSS 2.1 compliant and uses the author’s comment highlight hack.</p>\n<p><a href=\"http://www.giselejaquenod.com.ar/blog/2008/07/31/new-free-wordpress-theme-birdies-blog/\"><strong>Birdie&#8217;s Blog</strong></a></p>\n<p><img width=\"125\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/birdie1.jpg\" alt=\"birdie\" height=\"125\" /></p>\n<p>Two column, brown WordPress theme with a birdie calling out!</p>\n<p><a href=\"http://design.crushlabs.com/home/news/1-company-news/127-crush-magazine-wordpress-theme\"><strong>Crush</strong></a></p>\n<p><img width=\"134\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/crush1.png\" alt=\"Crush\" height=\"125\" /></p>\n<p>Crush is a simple three column theme with a single column sidebar and two columns for posts.</p>\n<p><a href=\"http://www.supersatellite.com/projects/grayplicity/\"><strong>Grayplicity</strong></a></p>\n<p><img width=\"188\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/grayplicity1.png\" alt=\"grayplicity\" height=\"125\" /></p>\n<p>Grayplicity is the public release version of the theme being used for SuperSatellite.com. The default setup is a one column, full width (split ~70%/30% content/sidebar) layout. Has support for several plugins as listed on the theme page.</p>\n<p><a href=\"http://easternwest.com/hellowest-wordpress-theme/\"><strong>HelloWest</strong></a></p>\n<p><img width=\"200\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/hellowest1.png\" alt=\"hellowest\" height=\"103\" /></p>\n<p>A dark, simple two column theme based on <a href=\"http://www.hellowiki.com\">HelloWiki’s</a> latest theme release.</p>\n<p><a href=\"http://www.dailyblogtips.com/bluesensation-wordpress-theme-released/\"><strong>BlueSensation</strong></a></p>\n<p><img width=\"163\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/bluesensation1.png\" alt=\"bluesensation\" height=\"125\" /></p>\n<p>Clean and minimalist theme with 2 columns, and ready to display 125&#215;125 ad blocks</p>\n<p><a href=\"http://www.longren.org/wordpress/unwakeable/\"><strong>Unwakeable</strong></a></p>\n<p><img width=\"200\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/unwakeable1.jpg\" alt=\"unwakeable\" height=\"118\" /></p>\n<p>A two column theme based on of <a href=\"http://www.getk2.com/\">K2</a> and <a href=\"http://openswitch.org/unsleepable\">Unsleepable</a>.</p>\n<p><a href=\"http://wordpress.org/extend/themes/1024px\"><strong>1024px</strong></a></p>\n<p>A light and extremely simple theme with 2 columns. It is based on Andreas&#8217; <a href=\"http://andreasviklund.com/templates/1024px/\">1024px XHTML/CSS template</a>. Updated to support tags, category descriptions, gravatars and more. Validates as XHTML 1.0 Strict. Public domain.</p>\n<p><a href=\"http://www.blogchemistry.com/2008/07/28/free-wordpress-theme-alibi/\"><strong>Alibi</strong></a></p>\n<p><img width=\"191\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/wordpress-theme-alibi1.png\" alt=\"wordpress-theme-alibi\" height=\"125\" /></p>\n<p>Alibi is a 2 column fixed-width, free WordPress theme, with righthand sidebar, enabled for widgets.</p>\n<p><a href=\"http://www.simplywp.net/2008/07/25/releasing-first-free-wp-theme-starlite-diva/\"><strong>Starlite Diva</strong></a></p>\n<p><img width=\"200\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/starlite-diva1.jpg\" alt=\"starlite_diva\" height=\"108\" /></p>\n<p>3 column, with pinkie violet colors, widget-ready sidebar. Tested on WordPress 2.5.</p>\";}i:17;a:7:{s:5:\"title\";s:41:\"Alex King: 404s and WordPress Server Load\";s:4:\"guid\";s:66:\"http://alexking.org/blog/2008/07/31/404s-and-wordpress-server-load\";s:4:\"link\";s:66:\"http://alexking.org/blog/2008/07/31/404s-and-wordpress-server-load\";s:11:\"description\";s:6422:\"<p>A few months back <a href=\"http://crowdfavorite.com\">we</a> launched a site redesign/redevelopment project for a client, and made a simple mistake that had some interesting ramifications. It&#8217;s worth posting here so others don&#8217;t make the same mistake.</p>\n<h4>What Happened</h4>\n<p>When redeveloping the site, we moved the <a href=\"http://wordpress.org/\" rel=\"external\">WordPress</a> instance from the web root:</p>\n<p>/public_html/wp-config.php<br />\n/public_html/wp-includes/<br />\n/public_html/&#8230;</p>\n<p>to a subdirectory within the web root:</p>\n<p>/public_html/wordpress/wp-config.php<br />\n/public_html/wordpress/wp-includes/<br />\n/public_html/&#8230;</p>\n<p>I consider this to be a best practice for WordPress powered sites, as it makes version upgrades a bit easier (among other things).<sup><a href=\"http://alexking.org/blog/feed?cat=16#fn1217490637389n\" id=\"fn1217490637389\" class=\"footnote\">1</a></sup> Some plugins don&#8217;t work well with symlinked &#8220;plugins&#8221; directories, but those generally have easy fixes (use ABSPATH, not dirname() in your plugin).</p>\n<p>These changes were made and thoroughly tested on our dev server with historical data. We then made the changes in a staging environment under a beta.example.com style domain name before pushing the changes live. In short, we were pretty careful and tested pretty well.</p>\n<h4>The Result</h4>\n<p>When we finally pushed everything live, the production server was brought to its knees. Yikes!</p>\n<p>A little poking around, some bug reports, and very shortly thereafter we figured out the problem. We had failed to set up a rewrite rule to map content from:</p>\n<p>/wp-content/uploads/</p>\n<p>to: </p>\n<p>/wordpress/wp-content/uploads/</p>\n<p>where it was now located.</p>\n<p>An omission like this wouldn&#8217;t have had much effect on server load in some site configurations, but one of the wonderful features of WordPress - its lovely permalinks - has a hidden caveat that can be exposed if someone makes a configuration mistake (like we did).</p>\n<p>WordPress&#8217;s permalink system basically works like this:</p>\n<ol>\n<li>See if a file exists in the location that was requested; if the file exists, serve it. This is how images, media, non-WordPress files, etc. are served without conflicting with WordPress.</li>\n<li>If no file is found at the location that was requested, then pass the URL to WordPress and see if WordPress can figure out what to show.</li>\n</ol>\n<p>It&#8217;s a really elegant system and works very well. However, it also means that 404 requests - http requests that result in a &#8220;file not found&#8221; message - have a much higher impact on the server than a traditional 404 request does. For every 404, the server instantiates WordPress, does some database work to try to see if it can figure out what to serve, etc.</p>\n<p>When we didn&#8217;t set up proper redirects for the content in &#8220;uploads&#8221;, we basically increased the server load by a factor of 20. Instead of a single request going to WordPress and 20 requests serving static content by Apache, all 21 requests were being sent to WordPress.</p>\n<p>The increased load on the server had some&#8230; adverse effects on performance. Yeah.</p>\n<h4>Why We Missed It</h4>\n<p>So this should be a pretty easy thing to catch, right? If the URLs to the images are broken, we&#8217;d all be seeing a bunch of broken images all over the place in our testing - right?</p>\n<p>Not exactly, and for two different reasons:</p>\n<ol>\n<li>When we tested on the production server using the beta.example.com hostname, the content was still pointing to example.com and the live production server was dutifully serving the images. It wasn&#8217;t until we pushed the changes live that the images were no longer at the previous URLs.</li>\n<li>Even when the images were no longer in the proper place, our browsers that we were testing in were showing the images properly. This was due to longer expires headers we had just implemented for media on the server in order to reduce overall server load. In casual testing, everything looked to us to be working properly.</li>\n</ol>\n<p>Once we tested in browsers that didn&#8217;t have the images cached, we immediately saw the problem.</p>\n<h4>Easy Fix</h4>\n<p>Luckily, this is a really easy problem to fix. A simple mod_rewrite rule (placed before the standard WordPress rewrite block) fixed everything right up:</p>\n<div class=\"code\"><code>RewriteRule ^wp-content/uploads/(.*)$ /wordpress/wp-content/uploads/$1 [L,R=301]</code></div>\n<p>(alternate code, improvements to syntax welcome)</p>\n<p>With this in place, the old URLs were redirected to the new location and no longer were spinning up WordPress on every image request.</p>\n<p>Hopefully this will be useful to people who might be doing something similar, or who have made the same mistake and are looking to recover from it. <img src=\"http://alexking.org/wp/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /> </p>\n<ol class=\"footnotes\">\n<li id=\"fn1217490637389n\">Corey has a nice writeup on the <a href=\"http://coreygilmore.com/blog/2008/04/02/wordpress-upgrade-new-design/\">file structure</a> he uses. [<a href=\"http://alexking.org/blog/feed?cat=16#fn1217490637389\">back</a>]</li>\n</ol>\n<p><a href=\"http://sharethis.com/item?&wp=2.2.3&amp;publisher=06654962-d77d-102a-861d-00161729a8a2&amp;title=404s+and+WordPress+Server+Load&amp;url=http%3A%2F%2Falexking.org%2Fblog%2F2008%2F07%2F31%2F404s-and-wordpress-server-load\">ShareThis</a></p>\n	<!--\n	<rdf:RDF xmlns=\"http://web.resource.org/cc/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\n	<Work rdf:about=\"\">\n		<license rdf:resource=\"http://creativecommons.org/licenses/by-nc-sa/2.5/\" />\n	<dc:type rdf:resource=\"http://purl.org/dc/dcmitype/Text\" />\n	</Work>\n	<License rdf:about=\"http://creativecommons.org/licenses/by-nc-sa/2.5/\"><permits rdf:resource=\"http://web.resource.org/cc/Reproduction\"/><permits rdf:resource=\"http://web.resource.org/cc/Distribution\"/><requires rdf:resource=\"http://web.resource.org/cc/Notice\"/><requires rdf:resource=\"http://web.resource.org/cc/Attribution\"/><prohibits rdf:resource=\"http://web.resource.org/cc/CommercialUse\"/><permits rdf:resource=\"http://web.resource.org/cc/DerivativeWorks\"/><requires rdf:resource=\"http://web.resource.org/cc/ShareAlike\"/></License></rdf:RDF>\n	-->\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 08:20:17 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Alex\";}s:7:\"summary\";s:6422:\"<p>A few months back <a href=\"http://crowdfavorite.com\">we</a> launched a site redesign/redevelopment project for a client, and made a simple mistake that had some interesting ramifications. It&#8217;s worth posting here so others don&#8217;t make the same mistake.</p>\n<h4>What Happened</h4>\n<p>When redeveloping the site, we moved the <a href=\"http://wordpress.org/\" rel=\"external\">WordPress</a> instance from the web root:</p>\n<p>/public_html/wp-config.php<br />\n/public_html/wp-includes/<br />\n/public_html/&#8230;</p>\n<p>to a subdirectory within the web root:</p>\n<p>/public_html/wordpress/wp-config.php<br />\n/public_html/wordpress/wp-includes/<br />\n/public_html/&#8230;</p>\n<p>I consider this to be a best practice for WordPress powered sites, as it makes version upgrades a bit easier (among other things).<sup><a href=\"http://alexking.org/blog/feed?cat=16#fn1217490637389n\" id=\"fn1217490637389\" class=\"footnote\">1</a></sup> Some plugins don&#8217;t work well with symlinked &#8220;plugins&#8221; directories, but those generally have easy fixes (use ABSPATH, not dirname() in your plugin).</p>\n<p>These changes were made and thoroughly tested on our dev server with historical data. We then made the changes in a staging environment under a beta.example.com style domain name before pushing the changes live. In short, we were pretty careful and tested pretty well.</p>\n<h4>The Result</h4>\n<p>When we finally pushed everything live, the production server was brought to its knees. Yikes!</p>\n<p>A little poking around, some bug reports, and very shortly thereafter we figured out the problem. We had failed to set up a rewrite rule to map content from:</p>\n<p>/wp-content/uploads/</p>\n<p>to: </p>\n<p>/wordpress/wp-content/uploads/</p>\n<p>where it was now located.</p>\n<p>An omission like this wouldn&#8217;t have had much effect on server load in some site configurations, but one of the wonderful features of WordPress - its lovely permalinks - has a hidden caveat that can be exposed if someone makes a configuration mistake (like we did).</p>\n<p>WordPress&#8217;s permalink system basically works like this:</p>\n<ol>\n<li>See if a file exists in the location that was requested; if the file exists, serve it. This is how images, media, non-WordPress files, etc. are served without conflicting with WordPress.</li>\n<li>If no file is found at the location that was requested, then pass the URL to WordPress and see if WordPress can figure out what to show.</li>\n</ol>\n<p>It&#8217;s a really elegant system and works very well. However, it also means that 404 requests - http requests that result in a &#8220;file not found&#8221; message - have a much higher impact on the server than a traditional 404 request does. For every 404, the server instantiates WordPress, does some database work to try to see if it can figure out what to serve, etc.</p>\n<p>When we didn&#8217;t set up proper redirects for the content in &#8220;uploads&#8221;, we basically increased the server load by a factor of 20. Instead of a single request going to WordPress and 20 requests serving static content by Apache, all 21 requests were being sent to WordPress.</p>\n<p>The increased load on the server had some&#8230; adverse effects on performance. Yeah.</p>\n<h4>Why We Missed It</h4>\n<p>So this should be a pretty easy thing to catch, right? If the URLs to the images are broken, we&#8217;d all be seeing a bunch of broken images all over the place in our testing - right?</p>\n<p>Not exactly, and for two different reasons:</p>\n<ol>\n<li>When we tested on the production server using the beta.example.com hostname, the content was still pointing to example.com and the live production server was dutifully serving the images. It wasn&#8217;t until we pushed the changes live that the images were no longer at the previous URLs.</li>\n<li>Even when the images were no longer in the proper place, our browsers that we were testing in were showing the images properly. This was due to longer expires headers we had just implemented for media on the server in order to reduce overall server load. In casual testing, everything looked to us to be working properly.</li>\n</ol>\n<p>Once we tested in browsers that didn&#8217;t have the images cached, we immediately saw the problem.</p>\n<h4>Easy Fix</h4>\n<p>Luckily, this is a really easy problem to fix. A simple mod_rewrite rule (placed before the standard WordPress rewrite block) fixed everything right up:</p>\n<div class=\"code\"><code>RewriteRule ^wp-content/uploads/(.*)$ /wordpress/wp-content/uploads/$1 [L,R=301]</code></div>\n<p>(alternate code, improvements to syntax welcome)</p>\n<p>With this in place, the old URLs were redirected to the new location and no longer were spinning up WordPress on every image request.</p>\n<p>Hopefully this will be useful to people who might be doing something similar, or who have made the same mistake and are looking to recover from it. <img src=\"http://alexking.org/wp/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /> </p>\n<ol class=\"footnotes\">\n<li id=\"fn1217490637389n\">Corey has a nice writeup on the <a href=\"http://coreygilmore.com/blog/2008/04/02/wordpress-upgrade-new-design/\">file structure</a> he uses. [<a href=\"http://alexking.org/blog/feed?cat=16#fn1217490637389\">back</a>]</li>\n</ol>\n<p><a href=\"http://sharethis.com/item?&wp=2.2.3&amp;publisher=06654962-d77d-102a-861d-00161729a8a2&amp;title=404s+and+WordPress+Server+Load&amp;url=http%3A%2F%2Falexking.org%2Fblog%2F2008%2F07%2F31%2F404s-and-wordpress-server-load\">ShareThis</a></p>\n	<!--\n	<rdf:RDF xmlns=\"http://web.resource.org/cc/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\n	<Work rdf:about=\"\">\n		<license rdf:resource=\"http://creativecommons.org/licenses/by-nc-sa/2.5/\" />\n	<dc:type rdf:resource=\"http://purl.org/dc/dcmitype/Text\" />\n	</Work>\n	<License rdf:about=\"http://creativecommons.org/licenses/by-nc-sa/2.5/\"><permits rdf:resource=\"http://web.resource.org/cc/Reproduction\"/><permits rdf:resource=\"http://web.resource.org/cc/Distribution\"/><requires rdf:resource=\"http://web.resource.org/cc/Notice\"/><requires rdf:resource=\"http://web.resource.org/cc/Attribution\"/><prohibits rdf:resource=\"http://web.resource.org/cc/CommercialUse\"/><permits rdf:resource=\"http://web.resource.org/cc/DerivativeWorks\"/><requires rdf:resource=\"http://web.resource.org/cc/ShareAlike\"/></License></rdf:RDF>\n	-->\";}i:18;a:7:{s:5:\"title\";s:24:\"Matt: Plugin Competition\";s:4:\"guid\";s:40:\"http://ma.tt/2008/07/plugin-competition/\";s:4:\"link\";s:40:\"http://ma.tt/2008/07/plugin-competition/\";s:11:\"description\";s:168:\"<p><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/26/its-time-to-cast-your-vote/\">The WordPress plugin competition at WLTC is now open for voting</a>.</p>\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 04:37:10 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:168:\"<p><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/26/its-time-to-cast-your-vote/\">The WordPress plugin competition at WLTC is now open for voting</a>.</p>\";}i:19;a:7:{s:5:\"title\";s:30:\"Matt: Using WordPress as a CMS\";s:4:\"guid\";s:46:\"http://ma.tt/2008/07/using-wordpress-as-a-cms/\";s:4:\"link\";s:46:\"http://ma.tt/2008/07/using-wordpress-as-a-cms/\";s:11:\"description\";s:198:\"<p><a href=\"http://www.devlounge.net/publishing/things-to-consider-when-using-wordpress-as-a-cms\">Things To Consider When Using WordPress as a CMS</a>. Would love to see more articles like this.</p>\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 04:36:16 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:198:\"<p><a href=\"http://www.devlounge.net/publishing/things-to-consider-when-using-wordpress-as-a-cms\">Things To Consider When Using WordPress as a CMS</a>. Would love to see more articles like this.</p>\";}i:20;a:7:{s:5:\"title\";s:32:\"Akismet: The Guardian on Akismet\";s:4:\"guid\";s:34:\"http://akismet.wordpress.com/?p=95\";s:4:\"link\";s:59:\"http://blog.akismet.com/2008/07/31/the-guardian-on-akismet/\";s:11:\"description\";s:2320:\"<div class=\"snap_preview\"><br /><p><a href=\"http://www.guardian.co.uk/\" target=\"_blank\">The Guardian&#8217;s</a> <a href=\"http://michaelpollitt.com/wordpress/\" target=\"_blank\">Michael Pollitt</a> says some kind words about Akismet in today&#8217;s <a href=\"http://www.guardian.co.uk/technology\" target=\"_blank\">Technology section</a>, as he notes that <a href=\"http://www.guardian.co.uk/technology/2008/jul/31/spam.blogging.technology\" target=\"_blank\">Comment spammers never take a holiday</a>. </p>\n<blockquote><p>Mullenweg publishes a graph on <a href=\"http://akismet.com/\">Akismet.com</a> which shows comments being exceeded by daily spam peaks of more than 25m. Checking out the spam blocked counter on my blog is satisfying, showing that the spammers are going nowhere fast.</p>\n<p>As an experiment, I turned off my spam protection to watch the spam-a-minute flow.</p></blockquote>\n<p><a href=\"http://www.guardian.co.uk/technology/2008/jul/31/spam.blogging.technology\" target=\"_blank\">Read the full article here.</a></p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/akismet.wordpress.com/95/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/akismet.wordpress.com/95/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/akismet.wordpress.com/95/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/akismet.wordpress.com/95/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/akismet.wordpress.com/95/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/akismet.wordpress.com/95/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/akismet.wordpress.com/95/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=blog.akismet.com&blog=116920&post=95&subd=akismet&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 02:59:45 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:2320:\"<div class=\"snap_preview\"><br /><p><a href=\"http://www.guardian.co.uk/\" target=\"_blank\">The Guardian&#8217;s</a> <a href=\"http://michaelpollitt.com/wordpress/\" target=\"_blank\">Michael Pollitt</a> says some kind words about Akismet in today&#8217;s <a href=\"http://www.guardian.co.uk/technology\" target=\"_blank\">Technology section</a>, as he notes that <a href=\"http://www.guardian.co.uk/technology/2008/jul/31/spam.blogging.technology\" target=\"_blank\">Comment spammers never take a holiday</a>. </p>\n<blockquote><p>Mullenweg publishes a graph on <a href=\"http://akismet.com/\">Akismet.com</a> which shows comments being exceeded by daily spam peaks of more than 25m. Checking out the spam blocked counter on my blog is satisfying, showing that the spammers are going nowhere fast.</p>\n<p>As an experiment, I turned off my spam protection to watch the spam-a-minute flow.</p></blockquote>\n<p><a href=\"http://www.guardian.co.uk/technology/2008/jul/31/spam.blogging.technology\" target=\"_blank\">Read the full article here.</a></p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/akismet.wordpress.com/95/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/akismet.wordpress.com/95/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/akismet.wordpress.com/95/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/akismet.wordpress.com/95/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/akismet.wordpress.com/95/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/akismet.wordpress.com/95/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/akismet.wordpress.com/95/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/akismet.wordpress.com/95/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=blog.akismet.com&blog=116920&post=95&subd=akismet&ref=&feed=1\" /></div>\";}i:21;a:7:{s:5:\"title\";s:67:\"Weblog Tools Collection: Firefly’s Chat-On-A-Webpage : Try it out\";s:4:\"guid\";s:91:\"http://weblogtoolscollection.com/archives/2008/07/30/fireflys-chat-on-a-webpage-try-it-out/\";s:4:\"link\";s:91:\"http://weblogtoolscollection.com/archives/2008/07/30/fireflys-chat-on-a-webpage-try-it-out/\";s:11:\"description\";s:866:\"<p><a href=\"http://www.alleyinsider.com/2008/7/firefly-s-chat-on-a-webpage-goes-live\">Firefly&#8217;s Chat-On-A-Webpage Goes Live</a>.: Found via GigaOm, Firefly is a cool little Javascript app that sits on tops of your blog or webpage and allows unfettered chatting and interaction ability for visitors. I have enabled it on WeblogToolsCollection for the time being for people to check it out. Enable at the bottom of the page (look for a small bar), click anywhere on the page and chat away! If you want to put it on your own blog, visit <a href=\"http://firef.ly/\">firef.ly</a> WARNING: It is addictive in an &#8220;Ooo! Shiny!&#8221; sort of way. <img src=\"http://weblogtoolscollection.com/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /> </p>\n<p>[EDIT] Now gone from this blog. Please use the demo on the firef.ly page to test it out.</p>\";s:7:\"pubdate\";s:31:\"Wed, 30 Jul 2008 22:25:20 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:866:\"<p><a href=\"http://www.alleyinsider.com/2008/7/firefly-s-chat-on-a-webpage-goes-live\">Firefly&#8217;s Chat-On-A-Webpage Goes Live</a>.: Found via GigaOm, Firefly is a cool little Javascript app that sits on tops of your blog or webpage and allows unfettered chatting and interaction ability for visitors. I have enabled it on WeblogToolsCollection for the time being for people to check it out. Enable at the bottom of the page (look for a small bar), click anywhere on the page and chat away! If you want to put it on your own blog, visit <a href=\"http://firef.ly/\">firef.ly</a> WARNING: It is addictive in an &#8220;Ooo! Shiny!&#8221; sort of way. <img src=\"http://weblogtoolscollection.com/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /> </p>\n<p>[EDIT] Now gone from this blog. Please use the demo on the firef.ly page to test it out.</p>\";}i:22;a:7:{s:5:\"title\";s:40:\"Mark Jaquith: One-line plugin of the day\";s:4:\"guid\";s:39:\"http://markjaquith.wordpress.com/?p=161\";s:4:\"link\";s:71:\"http://markjaquith.wordpress.com/2008/07/30/one-line-plugin-of-the-day/\";s:11:\"description\";s:1923:\"<div class=\"snap_preview\"><br /><p>Opens the post timestamp editing fields without you having to click the &#8220;Edit&#8221; link every time.  Great for people who are always scheduling their posts.</p>\n<p><code>add_action(\'admin_head\',create_function(\'$a\', \"echo \\\"&lt;script type=\'text/javascript\'&gt;<br />\n\\n&lt;!-<span>-</span>\\naddLoadEvent(function(){jQuery(\'.edit-timestamp\').click();});\\n//-<span>-</span>&gt;\\n<br />\n&lt;/script&gt;\\\";\"),50);</code></p>\n<p>Note: one-line plugin wrapped to three lines to make up for CSS issues in this theme.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/markjaquith.wordpress.com/161/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/markjaquith.wordpress.com/161/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/markjaquith.wordpress.com/161/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/markjaquith.wordpress.com/161/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/markjaquith.wordpress.com/161/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/markjaquith.wordpress.com/161/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/markjaquith.wordpress.com/161/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=markjaquith.wordpress.com&blog=316&post=161&subd=markjaquith&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Wed, 30 Jul 2008 17:47:19 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Mark Jaquith\";}s:7:\"summary\";s:1923:\"<div class=\"snap_preview\"><br /><p>Opens the post timestamp editing fields without you having to click the &#8220;Edit&#8221; link every time.  Great for people who are always scheduling their posts.</p>\n<p><code>add_action(\'admin_head\',create_function(\'$a\', \"echo \\\"&lt;script type=\'text/javascript\'&gt;<br />\n\\n&lt;!-<span>-</span>\\naddLoadEvent(function(){jQuery(\'.edit-timestamp\').click();});\\n//-<span>-</span>&gt;\\n<br />\n&lt;/script&gt;\\\";\"),50);</code></p>\n<p>Note: one-line plugin wrapped to three lines to make up for CSS issues in this theme.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/markjaquith.wordpress.com/161/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/markjaquith.wordpress.com/161/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/markjaquith.wordpress.com/161/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/markjaquith.wordpress.com/161/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/markjaquith.wordpress.com/161/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/markjaquith.wordpress.com/161/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/markjaquith.wordpress.com/161/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/markjaquith.wordpress.com/161/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=markjaquith.wordpress.com&blog=316&post=161&subd=markjaquith&ref=&feed=1\" /></div>\";}i:23;a:7:{s:5:\"title\";s:53:\"Weblog Tools Collection: WordPress As A CMS Checklist\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3801\";s:4:\"link\";s:82:\"http://weblogtoolscollection.com/archives/2008/07/30/wordpress-as-a-cms-checklist/\";s:11:\"description\";s:1829:\"<p><strong>Thord Daniel Hedengren</strong> over at <strong>Devlounge</strong> has published an <a href=\"http://www.devlounge.net/publishing/things-to-consider-when-using-wordpress-as-a-cms\" target=\"_blank\">extensive checklist</a> based on his experience of things to consider when using WordPress as a CMS, especially when it will be used for a client. His post covers a number of different situations you should think about ahead of time before you step into your favorite code editor. According to Thord, there are three things you need to consider before committing to WordPress:</p>\n<blockquote><p><strong>1. Is the functionality needed covered by the WordPress core functions, and/or with the addition of (not too many) plugins?</strong> This is usually managing information pages (using Pages), and publishing news/press releases (using Posts). If I need to add a lot of custom stuff, including the custom fields, then perhaps it gets too complicated for the client.</p>\n<p><strong>2. Is there a good translation of WordPress available, so that your client can get the backend in their own language?</strong> Why should my Swedish customers not have their CMS in Swedish? There is no reason, of course, and it is easy enough to install a language pack.</p>\n<p><strong>3. Will my client be able to upgrade WordPress themselves, or do I need to make plans for this as well?</strong> This is true for most platforms out there, but nevertheless you’ll need to have an upgrade strategy.</p></blockquote>\n<p>I&#8217;ve never dived into the realm of setting up WordPress for a client so this is all new to me. However, I know a lot of you install and configure WordPress for your own clients. With that in mind, I&#8217;d be interested in knowing your own checklist of things to consider before committing them to WordPress.</p>\";s:7:\"pubdate\";s:31:\"Wed, 30 Jul 2008 11:13:11 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"Jeff Chandler\";}s:7:\"summary\";s:1829:\"<p><strong>Thord Daniel Hedengren</strong> over at <strong>Devlounge</strong> has published an <a href=\"http://www.devlounge.net/publishing/things-to-consider-when-using-wordpress-as-a-cms\" target=\"_blank\">extensive checklist</a> based on his experience of things to consider when using WordPress as a CMS, especially when it will be used for a client. His post covers a number of different situations you should think about ahead of time before you step into your favorite code editor. According to Thord, there are three things you need to consider before committing to WordPress:</p>\n<blockquote><p><strong>1. Is the functionality needed covered by the WordPress core functions, and/or with the addition of (not too many) plugins?</strong> This is usually managing information pages (using Pages), and publishing news/press releases (using Posts). If I need to add a lot of custom stuff, including the custom fields, then perhaps it gets too complicated for the client.</p>\n<p><strong>2. Is there a good translation of WordPress available, so that your client can get the backend in their own language?</strong> Why should my Swedish customers not have their CMS in Swedish? There is no reason, of course, and it is easy enough to install a language pack.</p>\n<p><strong>3. Will my client be able to upgrade WordPress themselves, or do I need to make plans for this as well?</strong> This is true for most platforms out there, but nevertheless you’ll need to have an upgrade strategy.</p></blockquote>\n<p>I&#8217;ve never dived into the realm of setting up WordPress for a client so this is all new to me. However, I know a lot of you install and configure WordPress for your own clients. With that in mind, I&#8217;d be interested in knowing your own checklist of things to consider before committing them to WordPress.</p>\";}i:24;a:7:{s:5:\"title\";s:16:\"Matt: ALA Survey\";s:4:\"guid\";s:32:\"http://ma.tt/2008/07/ala-survey/\";s:4:\"link\";s:32:\"http://ma.tt/2008/07/ala-survey/\";s:11:\"description\";s:101:\"<p><a href=\"http://alistapart.com/articles/survey2008\">The new A List Apart for survey is up</a>.</p>\";s:7:\"pubdate\";s:31:\"Wed, 30 Jul 2008 02:43:20 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:101:\"<p><a href=\"http://alistapart.com/articles/survey2008\">The new A List Apart for survey is up</a>.</p>\";}i:25;a:7:{s:5:\"title\";s:60:\"Weblog Tools Collection: WordPress Plugin Releases for 07/29\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3799\";s:4:\"link\";s:88:\"http://weblogtoolscollection.com/archives/2008/07/29/wordpress-plugin-releases-for-0729/\";s:11:\"description\";s:2610:\"<p><a href=\"http://www.viper007bond.com/wordpress-plugins/page-breadcrumbs-for-wptitle/\"><strong>Breadcrumb Titles For Pages</strong></a></p>\n<p>This plugin modifies <code>wp_title()</code> (the function used to create the title of your website to output all parent Pages when viewing a Page.</p>\n<p><a href=\"http://www.viper007bond.com/wordpress-plugins/allow-numeric-stubs/\"><strong>Allow Numeric Stubs</strong></a></p>\n<p>Starting with WordPress 2.5, there is a bug where you cannot have a child Page that’s stub is a number. For example this will not work: <code>/about/5/</code>. That URL conflicts with paged content feature where you can posts and Pages with multiple pages of content by adding <code>&lt;!--nextpage--&gt;</code> within your content.</p>\n<p>This plugin allows you to have children Pages with numbers as stubs by giving up the ability to have paged Pages which isn’t a big deal as most people don’t use paged Pages anyway.</p>\n<p><a href=\"http://www.briandgoad.com/blog/downloads/p2pconverter/\"><strong>p2pConverter</strong></a></p>\n<p>Converts either a static Page into a Post, or a Post into a static Page! Just click on the Manage tab in the Admin interface, and click the Posts or Pages sub-tab.</p>\n<p><a href=\"http://www.briandgoad.com/blog/downloads/status-press-widget/\"><strong>Status Press Widget</strong></a></p>\n<p>Adds a Facebook Status Widget to display your Facebook status in your sidebar. Based on Status Press plugin (not required!) by Adam Walker Cleaveland &amp; C. Scott Andreas.</p>\n<p><a href=\"http://www.briandgoad.com/blog/downloads/customizable-search-widget/\"><strong>Customizable Search Widget</strong></a></p>\n<p>Adds a Customizable Search Widget to give you more control over how the search box looks on your sidebar.</p>\n<p><a href=\"http://lcb.me.uk/losingit/2008/06/29/list-drafts-widget\"><strong>List Drafts Widget</strong></a></p>\n<p>List Drafts is a simple sidebar widget which outputs a list of the titles of all posts currently saved as drafts. Users can set the title of the widget, and add a label to be used for all untitled drafts.</p>\n<p><a href=\"http://www.sillybean.net/code/post-and-page-excerpt-widgets/\"><strong>Post and Page Excerpt Widgets</strong></a></p>\n<p>Creates widgets that display excerpts from posts or pages in the sidebar. You may use ‘more’ links and/or link the widget title to the post or page.</p>\n<p>And, while on the topic of plugins, take a look at Clever Carrot&#8217;s <a href=\"http://www.clevercarrot.net/wordpress/10-wordpress-plugins-to-grow-great-sites/\">10 WordPress Plugins to Grow Great Sites</a>.</p>\";s:7:\"pubdate\";s:31:\"Tue, 29 Jul 2008 21:23:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:2610:\"<p><a href=\"http://www.viper007bond.com/wordpress-plugins/page-breadcrumbs-for-wptitle/\"><strong>Breadcrumb Titles For Pages</strong></a></p>\n<p>This plugin modifies <code>wp_title()</code> (the function used to create the title of your website to output all parent Pages when viewing a Page.</p>\n<p><a href=\"http://www.viper007bond.com/wordpress-plugins/allow-numeric-stubs/\"><strong>Allow Numeric Stubs</strong></a></p>\n<p>Starting with WordPress 2.5, there is a bug where you cannot have a child Page that’s stub is a number. For example this will not work: <code>/about/5/</code>. That URL conflicts with paged content feature where you can posts and Pages with multiple pages of content by adding <code>&lt;!--nextpage--&gt;</code> within your content.</p>\n<p>This plugin allows you to have children Pages with numbers as stubs by giving up the ability to have paged Pages which isn’t a big deal as most people don’t use paged Pages anyway.</p>\n<p><a href=\"http://www.briandgoad.com/blog/downloads/p2pconverter/\"><strong>p2pConverter</strong></a></p>\n<p>Converts either a static Page into a Post, or a Post into a static Page! Just click on the Manage tab in the Admin interface, and click the Posts or Pages sub-tab.</p>\n<p><a href=\"http://www.briandgoad.com/blog/downloads/status-press-widget/\"><strong>Status Press Widget</strong></a></p>\n<p>Adds a Facebook Status Widget to display your Facebook status in your sidebar. Based on Status Press plugin (not required!) by Adam Walker Cleaveland &amp; C. Scott Andreas.</p>\n<p><a href=\"http://www.briandgoad.com/blog/downloads/customizable-search-widget/\"><strong>Customizable Search Widget</strong></a></p>\n<p>Adds a Customizable Search Widget to give you more control over how the search box looks on your sidebar.</p>\n<p><a href=\"http://lcb.me.uk/losingit/2008/06/29/list-drafts-widget\"><strong>List Drafts Widget</strong></a></p>\n<p>List Drafts is a simple sidebar widget which outputs a list of the titles of all posts currently saved as drafts. Users can set the title of the widget, and add a label to be used for all untitled drafts.</p>\n<p><a href=\"http://www.sillybean.net/code/post-and-page-excerpt-widgets/\"><strong>Post and Page Excerpt Widgets</strong></a></p>\n<p>Creates widgets that display excerpts from posts or pages in the sidebar. You may use ‘more’ links and/or link the widget title to the post or page.</p>\n<p>And, while on the topic of plugins, take a look at Clever Carrot&#8217;s <a href=\"http://www.clevercarrot.net/wordpress/10-wordpress-plugins-to-grow-great-sites/\">10 WordPress Plugins to Grow Great Sites</a>.</p>\";}i:26;a:7:{s:5:\"title\";s:10:\"Matt: Cuil\";s:4:\"guid\";s:26:\"http://ma.tt/2008/07/cuil/\";s:4:\"link\";s:26:\"http://ma.tt/2008/07/cuil/\";s:11:\"description\";s:376:\"<p><a href=\"http://www.cuil.com/\">I like Cuil</a>. There&#8217;s just something very intuitive about the interface that fels comfortable to me, but I&#8217;m not sure how to articulate it. And it&#8217;s fast! Going to try it out as my default search engine for the next few days. Downside? Sometimes the images seem completely random, and I do see some spam in the index.</p>\";s:7:\"pubdate\";s:31:\"Tue, 29 Jul 2008 00:26:29 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:376:\"<p><a href=\"http://www.cuil.com/\">I like Cuil</a>. There&#8217;s just something very intuitive about the interface that fels comfortable to me, but I&#8217;m not sure how to articulate it. And it&#8217;s fast! Going to try it out as my default search engine for the next few days. Downside? Sometimes the images seem completely random, and I do see some spam in the index.</p>\";}i:27;a:7:{s:5:\"title\";s:60:\"Weblog Tools Collection: WordPress Plugin Releases for 07/28\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3796\";s:4:\"link\";s:88:\"http://weblogtoolscollection.com/archives/2008/07/28/wordpress-plugin-releases-for-0728/\";s:11:\"description\";s:2377:\"<p><strong><a href=\"http://www.thisrand.com/scripts/winex/\">WineX</a></strong></p>\n<p>WineX is a lightweight script that will import the listing contents of your CellarTracker wine cellar into your Wordpress website.</p>\n<p><strong><a href=\"http://www.infine.ru/support/wp_plugins/hidden-content.htm\">Hidden Content</a></strong></p>\n<p>Some blocks of the text can be shown/hidden for bots, logged-in users or all visitors (in posts, pages and comments) (<em>Page in Russian</em>)</p>\n<p><strong><a href=\"http://wp.gdragon.info/plugins/gd-star-rating/\">GD Star Rating</a></strong></p>\n<p>Star Rating plugin allows you to set up rating system for pages and/or posts in your blog. Also includes the widget for displaying votes and rating statistics for all posts and pages with votes.</p>\n<p><strong><a href=\"http://wp.gdragon.info/plugins/gd-linkedin-badge/\">GD LinkedIn Badge</a></strong></p>\n<p>This widget displays badge with a link to a LinkedIn profile page. It can also display some additional custom information above the picture and link.</p>\n<p><strong><a href=\"http://wp.gdragon.info/plugins/gd-pages-navigator/\">GD Pages Navigator</a></strong></p>\n<p>Standard display of pages in Wordpress can be a problem if you have a lot of pages with subpages causing the widget with list of pages to take a lot of space on the page.</p>\n<p><strong><a href=\"http://www.silverlight-travel.com/blog/wordpress-silverlight-plugin/\">Silverlight Content</a></strong></p>\n<p>Add Silverlight objects (XAP-files) to the post and manage default values. Like location, width, height and necessary Silverlight version.</p>\n<p><strong><a href=\"http://www.screenshine.net/blog/1474_wordpress-plugin-show-active-category\">Show Active Category</a></strong></p>\n<p>With Worpress it is only possible to show the active category when you browsing the category archive. But when you are on a single post page you have a possibility to show to which category other categories the current blogpost belongs to. This small plugin adds the CSS class &#8220;active_category&#8221; to all categories the active post belongs to. (<em>Page in German</em>)</p>\n<p><strong><a href=\"http://082net.com/2006/756/wp-slimstat-ex-plugin-en/\">WP-Slimstat-Ex</a></strong></p>\n<p>Track your blog stats. Based on Mr. Coolmann&#8217;s <a href=\"http://www.duechiacchiere.it/wp-slimstat/\" target=\"_blank\">Wp-SlimStat</a></p>\";s:7:\"pubdate\";s:31:\"Mon, 28 Jul 2008 19:44:44 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:2377:\"<p><strong><a href=\"http://www.thisrand.com/scripts/winex/\">WineX</a></strong></p>\n<p>WineX is a lightweight script that will import the listing contents of your CellarTracker wine cellar into your Wordpress website.</p>\n<p><strong><a href=\"http://www.infine.ru/support/wp_plugins/hidden-content.htm\">Hidden Content</a></strong></p>\n<p>Some blocks of the text can be shown/hidden for bots, logged-in users or all visitors (in posts, pages and comments) (<em>Page in Russian</em>)</p>\n<p><strong><a href=\"http://wp.gdragon.info/plugins/gd-star-rating/\">GD Star Rating</a></strong></p>\n<p>Star Rating plugin allows you to set up rating system for pages and/or posts in your blog. Also includes the widget for displaying votes and rating statistics for all posts and pages with votes.</p>\n<p><strong><a href=\"http://wp.gdragon.info/plugins/gd-linkedin-badge/\">GD LinkedIn Badge</a></strong></p>\n<p>This widget displays badge with a link to a LinkedIn profile page. It can also display some additional custom information above the picture and link.</p>\n<p><strong><a href=\"http://wp.gdragon.info/plugins/gd-pages-navigator/\">GD Pages Navigator</a></strong></p>\n<p>Standard display of pages in Wordpress can be a problem if you have a lot of pages with subpages causing the widget with list of pages to take a lot of space on the page.</p>\n<p><strong><a href=\"http://www.silverlight-travel.com/blog/wordpress-silverlight-plugin/\">Silverlight Content</a></strong></p>\n<p>Add Silverlight objects (XAP-files) to the post and manage default values. Like location, width, height and necessary Silverlight version.</p>\n<p><strong><a href=\"http://www.screenshine.net/blog/1474_wordpress-plugin-show-active-category\">Show Active Category</a></strong></p>\n<p>With Worpress it is only possible to show the active category when you browsing the category archive. But when you are on a single post page you have a possibility to show to which category other categories the current blogpost belongs to. This small plugin adds the CSS class &#8220;active_category&#8221; to all categories the active post belongs to. (<em>Page in German</em>)</p>\n<p><strong><a href=\"http://082net.com/2006/756/wp-slimstat-ex-plugin-en/\">WP-Slimstat-Ex</a></strong></p>\n<p>Track your blog stats. Based on Mr. Coolmann&#8217;s <a href=\"http://www.duechiacchiere.it/wp-slimstat/\" target=\"_blank\">Wp-SlimStat</a></p>\";}i:28;a:7:{s:5:\"title\";s:25:\"Donncha: WordPress MU 2.6\";s:4:\"guid\";s:29:\"http://ocaoimh.ie/?p=89493827\";s:4:\"link\";s:45:\"http://ocaoimh.ie/2008/07/28/wordpress-mu-26/\";s:11:\"description\";s:3003:\"<p>Version 2.6 of <a href=\"http://mu.wordpress.org/\">WordPress MU</a> is now out! WordPress MU is the multi blog version of the popular blogging software WordPress. It&#8217;s the engine behind <a href=\"http://wordpress.com/\">WordPress.com</a> and many other blogging sites.</p>\n<p>This version of WordPress MU is based on WordPress 2.6. There&#8217;s a long and interesting <a href=\"http://wordpress.org/development/2008/07/wordpress-26-tyner/\">WordPress.org post</a> on the new features in 2.6 so get over there to read up on post revisions, &#8220;Press This!&#8221;, Gears, Theme Previews, and the long list of developers who helped make this release a reality.</p>\n<p>Some of the new features in this release of MU:\n<ol>\n<li> Version number is 2.6 rather than 1.6 because it just makes sense to synchronise the major version numbers.</li>\n<li> Signup page now has a nonce which should help in the fight against spammers, for a short while anyway.</li>\n<li> Redirecting to the signup page for 404s and for unknown blogs is not enabled by default. Check out wp-config-sample.php for instructions.</li>\n<li> &#8220;allowed_themes&#8221; filter, much like the plugins filtered added previously.</li>\n<li> New functions: get_id_from_blogname(), is_main_blog().</li>\n<li> get_blog_details() can now take a blogname as well as a blog_id.</li>\n<li> Custom first posts didn&#8217;t always work. Now they do.</li>\n<li> Blognames in the &#8220;Add blog&#8221; form in wpmu-blogs.php are now sanitized.</li>\n<li> Added &#8220;pre_site_option_*&#8221; and &#8220;site_option_*&#8221; filters like the similar option filters.</li>\n<li> Meta fields will be passed on signup again.</li>\n<li> Added an &#8220;admin_header_navigation&#8221; filter so the top right navigation in the backend can be customised.</li>\n<li> The signup page uses &#8220;blogname&#8221; instead of &#8220;blog_id&#8221; to avoid confusion with the global variable of the same name. Plugins will break if not updated!</li>\n</ol>\n<p>That last change is quite a major one. If you have any plugins that interact with the signup form they will need to be updated!</p>\n<p>This release also addresses some security issues spotted by <a href=\"http://www.buayacorp.com/\">Alexander Concha</a> and <a href=\"http://blogs.ua.es/jgaliana\">Juan Galiana</a>. Thank you both for alerting us and for your patience while this release was prepared!</p>\n<p><img src=\"http://ocaoimh.ie/?voyeur=1\" /></p><p><strong>Related Posts</strong><ul><li><a href=\"http://ocaoimh.ie/2005/08/17/whats-this-wordpresscom-thing-anyway/\" rel=\"bookmark\" title=\"Permanent Link: What&#8217;s this WordPress.com thing anyway?\">What&#8217;s this WordPress.com thing anyway?</a></li><li><a href=\"http://ocaoimh.ie/2008/01/26/wordpress-mu-132/\" rel=\"bookmark\" title=\"Permanent Link: WordPress MU 1.3.2\">WordPress MU 1.3.2</a></li><li><a href=\"http://ocaoimh.ie/2008/02/05/wordpress-mu-133/\" rel=\"bookmark\" title=\"Permanent Link: WordPress MU 1.3.3\">WordPress MU 1.3.3</a></li></ul></p>\";s:7:\"pubdate\";s:31:\"Mon, 28 Jul 2008 17:26:36 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"Donncha\";}s:7:\"summary\";s:3003:\"<p>Version 2.6 of <a href=\"http://mu.wordpress.org/\">WordPress MU</a> is now out! WordPress MU is the multi blog version of the popular blogging software WordPress. It&#8217;s the engine behind <a href=\"http://wordpress.com/\">WordPress.com</a> and many other blogging sites.</p>\n<p>This version of WordPress MU is based on WordPress 2.6. There&#8217;s a long and interesting <a href=\"http://wordpress.org/development/2008/07/wordpress-26-tyner/\">WordPress.org post</a> on the new features in 2.6 so get over there to read up on post revisions, &#8220;Press This!&#8221;, Gears, Theme Previews, and the long list of developers who helped make this release a reality.</p>\n<p>Some of the new features in this release of MU:\n<ol>\n<li> Version number is 2.6 rather than 1.6 because it just makes sense to synchronise the major version numbers.</li>\n<li> Signup page now has a nonce which should help in the fight against spammers, for a short while anyway.</li>\n<li> Redirecting to the signup page for 404s and for unknown blogs is not enabled by default. Check out wp-config-sample.php for instructions.</li>\n<li> &#8220;allowed_themes&#8221; filter, much like the plugins filtered added previously.</li>\n<li> New functions: get_id_from_blogname(), is_main_blog().</li>\n<li> get_blog_details() can now take a blogname as well as a blog_id.</li>\n<li> Custom first posts didn&#8217;t always work. Now they do.</li>\n<li> Blognames in the &#8220;Add blog&#8221; form in wpmu-blogs.php are now sanitized.</li>\n<li> Added &#8220;pre_site_option_*&#8221; and &#8220;site_option_*&#8221; filters like the similar option filters.</li>\n<li> Meta fields will be passed on signup again.</li>\n<li> Added an &#8220;admin_header_navigation&#8221; filter so the top right navigation in the backend can be customised.</li>\n<li> The signup page uses &#8220;blogname&#8221; instead of &#8220;blog_id&#8221; to avoid confusion with the global variable of the same name. Plugins will break if not updated!</li>\n</ol>\n<p>That last change is quite a major one. If you have any plugins that interact with the signup form they will need to be updated!</p>\n<p>This release also addresses some security issues spotted by <a href=\"http://www.buayacorp.com/\">Alexander Concha</a> and <a href=\"http://blogs.ua.es/jgaliana\">Juan Galiana</a>. Thank you both for alerting us and for your patience while this release was prepared!</p>\n<p><img src=\"http://ocaoimh.ie/?voyeur=1\" /></p><p><strong>Related Posts</strong><ul><li><a href=\"http://ocaoimh.ie/2005/08/17/whats-this-wordpresscom-thing-anyway/\" rel=\"bookmark\" title=\"Permanent Link: What&#8217;s this WordPress.com thing anyway?\">What&#8217;s this WordPress.com thing anyway?</a></li><li><a href=\"http://ocaoimh.ie/2008/01/26/wordpress-mu-132/\" rel=\"bookmark\" title=\"Permanent Link: WordPress MU 1.3.2\">WordPress MU 1.3.2</a></li><li><a href=\"http://ocaoimh.ie/2008/02/05/wordpress-mu-133/\" rel=\"bookmark\" title=\"Permanent Link: WordPress MU 1.3.3\">WordPress MU 1.3.3</a></li></ul></p>\";}i:29;a:7:{s:5:\"title\";s:48:\"Weblog Tools Collection: WordPress Tips for 7/27\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3786\";s:4:\"link\";s:76:\"http://weblogtoolscollection.com/archives/2008/07/27/wordpress-tips-for-727/\";s:11:\"description\";s:1821:\"<p><a href=\"http://hackwordpress.com/wordpress-26-changes-how-wordpress-handles-images/\">Image alignment within a WordPress post</a>: WordPress 2.5 introduced floating image alignment within text on a post. This feature does not work on certain themes. Thanks to <a href=\"http://www.theglitch.ws\">Jason</a> for asking the question and to <a href=\"http://wprocks.com/\">Sadish</a> for the solution. Final thought: it would also be nice if a few gallery formatting options were available through the default interface instead of having to modify the shortcode and/or introduce hacks.</p>\n<p><a href=\"http://firewalker.kamusilmiah.com/wordpress-search-as-custom-google-search-without-additional-page-or-post/\">WordPress Search as a custom Google Search without addition page or post</a>: Add a custom Google Search to your WordPress blog (and make search advertising dollars?) without too much of a hassle. Complete instructions and examples are included in the post.</p>\n<p><a href=\"http://www.emanueleferonato.com/2008/07/27/creation-of-a-flash-arcade-site-using-wordpress-step-2/\">Create a Flash Arcade site using WordPress</a>: This set of instructions will help you convert a simple WordPress blog into a Flash arcade site. Pretty cool with detailed instructions on the transition.</p>\n<p>Here is a tip which I stumbled upon accidentally, that is somewhat unrelated to WordPress. Visit a NYTimes page such as <a href=\"http://www.nytimes.com/2008/07/27/technology/27proto.html?_r=1&amp;partner=rssnyt&amp;emc=rss\">a news item</a> and double click on a single word. You will need a free registration to view that page or have to be logged in if you are registered already. The javascript file that adds the functionality in question is copyrighted to Answers.com and I wonder if a WordPress plugin exists for the same.</p>\";s:7:\"pubdate\";s:31:\"Sun, 27 Jul 2008 21:55:21 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:1821:\"<p><a href=\"http://hackwordpress.com/wordpress-26-changes-how-wordpress-handles-images/\">Image alignment within a WordPress post</a>: WordPress 2.5 introduced floating image alignment within text on a post. This feature does not work on certain themes. Thanks to <a href=\"http://www.theglitch.ws\">Jason</a> for asking the question and to <a href=\"http://wprocks.com/\">Sadish</a> for the solution. Final thought: it would also be nice if a few gallery formatting options were available through the default interface instead of having to modify the shortcode and/or introduce hacks.</p>\n<p><a href=\"http://firewalker.kamusilmiah.com/wordpress-search-as-custom-google-search-without-additional-page-or-post/\">WordPress Search as a custom Google Search without addition page or post</a>: Add a custom Google Search to your WordPress blog (and make search advertising dollars?) without too much of a hassle. Complete instructions and examples are included in the post.</p>\n<p><a href=\"http://www.emanueleferonato.com/2008/07/27/creation-of-a-flash-arcade-site-using-wordpress-step-2/\">Create a Flash Arcade site using WordPress</a>: This set of instructions will help you convert a simple WordPress blog into a Flash arcade site. Pretty cool with detailed instructions on the transition.</p>\n<p>Here is a tip which I stumbled upon accidentally, that is somewhat unrelated to WordPress. Visit a NYTimes page such as <a href=\"http://www.nytimes.com/2008/07/27/technology/27proto.html?_r=1&amp;partner=rssnyt&amp;emc=rss\">a news item</a> and double click on a single word. You will need a free registration to view that page or have to be logged in if you are registered already. The javascript file that adds the functionality in question is copyrighted to Answers.com and I wonder if a WordPress plugin exists for the same.</p>\";}i:30;a:7:{s:5:\"title\";s:64:\"Weblog Tools Collection: WordCamp San Francisco 2008 Signup Open\";s:4:\"guid\";s:93:\"http://weblogtoolscollection.com/archives/2008/07/27/wordcamp-san-francisco-2008-signup-open/\";s:4:\"link\";s:93:\"http://weblogtoolscollection.com/archives/2008/07/27/wordcamp-san-francisco-2008-signup-open/\";s:11:\"description\";s:814:\"</p>\n<p>Matt has opened up the signup for the WordCamp San Francisco event which will be held on August 16th 2008.</p>\n<p>The WordCamp SF ticket prizes has been reduced by $5, each ticket will only cost $20 for this event. The event is being hosted at a bigger venue this time around, so you can expect better and comfortable seating arrangements, one of things Matt has ensured us about.</p>\n<p>Going by the success of past WordCamp&#8217;s, we believe this one will be even bigger and better. You can book your tickets for the WordCamp SF event by visiting <a href=\"http://2008.sf.wordcamp.org/signup/\" target=\"_blank\">this page</a>. Don’t forget to read more about <a href=\"http://weblogtoolscollection.com/archives/2008/01/29/what-is-wordcamp/\" target=\"_blank\">what is WordCamp</a>, if you are new to it.</p>\";s:7:\"pubdate\";s:31:\"Sun, 27 Jul 2008 18:08:22 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:814:\"</p>\n<p>Matt has opened up the signup for the WordCamp San Francisco event which will be held on August 16th 2008.</p>\n<p>The WordCamp SF ticket prizes has been reduced by $5, each ticket will only cost $20 for this event. The event is being hosted at a bigger venue this time around, so you can expect better and comfortable seating arrangements, one of things Matt has ensured us about.</p>\n<p>Going by the success of past WordCamp&#8217;s, we believe this one will be even bigger and better. You can book your tickets for the WordCamp SF event by visiting <a href=\"http://2008.sf.wordcamp.org/signup/\" target=\"_blank\">this page</a>. Don’t forget to read more about <a href=\"http://weblogtoolscollection.com/archives/2008/01/29/what-is-wordcamp/\" target=\"_blank\">what is WordCamp</a>, if you are new to it.</p>\";}i:31;a:7:{s:5:\"title\";s:18:\"Matt: Another one!\";s:4:\"guid\";s:33:\"http://ma.tt/2008/07/another-one/\";s:4:\"link\";s:33:\"http://ma.tt/2008/07/another-one/\";s:11:\"description\";s:588:\"<p><a href=\"http://s.ma.tt/files/2008/07/p-640-480-5f0856fa-7afe-4b81-853d-f141ec95b8ed.jpeg\"><img class=\"size-full wp-image-364 alignright\" src=\"http://s.ma.tt/files/2008/07/p-640-480-5f0856fa-7afe-4b81-853d-f141ec95b8ed.jpeg\" alt=\"WordPress Book\" width=\"225\" height=\"300\" /></a> I just got another Japanese WordPress book in the mail. <img src=\"http://ma.tt/blog/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /> I can&#8217;t wait to visit there later in the year. P.S. I&#8217;m not sure who this is from, so if you&#8217;re the person who sent it email me.</p>\";s:7:\"pubdate\";s:31:\"Sun, 27 Jul 2008 17:13:55 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:588:\"<p><a href=\"http://s.ma.tt/files/2008/07/p-640-480-5f0856fa-7afe-4b81-853d-f141ec95b8ed.jpeg\"><img class=\"size-full wp-image-364 alignright\" src=\"http://s.ma.tt/files/2008/07/p-640-480-5f0856fa-7afe-4b81-853d-f141ec95b8ed.jpeg\" alt=\"WordPress Book\" width=\"225\" height=\"300\" /></a> I just got another Japanese WordPress book in the mail. <img src=\"http://ma.tt/blog/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /> I can&#8217;t wait to visit there later in the year. P.S. I&#8217;m not sure who this is from, so if you&#8217;re the person who sent it email me.</p>\";}i:32;a:7:{s:5:\"title\";s:54:\"Weblog Tools Collection: It’s time to cast your vote\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3764\";s:4:\"link\";s:80:\"http://weblogtoolscollection.com/archives/2008/07/26/its-time-to-cast-your-vote/\";s:11:\"description\";s:5861:\"<p>Listed here are the entries to the WordPress Plugin Competition 2.5.</p>\n<p>Please help us choose the winners by visiting each of the plugins below and rating them on the linked page.</p>\n<ul>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/22/adminimize/\">Adminimize</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/anyvar/\">AnyVar</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/attachment-extender/\">Attachment Extender</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/19/automatic-timezone/\">Automatic Timezone</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/11/broken-links-remover/\">Broken links Remover</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/click-tracking-plugin/\">Click</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/06/comicpress-manager/\">ComicPress Manager</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/dk-obama-slideshow-plugin/\">Dk Obama Gallery</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/11/dodo/\">DoDo</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/editor-extender/\">Editor Extender</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/flaresmith/\">Flaresmith</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/flickr-gallery/\">Flickr Gallery</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/09/fun-with-in-context-comments/\">Fun with in-context comments</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/24/great-real-estate/\">Great Real Estate</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/idealien-category-enhancements/\">Idealien Category Enhancements</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/wordpress-image-browser-extender/\">Image Browser Extender</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/invite-friends/\">Invite friends</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/live-blogroll/\">Live Blogroll</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/last-release-of-mailpress-ver-12/\">MailPress</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/25/manageable/\">Manageable</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/25/maxref-widgets/\">Maxref Widgets</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/media-in-response/\">Media in response</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/08/multisitemanager-unleashed/\">MultiSiteManager</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/25/nonce-please/\">Nonce! Please</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/04/photojar-base/\">PhotoJAR: Base</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/25/pics-picture-increment-cron-system/\">Picture Increment Cron System</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/26/postgroups/\">Post Groups</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/24/post-tabs/\">Post Tabs</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/wpmu-plugin-repost/\">Repost</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/11/role-scoper/\">Role Scoper</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/22/seo-smart-links/\">SEO Smart Links</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/12/sermon-management-plug-in-for-churches/\">Sermon Management</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/show-me-options/\">Show Me Options</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/01/sidebars/\">Sidebars</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/simple-flash-video-plugin-15-beta-2/\">Simple Flash Video</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/slideshare-plugin/\">SlideShare Presentation List</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/snazzy-archives-2/\">Snazzy Archives</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/paypal-api-subscriptions/\">Paypal API Subscriptions</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/tabbed-widgets-plugin/\">Tabbed Widgets</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/02/twptter-wordpress-microblogging/\">tWPtter</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/vent-add-events-to-your-blog/\">Vent</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/wpmu-plugin-vote2publish/\">Vote2Publish</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/wp125-easy-125x125-ad-management-for-wordpress/\">WP125</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/05/30/wp-comment-remix/\">WP Comment Remix</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/29/wp-easy-uploader/\">WP Easy Uploader</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/08/wp-sentry-private-posting-plugin/\">WP Sentry</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/06/wp-wall/\">WP Wall</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/yank-widget/\">Yank Widget</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/24/zdmultilang/\">ZdMultiLang</a></li>\n</ul>\n<p>Remember to test the plugin properly before giving your vote.</p>\";s:7:\"pubdate\";s:31:\"Sat, 26 Jul 2008 18:01:53 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:5861:\"<p>Listed here are the entries to the WordPress Plugin Competition 2.5.</p>\n<p>Please help us choose the winners by visiting each of the plugins below and rating them on the linked page.</p>\n<ul>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/22/adminimize/\">Adminimize</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/anyvar/\">AnyVar</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/attachment-extender/\">Attachment Extender</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/19/automatic-timezone/\">Automatic Timezone</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/11/broken-links-remover/\">Broken links Remover</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/click-tracking-plugin/\">Click</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/06/comicpress-manager/\">ComicPress Manager</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/dk-obama-slideshow-plugin/\">Dk Obama Gallery</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/11/dodo/\">DoDo</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/editor-extender/\">Editor Extender</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/flaresmith/\">Flaresmith</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/flickr-gallery/\">Flickr Gallery</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/09/fun-with-in-context-comments/\">Fun with in-context comments</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/24/great-real-estate/\">Great Real Estate</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/idealien-category-enhancements/\">Idealien Category Enhancements</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/wordpress-image-browser-extender/\">Image Browser Extender</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/invite-friends/\">Invite friends</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/live-blogroll/\">Live Blogroll</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/last-release-of-mailpress-ver-12/\">MailPress</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/25/manageable/\">Manageable</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/25/maxref-widgets/\">Maxref Widgets</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/09/media-in-response/\">Media in response</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/08/multisitemanager-unleashed/\">MultiSiteManager</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/25/nonce-please/\">Nonce! Please</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/04/photojar-base/\">PhotoJAR: Base</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/25/pics-picture-increment-cron-system/\">Picture Increment Cron System</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/26/postgroups/\">Post Groups</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/24/post-tabs/\">Post Tabs</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/wpmu-plugin-repost/\">Repost</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/11/role-scoper/\">Role Scoper</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/22/seo-smart-links/\">SEO Smart Links</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/12/sermon-management-plug-in-for-churches/\">Sermon Management</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/show-me-options/\">Show Me Options</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/01/sidebars/\">Sidebars</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/simple-flash-video-plugin-15-beta-2/\">Simple Flash Video</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/slideshare-plugin/\">SlideShare Presentation List</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/snazzy-archives-2/\">Snazzy Archives</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/paypal-api-subscriptions/\">Paypal API Subscriptions</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/tabbed-widgets-plugin/\">Tabbed Widgets</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/02/twptter-wordpress-microblogging/\">tWPtter</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/vent-add-events-to-your-blog/\">Vent</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/07/wpmu-plugin-vote2publish/\">Vote2Publish</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/wp125-easy-125x125-ad-management-for-wordpress/\">WP125</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/05/30/wp-comment-remix/\">WP Comment Remix</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/29/wp-easy-uploader/\">WP Easy Uploader</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/08/wp-sentry-private-posting-plugin/\">WP Sentry</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/06/wp-wall/\">WP Wall</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/07/10/yank-widget/\">Yank Widget</a></li>\n<li><a href=\"http://weblogtoolscollection.com/pluginblog/2008/06/24/zdmultilang/\">ZdMultiLang</a></li>\n</ul>\n<p>Remember to test the plugin properly before giving your vote.</p>\";}i:33;a:7:{s:5:\"title\";s:68:\"Lorelle on WP: Liz Strauss is Breaking the Web and Pushing WordPress\";s:4:\"guid\";s:36:\"http://lorelle.wordpress.com/?p=2720\";s:4:\"link\";s:94:\"http://lorelle.wordpress.com/2008/07/26/liz-strauss-is-breaking-the-web-and-pushing-wordpress/\";s:11:\"description\";s:3077:\"<div class=\"snap_preview\"><br /><p><a href=\"http://www.successful-blog.com/\" title=\"Liz Strauss\">Liz Strauss of Successful and Outstanding Bloggers</a> is attempting to &#8220;break the web&#8221; today at noon CST from Chicago (GMT -6). As part of her <a href=\"http://www.successful-blog.com/1/blog-to-show-showcase-your-blog-at-successful-blog-july-26-27/\" title=\"Showcase your blog\">Showcase Your Blog</a> series, she will be using the <a href=\"http://lorelle.wordpress.com/2006/10/01/working-ahead-future-posts-with-wordpress/\" title=\"Future Posts in WordPress\">future post feature in WordPress</a> to publish <em>a blog post a minute</em> showcasing more than 250 blogs.</p>\n<p>Has anyone set up WordPress to do this before? Maybe some splogs or scrapers, but this is an exciting example of how a community project can use the WordPress future post feature, and push WordPress and the server to keep up with the server and database load. Liz is part of the b5media network, so this is a test of their setup as well.</p>\n<p>Liz tells me that the blogs she is showcasing are excellent, some of the most exciting, beautiful, and social blogs around, so you are in for a treat as you watch her Blog Showcase churn out, minute by minute. I wonder what the traffic count is going to be on her blog&#8217;s front page. Luckily, she&#8217;s using <a href=\"http://www.woopra.com/\" title=\"Woopra Web Analytics\">Woopra</a> to track her traffic, stats, and visitors <em>live</em>. What fun!</p>\n<p>Go <a href=\"http://www.successful-blog.com/\" title=\"Liz Strauss\">check out the minute-by-minute Blog Showcase</a>! You can also follow the Twitter stream at <a href=\"http://www.twitter.com/b2show/\" title=\"Twitter for Blog Showcase with Liz Strauss\">b2show</a>.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/lorelle.wordpress.com/2720/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/lorelle.wordpress.com/2720/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/lorelle.wordpress.com/2720/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/lorelle.wordpress.com/2720/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/lorelle.wordpress.com/2720/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/lorelle.wordpress.com/2720/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/lorelle.wordpress.com/2720/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=lorelle.wordpress.com&blog=72&post=2720&subd=lorelle&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Sat, 26 Jul 2008 17:40:23 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:17:\"Lorelle VanFossen\";}s:7:\"summary\";s:3077:\"<div class=\"snap_preview\"><br /><p><a href=\"http://www.successful-blog.com/\" title=\"Liz Strauss\">Liz Strauss of Successful and Outstanding Bloggers</a> is attempting to &#8220;break the web&#8221; today at noon CST from Chicago (GMT -6). As part of her <a href=\"http://www.successful-blog.com/1/blog-to-show-showcase-your-blog-at-successful-blog-july-26-27/\" title=\"Showcase your blog\">Showcase Your Blog</a> series, she will be using the <a href=\"http://lorelle.wordpress.com/2006/10/01/working-ahead-future-posts-with-wordpress/\" title=\"Future Posts in WordPress\">future post feature in WordPress</a> to publish <em>a blog post a minute</em> showcasing more than 250 blogs.</p>\n<p>Has anyone set up WordPress to do this before? Maybe some splogs or scrapers, but this is an exciting example of how a community project can use the WordPress future post feature, and push WordPress and the server to keep up with the server and database load. Liz is part of the b5media network, so this is a test of their setup as well.</p>\n<p>Liz tells me that the blogs she is showcasing are excellent, some of the most exciting, beautiful, and social blogs around, so you are in for a treat as you watch her Blog Showcase churn out, minute by minute. I wonder what the traffic count is going to be on her blog&#8217;s front page. Luckily, she&#8217;s using <a href=\"http://www.woopra.com/\" title=\"Woopra Web Analytics\">Woopra</a> to track her traffic, stats, and visitors <em>live</em>. What fun!</p>\n<p>Go <a href=\"http://www.successful-blog.com/\" title=\"Liz Strauss\">check out the minute-by-minute Blog Showcase</a>! You can also follow the Twitter stream at <a href=\"http://www.twitter.com/b2show/\" title=\"Twitter for Blog Showcase with Liz Strauss\">b2show</a>.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/lorelle.wordpress.com/2720/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/lorelle.wordpress.com/2720/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/lorelle.wordpress.com/2720/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/lorelle.wordpress.com/2720/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/lorelle.wordpress.com/2720/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/lorelle.wordpress.com/2720/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/lorelle.wordpress.com/2720/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/lorelle.wordpress.com/2720/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=lorelle.wordpress.com&blog=72&post=2720&subd=lorelle&ref=&feed=1\" /></div>\";}i:34;a:7:{s:5:\"title\";s:52:\"Lorelle on WP: Liz Strauss is Going to Break the Web\";s:4:\"guid\";s:36:\"http://lorelle.wordpress.com/?p=2723\";s:4:\"link\";s:78:\"http://lorelle.wordpress.com/2008/07/26/liz-strauss-is-going-to-break-the-web/\";s:11:\"description\";s:1862:\"<div class=\"snap_preview\"><br /><p><a href=\"http://www.successful-blog.com/\" title=\"Liz Strauss\">Liz Strauss of Successful and Outstanding Bloggers</a> just called me and said that as of noon CST from Chicago (GMT -6), she is going to break the web as part of her <a href=\"http://www.successful-blog.com/1/blog-to-show-showcase-your-blog-at-successful-blog-july-26-27/\" title=\"Showcase your blog\">Showcase Your Blog</a> series. Stay tuned for more information on how she is going to use her WordPress blog in a way few have done before.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/lorelle.wordpress.com/2723/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/lorelle.wordpress.com/2723/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/lorelle.wordpress.com/2723/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/lorelle.wordpress.com/2723/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/lorelle.wordpress.com/2723/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/lorelle.wordpress.com/2723/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/lorelle.wordpress.com/2723/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=lorelle.wordpress.com&blog=72&post=2723&subd=lorelle&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Sat, 26 Jul 2008 16:56:09 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:17:\"Lorelle VanFossen\";}s:7:\"summary\";s:1862:\"<div class=\"snap_preview\"><br /><p><a href=\"http://www.successful-blog.com/\" title=\"Liz Strauss\">Liz Strauss of Successful and Outstanding Bloggers</a> just called me and said that as of noon CST from Chicago (GMT -6), she is going to break the web as part of her <a href=\"http://www.successful-blog.com/1/blog-to-show-showcase-your-blog-at-successful-blog-july-26-27/\" title=\"Showcase your blog\">Showcase Your Blog</a> series. Stay tuned for more information on how she is going to use her WordPress blog in a way few have done before.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/lorelle.wordpress.com/2723/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/lorelle.wordpress.com/2723/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/lorelle.wordpress.com/2723/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/lorelle.wordpress.com/2723/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/lorelle.wordpress.com/2723/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/lorelle.wordpress.com/2723/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/lorelle.wordpress.com/2723/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/lorelle.wordpress.com/2723/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=lorelle.wordpress.com&blog=72&post=2723&subd=lorelle&ref=&feed=1\" /></div>\";}i:35;a:7:{s:5:\"title\";s:23:\"Matt: Beyond REST? XMPP\";s:4:\"guid\";s:38:\"http://ma.tt/2008/07/beyond-rest-xmpp/\";s:4:\"link\";s:38:\"http://ma.tt/2008/07/beyond-rest-xmpp/\";s:11:\"description\";s:174:\"<p><a href=\"http://laughingmeme.org/2008/07/23/beyond-rest-building-data-services-with-xmpp/\">Beyond REST? Building data services with XMPP</a>. Pretty fascinating stuff.</p>\";s:7:\"pubdate\";s:31:\"Sat, 26 Jul 2008 15:03:13 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:174:\"<p><a href=\"http://laughingmeme.org/2008/07/23/beyond-rest-building-data-services-with-xmpp/\">Beyond REST? Building data services with XMPP</a>. Pretty fascinating stuff.</p>\";}i:36;a:7:{s:5:\"title\";s:32:\"Matt: Single Malt Scotch Episode\";s:4:\"guid\";s:48:\"http://ma.tt/2008/07/single-malt-scotch-episode/\";s:4:\"link\";s:48:\"http://ma.tt/2008/07/single-malt-scotch-episode/\";s:11:\"description\";s:690:\"<p></p>\n<p><a href=\"http://tv.winelibrary.com/2008/07/25/the-single-malt-scotch-episode-episode-509/\">The Single Malt Scotch Episode - Episode #509</a>. I was on Wine Library TV with the inimitable <a href=\"http://garyvaynerchuk.com/\">Gary Vaynerchuk</a> talking not about wine, but scotch. Much trouble ensues. <a href=\"http://tv.winelibrary.com/2008/07/25/the-single-malt-scotch-episode-episode-509/\">Check it out</a>! I ended up leaving my laptop at Gary&#8217;s store and had to go back to Jersey later that day. I took a few moments to browse around and discovered <a href=\"http://www.formaggiokitchen.com/shop/product_info.php?products_id=1994\">an awesome cheese called Ombra</a>.</p>\";s:7:\"pubdate\";s:31:\"Sat, 26 Jul 2008 00:30:20 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:690:\"<p></p>\n<p><a href=\"http://tv.winelibrary.com/2008/07/25/the-single-malt-scotch-episode-episode-509/\">The Single Malt Scotch Episode - Episode #509</a>. I was on Wine Library TV with the inimitable <a href=\"http://garyvaynerchuk.com/\">Gary Vaynerchuk</a> talking not about wine, but scotch. Much trouble ensues. <a href=\"http://tv.winelibrary.com/2008/07/25/the-single-malt-scotch-episode-episode-509/\">Check it out</a>! I ended up leaving my laptop at Gary&#8217;s store and had to go back to Jersey later that day. I took a few moments to browse around and discovered <a href=\"http://www.formaggiokitchen.com/shop/product_info.php?products_id=1994\">an awesome cheese called Ombra</a>.</p>\";}i:37;a:7:{s:5:\"title\";s:59:\"Weblog Tools Collection: WordPress Theme Releases for 07/25\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3783\";s:4:\"link\";s:87:\"http://weblogtoolscollection.com/archives/2008/07/25/wordpress-theme-releases-for-0725/\";s:11:\"description\";s:2780:\"<p><a href=\"http://www.jauhari.net/themes/vita\"><strong>Vita</strong></a></p>\n<p><img alt=\"vita\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/vita.jpg\" width=\"200\" height=\"109\" /></p>\n<p>Three column, widget ready, fixed width, custom image header theme</p>\n<p><a href=\"http://myblogit.net/2008/07/15/free-wordpress-theme-kitten-in-pink/\"><strong>Kitten in Pink</strong></a></p>\n<p><a href=\"http://knowledge.myblogit.net/index.php?preview_theme=Kitten%20in%20Pink\"><img alt=\"KittenInPink\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/kitteninpink.jpg\" width=\"188\" height=\"125\" /></a></p>\n<p>Two column, pink, widget ready theme</p>\n<p><a href=\"http://wordpress.org/extend/themes/wp-andreas01\"><strong>WP-Andreas01</strong></a></p>\n<p><img alt=\"WP-Andreas01\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/wp-andreas01.png\" width=\"167\" height=\"125\" /></p>\n<p>Three column, widget ready theme with support for tags, gravatars and including 12 additional header images.</p>\n<p><a href=\"http://wordpress.org/extend/themes/daleri-sweet\"><strong>Daleri Sweet</strong></a></p>\n<p><img alt=\"DaleriSweet\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/dalerisweet.png\" width=\"167\" height=\"125\" /></p>\n<p>Fixed width, two / three column, widget ready blue WordPress theme</p>\n<p><a href=\"http://3oneseven.com/24/secondary-wordpress-theme/\"><strong>Secondary </strong></a></p>\n<p><a href=\"http://wp.3oneseven.com/index.php?wptheme=Secondary\"><img alt=\"Secondary\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/secondary.jpg\" width=\"200\" height=\"100\" /></a></p>\n<p>Consists of a fluid two column outfit with news slider built into the frontpage: widgetized sidebar and footer columns with typical WordPress action like calendar, built in are recent comments with gravatars, related posts, social bookmarking enabled, native WP tag system, flickr is enabled in its own options panel.</p>\n<p><a href=\"http://www.quickonlinetips.com/archives/quickpress/\"><strong>QuickPress</strong></a></p>\n<p><a href=\"http://www.quickonlinetips.com/wordpress/quickpress/\"><img alt=\"QuickPress\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/quickpress.jpg\" width=\"200\" height=\"121\" /></a></p>\n<p>A 2/3-column, widget-ready, SEO-optimized, Ad-ready, WordPress 2.6 compatible wordpress theme which is useful for all types of bloggers.</p>\n<p><strong><a href=\"http://wordpress.positivenoize.com/wordpress-themes/\">TheGrunge</a></strong></p>\n<p><a href=\"http://wordpress.positivenoize.com/index.php?wptheme=TheGrunge\"><img alt=\"TheGrunge\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/thegrunge.jpg\" width=\"180\" height=\"125\" /></a></p>\n<p>TheGrunge WordPress Theme is a widget and ad-ready WordPress 2.5 Grunge Theme re-developed by the PositiveNoize design team.</p>\";s:7:\"pubdate\";s:31:\"Fri, 25 Jul 2008 19:55:37 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:2780:\"<p><a href=\"http://www.jauhari.net/themes/vita\"><strong>Vita</strong></a></p>\n<p><img alt=\"vita\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/vita.jpg\" width=\"200\" height=\"109\" /></p>\n<p>Three column, widget ready, fixed width, custom image header theme</p>\n<p><a href=\"http://myblogit.net/2008/07/15/free-wordpress-theme-kitten-in-pink/\"><strong>Kitten in Pink</strong></a></p>\n<p><a href=\"http://knowledge.myblogit.net/index.php?preview_theme=Kitten%20in%20Pink\"><img alt=\"KittenInPink\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/kitteninpink.jpg\" width=\"188\" height=\"125\" /></a></p>\n<p>Two column, pink, widget ready theme</p>\n<p><a href=\"http://wordpress.org/extend/themes/wp-andreas01\"><strong>WP-Andreas01</strong></a></p>\n<p><img alt=\"WP-Andreas01\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/wp-andreas01.png\" width=\"167\" height=\"125\" /></p>\n<p>Three column, widget ready theme with support for tags, gravatars and including 12 additional header images.</p>\n<p><a href=\"http://wordpress.org/extend/themes/daleri-sweet\"><strong>Daleri Sweet</strong></a></p>\n<p><img alt=\"DaleriSweet\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/dalerisweet.png\" width=\"167\" height=\"125\" /></p>\n<p>Fixed width, two / three column, widget ready blue WordPress theme</p>\n<p><a href=\"http://3oneseven.com/24/secondary-wordpress-theme/\"><strong>Secondary </strong></a></p>\n<p><a href=\"http://wp.3oneseven.com/index.php?wptheme=Secondary\"><img alt=\"Secondary\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/secondary.jpg\" width=\"200\" height=\"100\" /></a></p>\n<p>Consists of a fluid two column outfit with news slider built into the frontpage: widgetized sidebar and footer columns with typical WordPress action like calendar, built in are recent comments with gravatars, related posts, social bookmarking enabled, native WP tag system, flickr is enabled in its own options panel.</p>\n<p><a href=\"http://www.quickonlinetips.com/archives/quickpress/\"><strong>QuickPress</strong></a></p>\n<p><a href=\"http://www.quickonlinetips.com/wordpress/quickpress/\"><img alt=\"QuickPress\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/quickpress.jpg\" width=\"200\" height=\"121\" /></a></p>\n<p>A 2/3-column, widget-ready, SEO-optimized, Ad-ready, WordPress 2.6 compatible wordpress theme which is useful for all types of bloggers.</p>\n<p><strong><a href=\"http://wordpress.positivenoize.com/wordpress-themes/\">TheGrunge</a></strong></p>\n<p><a href=\"http://wordpress.positivenoize.com/index.php?wptheme=TheGrunge\"><img alt=\"TheGrunge\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/thegrunge.jpg\" width=\"180\" height=\"125\" /></a></p>\n<p>TheGrunge WordPress Theme is a widget and ad-ready WordPress 2.5 Grunge Theme re-developed by the PositiveNoize design team.</p>\";}i:38;a:7:{s:5:\"title\";s:26:\"Matt: WordCamp Signup Open\";s:4:\"guid\";s:42:\"http://ma.tt/2008/07/wordcamp-signup-open/\";s:4:\"link\";s:42:\"http://ma.tt/2008/07/wordcamp-signup-open/\";s:11:\"description\";s:484:\"<p><a href=\"http://2008.sf.wordcamp.org/signup/\">WordCamp San Francisco 2008 Signup</a>. Signup is now open for WordCamp SF, and it&#8217;s $5 cheaper than last year even though we&#8217;re at a bigger venue (with more comfortable seats, based on your feedback) and should be packing more awesomeness. Who&#8217;s going to be speaking on what? Well that&#8217;s still a surprise. <img src=\"http://ma.tt/blog/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /></p>\";s:7:\"pubdate\";s:31:\"Fri, 25 Jul 2008 16:11:13 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:484:\"<p><a href=\"http://2008.sf.wordcamp.org/signup/\">WordCamp San Francisco 2008 Signup</a>. Signup is now open for WordCamp SF, and it&#8217;s $5 cheaper than last year even though we&#8217;re at a bigger venue (with more comfortable seats, based on your feedback) and should be packing more awesomeness. Who&#8217;s going to be speaking on what? Well that&#8217;s still a surprise. <img src=\"http://ma.tt/blog/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /></p>\";}i:39;a:7:{s:5:\"title\";s:52:\"Lloyd: WordPress for iPhone + Source Code Available!\";s:4:\"guid\";s:30:\"http://foolswisdom.com/?p=1291\";s:4:\"link\";s:66:\"http://foolswisdom.com/wordpress-for-iphone-source-code-available/\";s:11:\"description\";s:2463:\"<p><img class=\"alignright\" title=\"WordPress for iPhone Logo\" src=\"http://farm4.static.flickr.com/3272/2701598580_b3c2b4a169_o.jpg\" alt=\"\" width=\"100\" height=\"100\" />Earlier this week my <a href=\"http://automattic.com/\">Automattic</a> colleagues and <a href=\"http://www.effigent.com/\">Effigent</a> released <a href=\"http://iphone.wordpress.org/\">WordPress for iPhone</a> and iPod Touch. Now, the source is also available and Trac is set up for reporting bugs and <a href=\"http://iphone.trac.wordpress.org/\">participating in development</a>!</p>\n<p>I&#8217;ve tried it on my iPod Touch. They&#8217;ve done a great job! And are <a href=\"http://iphone.wordpress.org/2008/07/24/version-11-and-beyond/\">urgently working on fixing</a> the worse bugs.</p>\n<p>It&#8217;s already in the top 50 free apps.</p>\n<p><span id=\"more-1291\"></span>Development has been managed by <a href=\"http://raanan.com/\">Raanan Bar-Cohen</a> and the app is designed by <a href=\"http://iammattthomas.com/\">Matt Thomas</a>.</p>\n<p>WordPress for iPhone has the clean Matt aesthetic, though I expect the iPhone SDK probably doesn&#8217;t allow much deviation from clean simplicity.</p>\n<p>There is <a href=\"http://search.twitter.com/search?q=wordpress+nda\">quite some</a> <a href=\"http://www.tuaw.com/2008/07/24/wordpress-for-iphone-source-now-available/\">stink up</a> about whether or not we are breaking Apple&#8217;s iPhone development NDA by our application being open source, and speculation that Apple will sue us.</p>\n<p><span class=\"byline\"><a href=\"http://dynamic.boingboing.net/profile/John%20Brownlee\">John Brownlee</a> of BoingBoing Gadgets writes:</span></p>\n<blockquote><p>I was curious about the &#8220;Open Source&#8221; proclamation  at the top of its non-App-Store webpage.</p></blockquote>\n<p>WordPress for iPhone is in the App Store. WordPress for iPhone&#8217;s Application Description has always stated it&#8217;s open source. Someone at Apple approved &#8220;WordPress for iPhone Open Source&#8221;.</p>\n<p>Although it makes for a great story and gives us <a href=\"http://daringfireball.net/linked/2008/07/24/wordpress-iphone\">freetards reason to bang our chests</a>, my thought is Apple has it&#8217;s anti-open source clause in the agreement to cover their butts from their code being &#8220;poisoned&#8221; by a viral license. Butt covered, let the <a href=\"http://foolswisdom.com/open-source-in-iphone-app-store/\">open source applications flow</a>, but no squirting.</p>\";s:7:\"pubdate\";s:31:\"Fri, 25 Jul 2008 15:17:54 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:5:\"Lloyd\";}s:7:\"summary\";s:2463:\"<p><img class=\"alignright\" title=\"WordPress for iPhone Logo\" src=\"http://farm4.static.flickr.com/3272/2701598580_b3c2b4a169_o.jpg\" alt=\"\" width=\"100\" height=\"100\" />Earlier this week my <a href=\"http://automattic.com/\">Automattic</a> colleagues and <a href=\"http://www.effigent.com/\">Effigent</a> released <a href=\"http://iphone.wordpress.org/\">WordPress for iPhone</a> and iPod Touch. Now, the source is also available and Trac is set up for reporting bugs and <a href=\"http://iphone.trac.wordpress.org/\">participating in development</a>!</p>\n<p>I&#8217;ve tried it on my iPod Touch. They&#8217;ve done a great job! And are <a href=\"http://iphone.wordpress.org/2008/07/24/version-11-and-beyond/\">urgently working on fixing</a> the worse bugs.</p>\n<p>It&#8217;s already in the top 50 free apps.</p>\n<p><span id=\"more-1291\"></span>Development has been managed by <a href=\"http://raanan.com/\">Raanan Bar-Cohen</a> and the app is designed by <a href=\"http://iammattthomas.com/\">Matt Thomas</a>.</p>\n<p>WordPress for iPhone has the clean Matt aesthetic, though I expect the iPhone SDK probably doesn&#8217;t allow much deviation from clean simplicity.</p>\n<p>There is <a href=\"http://search.twitter.com/search?q=wordpress+nda\">quite some</a> <a href=\"http://www.tuaw.com/2008/07/24/wordpress-for-iphone-source-now-available/\">stink up</a> about whether or not we are breaking Apple&#8217;s iPhone development NDA by our application being open source, and speculation that Apple will sue us.</p>\n<p><span class=\"byline\"><a href=\"http://dynamic.boingboing.net/profile/John%20Brownlee\">John Brownlee</a> of BoingBoing Gadgets writes:</span></p>\n<blockquote><p>I was curious about the &#8220;Open Source&#8221; proclamation  at the top of its non-App-Store webpage.</p></blockquote>\n<p>WordPress for iPhone is in the App Store. WordPress for iPhone&#8217;s Application Description has always stated it&#8217;s open source. Someone at Apple approved &#8220;WordPress for iPhone Open Source&#8221;.</p>\n<p>Although it makes for a great story and gives us <a href=\"http://daringfireball.net/linked/2008/07/24/wordpress-iphone\">freetards reason to bang our chests</a>, my thought is Apple has it&#8217;s anti-open source clause in the agreement to cover their butts from their code being &#8220;poisoned&#8221; by a viral license. Butt covered, let the <a href=\"http://foolswisdom.com/open-source-in-iphone-app-store/\">open source applications flow</a>, but no squirting.</p>\";}i:40;a:7:{s:5:\"title\";s:24:\"Matt: WordPress Geniuses\";s:4:\"guid\";s:40:\"http://ma.tt/2008/07/wordpress-geniuses/\";s:4:\"link\";s:40:\"http://ma.tt/2008/07/wordpress-geniuses/\";s:11:\"description\";s:244:\"<p><a href=\"http://foolswisdom.com/geniuses-for-wordcamp-sf-bar/\">Geniuses for WordCamp SF Bar?</a>. Lloyd is organizing a &#8220;Genius bar&#8221; at WordCamp San Francisco of volunteers to help everybody with their WP problems. I love it!</p>\";s:7:\"pubdate\";s:31:\"Fri, 25 Jul 2008 14:50:59 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:244:\"<p><a href=\"http://foolswisdom.com/geniuses-for-wordcamp-sf-bar/\">Geniuses for WordCamp SF Bar?</a>. Lloyd is organizing a &#8220;Genius bar&#8221; at WordCamp San Francisco of volunteers to help everybody with their WP problems. I love it!</p>\";}i:41;a:7:{s:5:\"title\";s:24:\"Matt: Visiting Techstars\";s:4:\"guid\";s:20:\"http://ma.tt/?p=6735\";s:4:\"link\";s:40:\"http://ma.tt/2008/07/visiting-techstars/\";s:11:\"description\";s:9976:\"<p>Visiting the Techstars in Boulder, Colorado.</p>\n<p><span id=\"more-6735\"></span></p>\n\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4767/\" title=\"mcm_4767\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4767-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4768/\" title=\"mcm_4768\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4768-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4772/\" title=\"mcm_4772\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4772-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4773/\" title=\"mcm_4773\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4773-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4774/\" title=\"mcm_4774\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4774-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4775/\" title=\"mcm_4775\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4775-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4783/\" title=\"mcm_4783\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4783-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4784/\" title=\"mcm_4784\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4784-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4785/\" title=\"mcm_4785\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4785-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4789/\" title=\"mcm_4789\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4789-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4790/\" title=\"mcm_4790\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4790-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4791/\" title=\"mcm_4791\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4791-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4794/\" title=\"mcm_4794\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4794-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4799/\" title=\"mcm_4799\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4799-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4801/\" title=\"mcm_4801\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4801-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4805/\" title=\"mcm_4805\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4805-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4806/\" title=\"mcm_4806\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4806-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4808/\" title=\"mcm_4808\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4808-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4809/\" title=\"mcm_4809\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4809-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4812/\" title=\"mcm_4812\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4812-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4813/\" title=\"mcm_4813\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4813-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4815/\" title=\"mcm_4815\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4815-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4816/\" title=\"mcm_4816\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4816-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4820/\" title=\"mcm_4820\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4820-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4822/\" title=\"mcm_4822\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4822-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4826/\" title=\"mcm_4826\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4826-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4827/\" title=\"mcm_4827\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4827-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4833/\" title=\"mcm_4833\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4833-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4835/\" title=\"mcm_4835\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4835-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4836/\" title=\"mcm_4836\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4836-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4839/\" title=\"mcm_4839\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4839-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4841/\" title=\"mcm_4841\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4841-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4843/\" title=\"mcm_4843\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4843-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4845/\" title=\"mcm_4845\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4845-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4848/\" title=\"mcm_4848\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4848-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4849/\" title=\"mcm_4849\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4849-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4857/\" title=\"mcm_4857\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4857-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4862/\" title=\"mcm_4862\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4862-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4868/\" title=\"mcm_4868\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4868-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4874/\" title=\"mcm_4874\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4874-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4877/\" title=\"mcm_4877\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4877-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4880/\" title=\"mcm_4880\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4880-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4882/\" title=\"mcm_4882\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4882-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4887/\" title=\"mcm_4887\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4887-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4889/\" title=\"mcm_4889\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4889-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4890/\" title=\"mcm_4890\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4890-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4892/\" title=\"mcm_4892\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4892-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4896/\" title=\"mcm_4896\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4896-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\";s:7:\"pubdate\";s:31:\"Fri, 25 Jul 2008 07:00:17 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:9976:\"<p>Visiting the Techstars in Boulder, Colorado.</p>\n<p><span id=\"more-6735\"></span></p>\n\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4767/\" title=\"mcm_4767\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4767-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4768/\" title=\"mcm_4768\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4768-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4772/\" title=\"mcm_4772\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4772-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4773/\" title=\"mcm_4773\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4773-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4774/\" title=\"mcm_4774\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4774-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4775/\" title=\"mcm_4775\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4775-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4783/\" title=\"mcm_4783\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4783-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4784/\" title=\"mcm_4784\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4784-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4785/\" title=\"mcm_4785\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4785-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4789/\" title=\"mcm_4789\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4789-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4790/\" title=\"mcm_4790\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4790-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4791/\" title=\"mcm_4791\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4791-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4794/\" title=\"mcm_4794\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4794-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4799/\" title=\"mcm_4799\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4799-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4801/\" title=\"mcm_4801\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4801-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4805/\" title=\"mcm_4805\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4805-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4806/\" title=\"mcm_4806\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4806-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4808/\" title=\"mcm_4808\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4808-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4809/\" title=\"mcm_4809\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4809-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4812/\" title=\"mcm_4812\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4812-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4813/\" title=\"mcm_4813\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4813-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4815/\" title=\"mcm_4815\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4815-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4816/\" title=\"mcm_4816\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4816-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4820/\" title=\"mcm_4820\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4820-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4822/\" title=\"mcm_4822\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4822-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4826/\" title=\"mcm_4826\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4826-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4827/\" title=\"mcm_4827\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4827-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4833/\" title=\"mcm_4833\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4833-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4835/\" title=\"mcm_4835\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4835-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4836/\" title=\"mcm_4836\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4836-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4839/\" title=\"mcm_4839\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4839-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4841/\" title=\"mcm_4841\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4841-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4843/\" title=\"mcm_4843\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4843-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4845/\" title=\"mcm_4845\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4845-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4848/\" title=\"mcm_4848\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4848-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4849/\" title=\"mcm_4849\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4849-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4857/\" title=\"mcm_4857\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4857-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4862/\" title=\"mcm_4862\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4862-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4868/\" title=\"mcm_4868\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4868-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4874/\" title=\"mcm_4874\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4874-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4877/\" title=\"mcm_4877\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4877-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4880/\" title=\"mcm_4880\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4880-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4882/\" title=\"mcm_4882\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4882-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4887/\" title=\"mcm_4887\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4887-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4889/\" title=\"mcm_4889\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4889-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4890/\" title=\"mcm_4890\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4890-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4892/\" title=\"mcm_4892\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4892-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/07/visiting-techstars/mcm_4896/\" title=\"mcm_4896\"><img src=\"http://s.ma.tt/files/2008/07/mcm_4896-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\";}i:42;a:7:{s:5:\"title\";s:24:\"Matt: 2.6 by the numbers\";s:4:\"guid\";s:20:\"http://ma.tt/?p=6733\";s:4:\"link\";s:39:\"http://ma.tt/2008/07/26-by-the-numbers/\";s:11:\"description\";s:1432:\"<p>Now that we&#8217;re now 10 days into the release of <a href=\"http://wordpress.org/development/2008/07/wordpress-26-tyner/\">version 2.6 of WordPress</a>, it&#8217;d be interesting to look at a few of the numbers around it.</p>\n<ul>\n<li>There have been around 23 thousand downloads per day. (Of just the English version.)</li>\n<li>According to the update system there are 201 thousand blogs using 2.6 already.</li>\n<li>That&#8217;s about 9% of all known WordPress.org blogs in 10 days.</li>\n<li>The video in the announcement post has been viewed 665,080 times.</li>\n<li>There have been over 300 themes submitted to the new <a href=\"http://wordpress.org/extend/themes/\">Theme directory</a>, which <a href=\"http://wordpress.org/development/2008/07/theme-directory/\">launched</a> just 6 days ago.</li>\n<li>In the same period (10 days) there were 579,871 downloads of 2,527 plugins.</li>\n</ul>\n<p>I imagine 2.6 adoption will pick up after the 2.6.1 release &#8212; a lot of people wait for the .1 before upgrading.</p>\n<p>How are we celebrating? <strong>By working on 2.7!</strong></p>\n<p>It should be a fun release both for the features we have planned and also because it might incorporate some of the aspects of Crazyhorse, our experimental bizarro world <a href=\"http://svn.automattic.com/wordpress/branches/crazyhorse/\">dev branch</a> which we&#8217;re laser-eye-testing in NYC next week. (700 blogs are running 2.7 already.)</p>\";s:7:\"pubdate\";s:31:\"Fri, 25 Jul 2008 05:34:17 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:1432:\"<p>Now that we&#8217;re now 10 days into the release of <a href=\"http://wordpress.org/development/2008/07/wordpress-26-tyner/\">version 2.6 of WordPress</a>, it&#8217;d be interesting to look at a few of the numbers around it.</p>\n<ul>\n<li>There have been around 23 thousand downloads per day. (Of just the English version.)</li>\n<li>According to the update system there are 201 thousand blogs using 2.6 already.</li>\n<li>That&#8217;s about 9% of all known WordPress.org blogs in 10 days.</li>\n<li>The video in the announcement post has been viewed 665,080 times.</li>\n<li>There have been over 300 themes submitted to the new <a href=\"http://wordpress.org/extend/themes/\">Theme directory</a>, which <a href=\"http://wordpress.org/development/2008/07/theme-directory/\">launched</a> just 6 days ago.</li>\n<li>In the same period (10 days) there were 579,871 downloads of 2,527 plugins.</li>\n</ul>\n<p>I imagine 2.6 adoption will pick up after the 2.6.1 release &#8212; a lot of people wait for the .1 before upgrading.</p>\n<p>How are we celebrating? <strong>By working on 2.7!</strong></p>\n<p>It should be a fun release both for the features we have planned and also because it might incorporate some of the aspects of Crazyhorse, our experimental bizarro world <a href=\"http://svn.automattic.com/wordpress/branches/crazyhorse/\">dev branch</a> which we&#8217;re laser-eye-testing in NYC next week. (700 blogs are running 2.7 already.)</p>\";}i:43;a:7:{s:5:\"title\";s:57:\"Weblog Tools Collection: Slickdeals.net Blog on WordPress\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3775\";s:4:\"link\";s:85:\"http://weblogtoolscollection.com/archives/2008/07/24/slickdealsnet-blog-on-wordpress/\";s:11:\"description\";s:462:\"<p><a href=\"http://blog.slickdeals.net/\">Slickdeals.net Blog</a>: Slickdeals recently started a new &#8220;Slickblog&#8221; with WordPress. They seem to have put together a pretty seamless integeration between WordPress and their highly complex vBulletin installation and the comments on their blog are redirected to the main forums. I really like their sidebar items, including their hot topics by ratings and up and coming topics by the number of comments.</p>\";s:7:\"pubdate\";s:31:\"Fri, 25 Jul 2008 01:21:42 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:462:\"<p><a href=\"http://blog.slickdeals.net/\">Slickdeals.net Blog</a>: Slickdeals recently started a new &#8220;Slickblog&#8221; with WordPress. They seem to have put together a pretty seamless integeration between WordPress and their highly complex vBulletin installation and the comments on their blog are redirected to the main forums. I really like their sidebar items, including their hot topics by ratings and up and coming topics by the number of comments.</p>\";}i:44;a:7:{s:5:\"title\";s:30:\"Matt: Caloric Restriction Pill\";s:4:\"guid\";s:46:\"http://ma.tt/2008/07/caloric-restriction-pill/\";s:4:\"link\";s:46:\"http://ma.tt/2008/07/caloric-restriction-pill/\";s:11:\"description\";s:121:\"<p><a href=\"http://blog.wired.com/wiredscience/2008/07/caloric-restric.html\">Caloric Restriction Comes in a Pill</a>.</p>\";s:7:\"pubdate\";s:31:\"Thu, 24 Jul 2008 21:24:59 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:121:\"<p><a href=\"http://blog.wired.com/wiredscience/2008/07/caloric-restric.html\">Caloric Restriction Comes in a Pill</a>.</p>\";}i:45;a:7:{s:5:\"title\";s:56:\"Weblog Tools Collection: WordPress Plugin Releases 07/24\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3774\";s:4:\"link\";s:84:\"http://weblogtoolscollection.com/archives/2008/07/24/wordpress-plugin-releases-0724/\";s:11:\"description\";s:3627:\"<p><a href=\"http://beyn.org/sidecat/\"><b>Category Selector Back to the Sidebar</b></a></p>\n<p>A simple plugin which moves the category selector back to the sidebar of the Post New page. In return, it removes the Related section (just because that I&#8217;m new in PHP and this is my first plugin), except the Press This bookmarklet.</p>\n<p><a href=\"http://scribu.net/projects/deviant-thumbs.html\"><b>Deviant Thumbs</b></a><b> </b></p>\n<p>This plugin displays thumbs from deviantART in a configurable carousel. It has widget support and is licensed under the GPL.</p>\n<p><a href=\"http://www.themoviequotes.com/tools/wordpress\"><b>Movie Quotes</b></a></p>\n<p>The Movie Quotes widget will fetch the latest, random or top rated quotes from themoviequotes.com website, and display them on your blog.</p>\n<p><a href=\"http://www.prelovac.com/vladimir/wordpress-plugins/snazzy-archives\"><b>Snazzy Archives</b></a></p>\n<p>A visualization plugin for your WordPress site featuring an unique way to display all your posts. Your archive page will never be boring again!</p>\n<p><a href=\"http://www.prelovac.com/vladimir/wordpress-plugins/show-me-options\"><b>Show Me Options</b></a></p>\n<p>How many times have you activated a plugin and the searched hopelessly for it’s options page? When you activate a new plugin, Show Me Options will show you a link to the options page you can use immediately.</p>\n<p><a href=\"http://wordpress.org/extend/plugins/zdmultilang/\"><b>ZdMultilang</b></a></p>\n<p>A Multilingual plugin for WordPress allowing you to blog in more than one language. You can define multiple languages and translate posts/tags/categories using the interface that will be added to your blog’s administration panel.</p>\n<p><a href=\"http://www.zen-dreams.com/en/zdstats/\"><b>ZdStats</b></a></p>\n<p>A WordPress plugin allowing you to trace your visitors. This is not a simple statistics plugin, it’s very flexible and dynamic. This means you can update your filters by yourself and therefore be as realistic as possible.</p>\n<p><a href=\"http://techblissonline.com/broken-links-remover-wordpress-plugin/\"><b>Broken Links Remover</b></a></p>\n<p>It helps you fix bad/broken links, both text links(anchor tags) and image links (img tags). It blocks the display of such links on your WordPress post/page, until you fix them. These are not made visible to search engine robots as well, until you correct them.</p>\n<p><a href=\"http://www.dkcell.com.br/design/2008/dk-obama-plugin/\"><b>Dk Obama</b></a></p>\n<p>Imitates the gallery as on Barack Obama&#8217;s site.</p>\n<p><a href=\"http://www.webmaster-source.com/wp125-ad-plugin-wordpress/\"><b>WP125</b></a></p>\n<p>The WP125 plugin can help you manage your ads more efficiently, leaving you with more time to write new posts. Ads are automatically taken down when their time is up, and optional click counting is available.</p>\n<p><a href=\"http://nwfloridarepublican.com/?page_id=77\"><b>McCain Widget</b></a></p>\n<p>Place a Blogs for McCain widget in your side bar.</p>\n<p><a href=\"http://calendarscripts.info/overweight-calculator-wordpress-plugin.html\"><b>Overweight Calculator</b></a></p>\n<p>This bmi calculator is intended to blogs who write about health, wellness, fitness, food and recipes and sport. The plugin helps your visitors calculate their body mass index and if they are overweight, to understand what is the weight range they should aim for.</p>\n<p><a href=\"http://sharkinvestor.com/investment-calculator-wordpress-plugin/\"><b>Investment Calculator</b></a></p>\n<p>This plugin will be most useful for personal finance, credit, loans, investment related or trading blogs.</p>\";s:7:\"pubdate\";s:31:\"Thu, 24 Jul 2008 18:52:30 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:3627:\"<p><a href=\"http://beyn.org/sidecat/\"><b>Category Selector Back to the Sidebar</b></a></p>\n<p>A simple plugin which moves the category selector back to the sidebar of the Post New page. In return, it removes the Related section (just because that I&#8217;m new in PHP and this is my first plugin), except the Press This bookmarklet.</p>\n<p><a href=\"http://scribu.net/projects/deviant-thumbs.html\"><b>Deviant Thumbs</b></a><b> </b></p>\n<p>This plugin displays thumbs from deviantART in a configurable carousel. It has widget support and is licensed under the GPL.</p>\n<p><a href=\"http://www.themoviequotes.com/tools/wordpress\"><b>Movie Quotes</b></a></p>\n<p>The Movie Quotes widget will fetch the latest, random or top rated quotes from themoviequotes.com website, and display them on your blog.</p>\n<p><a href=\"http://www.prelovac.com/vladimir/wordpress-plugins/snazzy-archives\"><b>Snazzy Archives</b></a></p>\n<p>A visualization plugin for your WordPress site featuring an unique way to display all your posts. Your archive page will never be boring again!</p>\n<p><a href=\"http://www.prelovac.com/vladimir/wordpress-plugins/show-me-options\"><b>Show Me Options</b></a></p>\n<p>How many times have you activated a plugin and the searched hopelessly for it’s options page? When you activate a new plugin, Show Me Options will show you a link to the options page you can use immediately.</p>\n<p><a href=\"http://wordpress.org/extend/plugins/zdmultilang/\"><b>ZdMultilang</b></a></p>\n<p>A Multilingual plugin for WordPress allowing you to blog in more than one language. You can define multiple languages and translate posts/tags/categories using the interface that will be added to your blog’s administration panel.</p>\n<p><a href=\"http://www.zen-dreams.com/en/zdstats/\"><b>ZdStats</b></a></p>\n<p>A WordPress plugin allowing you to trace your visitors. This is not a simple statistics plugin, it’s very flexible and dynamic. This means you can update your filters by yourself and therefore be as realistic as possible.</p>\n<p><a href=\"http://techblissonline.com/broken-links-remover-wordpress-plugin/\"><b>Broken Links Remover</b></a></p>\n<p>It helps you fix bad/broken links, both text links(anchor tags) and image links (img tags). It blocks the display of such links on your WordPress post/page, until you fix them. These are not made visible to search engine robots as well, until you correct them.</p>\n<p><a href=\"http://www.dkcell.com.br/design/2008/dk-obama-plugin/\"><b>Dk Obama</b></a></p>\n<p>Imitates the gallery as on Barack Obama&#8217;s site.</p>\n<p><a href=\"http://www.webmaster-source.com/wp125-ad-plugin-wordpress/\"><b>WP125</b></a></p>\n<p>The WP125 plugin can help you manage your ads more efficiently, leaving you with more time to write new posts. Ads are automatically taken down when their time is up, and optional click counting is available.</p>\n<p><a href=\"http://nwfloridarepublican.com/?page_id=77\"><b>McCain Widget</b></a></p>\n<p>Place a Blogs for McCain widget in your side bar.</p>\n<p><a href=\"http://calendarscripts.info/overweight-calculator-wordpress-plugin.html\"><b>Overweight Calculator</b></a></p>\n<p>This bmi calculator is intended to blogs who write about health, wellness, fitness, food and recipes and sport. The plugin helps your visitors calculate their body mass index and if they are overweight, to understand what is the weight range they should aim for.</p>\n<p><a href=\"http://sharkinvestor.com/investment-calculator-wordpress-plugin/\"><b>Investment Calculator</b></a></p>\n<p>This plugin will be most useful for personal finance, credit, loans, investment related or trading blogs.</p>\";}i:46;a:7:{s:5:\"title\";s:34:\"Lloyd: Not Invited to the Open Web\";s:4:\"guid\";s:30:\"http://foolswisdom.com/?p=1293\";s:4:\"link\";s:51:\"http://foolswisdom.com/not-invited-to-the-open-web/\";s:11:\"description\";s:550:\"<p>Some friends have asked if WordPress and Automattic will be participating in the <a href=\"http://openwebfoundation.org/\">Open Web Foundation</a>. It does look like a brilliant initiative, but as far as I know we weren&#8217;t invited. That&#8217;s really unfortunate, because I don&#8217;t think there could have been a greater gesture of co-operation in the blogging industry than David Recordon of Six Apart inviting us to be founding members.</p>\n<p>Don&#8217;t worry, it won&#8217;t stop us doing everything we can to support the open web.</p>\";s:7:\"pubdate\";s:31:\"Thu, 24 Jul 2008 18:43:35 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:5:\"Lloyd\";}s:7:\"summary\";s:550:\"<p>Some friends have asked if WordPress and Automattic will be participating in the <a href=\"http://openwebfoundation.org/\">Open Web Foundation</a>. It does look like a brilliant initiative, but as far as I know we weren&#8217;t invited. That&#8217;s really unfortunate, because I don&#8217;t think there could have been a greater gesture of co-operation in the blogging industry than David Recordon of Six Apart inviting us to be founding members.</p>\n<p>Don&#8217;t worry, it won&#8217;t stop us doing everything we can to support the open web.</p>\";}i:47;a:7:{s:5:\"title\";s:62:\"Mark Jaquith: How to write a solid and stable WordPress plugin\";s:4:\"guid\";s:39:\"http://markjaquith.wordpress.com/?p=155\";s:4:\"link\";s:93:\"http://markjaquith.wordpress.com/2008/07/23/how-to-write-a-solid-and-stable-wordpress-plugin/\";s:11:\"description\";s:4465:\"<div class=\"snap_preview\"><br /><p>The <a href=\"http://b5media.com/\">b5media</a> tech team was having a discussion today about what criteria we use when reviewing a WordPress plugin for possible inclusion on one of our sites or across our network.  It makes for a good list of what to do (or not do) when writing a WordPress plugin, something that might be generally useful to plugin authors.  These things won&#8217;t make your plugin <em>good</em> &#8212; they&#8217;ll just help make it secure and stable.</p>\n<h3>Don&#8217;t ignore security</h3>\n<p>WordPress plugins have unlimited power, which is a blessing (it makes WordPress very flexible), but it can be a curse if your plugin is not code with security in mind.  If things like <a href=\"http://markjaquith.wordpress.com/2006/06/02/wordpress-203-nonces/\">CSRF</a>, <a href=\"http://apeatling.wordpress.com/2008/06/25/prepare-dont-escape/\">the prepare() method</a> (SQL injection protection), XSS are not intimately known to you, it&#8217;s time for you to do some research.  When someone installs their plugin, they are potentially putting their site&#8217;s security in your hands.  Rise to the occasion.</p>\n<h3>Make proper use of the role/capability system</h3>\n<p>WordPress controls admin access using a roles/capabilities system.  Make sure your plugin is checking against an appropriate capability.  If it is the sort of thing that only &#8220;admins&#8221; should use, you should be checking against the <code>manage_options</code> capability.  If this is something that an author should be able to do, you might use the <code>publish_post</code> capability.  Do not check against roles.  Roles are only containers for capabilities, and you can&#8217;t assume that an &#8220;author&#8221; on one blog means the same thing on all of them.</p>\n<h3>Use current API functions</h3>\n<p>WordPress has a lot of legacy code and deprecated functions.  We try not to break stuff without good reason, so often times we&#8217;ll leave in compatibility for old API functions.  If your plugin is using these functions, it tells me that you&#8217;re not up on current WordPress development.  This means that your plugin might be more likely to break with a WordPress upgrade, and it suggests sloppiness that can lead to security issues.</p>\n<h3>Mind your performance</h3>\n<p>The number of queries that your plugin makes can affect its performance.  We don&#8217;t care how cool your plugin is &#8212; if it uses 10 MySQL queries on every page, it&#8217;s not going in.  You should work to minimize these queries as much as possible.  Other performance issues like the efficiency of your PHP code should be taken into consideration as well.</p>\n<h3>Use existing WordPress data structures whenever possible</h3>\n<p>Don&#8217;t create your own tables unless you&#8217;re absolutely certain there is no way to accomplish your functionality within the WordPress data system.</p>\n<p>I&#8217;m going to go into more detail on these topics at a later date.  Maybe even with screencasts, which seem to be a great way of demonstrating these sorts of things.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/markjaquith.wordpress.com/155/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/markjaquith.wordpress.com/155/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/markjaquith.wordpress.com/155/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/markjaquith.wordpress.com/155/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/markjaquith.wordpress.com/155/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/markjaquith.wordpress.com/155/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/markjaquith.wordpress.com/155/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=markjaquith.wordpress.com&blog=316&post=155&subd=markjaquith&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Thu, 24 Jul 2008 04:00:44 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Mark Jaquith\";}s:7:\"summary\";s:4465:\"<div class=\"snap_preview\"><br /><p>The <a href=\"http://b5media.com/\">b5media</a> tech team was having a discussion today about what criteria we use when reviewing a WordPress plugin for possible inclusion on one of our sites or across our network.  It makes for a good list of what to do (or not do) when writing a WordPress plugin, something that might be generally useful to plugin authors.  These things won&#8217;t make your plugin <em>good</em> &#8212; they&#8217;ll just help make it secure and stable.</p>\n<h3>Don&#8217;t ignore security</h3>\n<p>WordPress plugins have unlimited power, which is a blessing (it makes WordPress very flexible), but it can be a curse if your plugin is not code with security in mind.  If things like <a href=\"http://markjaquith.wordpress.com/2006/06/02/wordpress-203-nonces/\">CSRF</a>, <a href=\"http://apeatling.wordpress.com/2008/06/25/prepare-dont-escape/\">the prepare() method</a> (SQL injection protection), XSS are not intimately known to you, it&#8217;s time for you to do some research.  When someone installs their plugin, they are potentially putting their site&#8217;s security in your hands.  Rise to the occasion.</p>\n<h3>Make proper use of the role/capability system</h3>\n<p>WordPress controls admin access using a roles/capabilities system.  Make sure your plugin is checking against an appropriate capability.  If it is the sort of thing that only &#8220;admins&#8221; should use, you should be checking against the <code>manage_options</code> capability.  If this is something that an author should be able to do, you might use the <code>publish_post</code> capability.  Do not check against roles.  Roles are only containers for capabilities, and you can&#8217;t assume that an &#8220;author&#8221; on one blog means the same thing on all of them.</p>\n<h3>Use current API functions</h3>\n<p>WordPress has a lot of legacy code and deprecated functions.  We try not to break stuff without good reason, so often times we&#8217;ll leave in compatibility for old API functions.  If your plugin is using these functions, it tells me that you&#8217;re not up on current WordPress development.  This means that your plugin might be more likely to break with a WordPress upgrade, and it suggests sloppiness that can lead to security issues.</p>\n<h3>Mind your performance</h3>\n<p>The number of queries that your plugin makes can affect its performance.  We don&#8217;t care how cool your plugin is &#8212; if it uses 10 MySQL queries on every page, it&#8217;s not going in.  You should work to minimize these queries as much as possible.  Other performance issues like the efficiency of your PHP code should be taken into consideration as well.</p>\n<h3>Use existing WordPress data structures whenever possible</h3>\n<p>Don&#8217;t create your own tables unless you&#8217;re absolutely certain there is no way to accomplish your functionality within the WordPress data system.</p>\n<p>I&#8217;m going to go into more detail on these topics at a later date.  Maybe even with screencasts, which seem to be a great way of demonstrating these sorts of things.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/markjaquith.wordpress.com/155/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/markjaquith.wordpress.com/155/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/markjaquith.wordpress.com/155/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/markjaquith.wordpress.com/155/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/markjaquith.wordpress.com/155/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/markjaquith.wordpress.com/155/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/markjaquith.wordpress.com/155/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/markjaquith.wordpress.com/155/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=markjaquith.wordpress.com&blog=316&post=155&subd=markjaquith&ref=&feed=1\" /></div>\";}i:48;a:7:{s:5:\"title\";s:55:\"Weblog Tools Collection: WordPress Theme Releases 07/23\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3773\";s:4:\"link\";s:83:\"http://weblogtoolscollection.com/archives/2008/07/23/wordpress-theme-releases-0723/\";s:11:\"description\";s:2761:\"<p><a href=\"http://www.rodrigoghedin.com.br/squared/\"><strong>Squared</strong></a></p>\n<p><img alt=\"squared-10\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/squared-10.gif\" height=\"125\" width=\"155\" /></p>\n<p>Squared is a very simple theme, two columns, widget ready, and valid XHTML 1.0</p>\n<p><a href=\"http://www.mubinahmed.com/wordpress-theme-directory-now-open/\"><strong>Simple</strong></a></p>\n<p><img alt=\"Simple\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/simple.png\" height=\"125\" width=\"167\" /></p>\n<p>A two column colorful theme</p>\n<p><a href=\"http://www.slimmity.com/ragteka-v1/\"><strong>Rateka</strong></a></p>\n<p><a href=\"http://demo.slimmity.com/index.php?wptheme=Ragteka+Premium\"><img alt=\"rateka\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/rateka.gif\" height=\"119\" width=\"200\" /></a></p>\n<p>Two column theme comes widget ready. Custom sidebar ad widgets. Also has 3 color schemes incorporated. Allows you to add subscription by email from admin panel. Easy to manage ads from php file.</p>\n<p><a href=\"http://fresh01.co.za/preview/index.php?wptheme=Scruffy\"><strong>Scruffy</strong></a></p>\n<p><a href=\"http://fresh01.co.za/preview/index.php?wptheme=Scruffy\"><img alt=\"Scruffy\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/scruffy.jpg\" height=\"118\" width=\"200\" /></a></p>\n<p>Scruffy is a WordPress theme with a standard two column layout and all the basic WP features. What makes this theme special is it&#8217;s unique &#8220;grunge&#8221; design.</p>\n<p><a href=\"http://www.teckitech.com/07/wordpress-theme-web-20-blue/\"><strong>Web 2.0 Blue</strong></a></p>\n<p><a href=\"http://ourfather.co.cc/index.php?wptheme=Web+2.0+Blue\"><img alt=\"web2-0blue\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/web2-0blue.jpg\" height=\"125\" width=\"175\" /></a></p>\n<p>Great web 2.0 looking, Great use of colors, Simple, Plain, Minimalist theme, Hemingway Style, Can be customized easily through detailed CSS. Support all plugins that Hemingway supports, Huge space for Widget lovers as it is 3 Column theme, Can customize images for blockquotes and so on&#8230;</p>\n<p><a href=\"http://www.fikirbozan.net/if-wordpress-theme/\"><strong>iF</strong></a></p>\n<p><img alt=\"iF\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/if.png\" height=\"116\" width=\"200\" /></p>\n<p>iF is a clean, simple theme with a left sidebar.</p>\n<p><a href=\"http://www.wpthemedesigner.com/2008/07/13/funky-zine-theme/\"><strong>FunkyZine</strong></a></p>\n<p><a href=\"http://ythv.info/funkyzine/\"><img alt=\"FunkyZine\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/funkyzine.jpg\" height=\"114\" width=\"200\" /></a></p>\n<p>2 Columns, Wood Design, Rounded Corners, Dark, Banner Ad Ready, Tested on WordPress 2.5, Fixed Width, PSD logo Attached.</p>\";s:7:\"pubdate\";s:31:\"Wed, 23 Jul 2008 20:45:07 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:2761:\"<p><a href=\"http://www.rodrigoghedin.com.br/squared/\"><strong>Squared</strong></a></p>\n<p><img alt=\"squared-10\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/squared-10.gif\" height=\"125\" width=\"155\" /></p>\n<p>Squared is a very simple theme, two columns, widget ready, and valid XHTML 1.0</p>\n<p><a href=\"http://www.mubinahmed.com/wordpress-theme-directory-now-open/\"><strong>Simple</strong></a></p>\n<p><img alt=\"Simple\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/simple.png\" height=\"125\" width=\"167\" /></p>\n<p>A two column colorful theme</p>\n<p><a href=\"http://www.slimmity.com/ragteka-v1/\"><strong>Rateka</strong></a></p>\n<p><a href=\"http://demo.slimmity.com/index.php?wptheme=Ragteka+Premium\"><img alt=\"rateka\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/rateka.gif\" height=\"119\" width=\"200\" /></a></p>\n<p>Two column theme comes widget ready. Custom sidebar ad widgets. Also has 3 color schemes incorporated. Allows you to add subscription by email from admin panel. Easy to manage ads from php file.</p>\n<p><a href=\"http://fresh01.co.za/preview/index.php?wptheme=Scruffy\"><strong>Scruffy</strong></a></p>\n<p><a href=\"http://fresh01.co.za/preview/index.php?wptheme=Scruffy\"><img alt=\"Scruffy\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/scruffy.jpg\" height=\"118\" width=\"200\" /></a></p>\n<p>Scruffy is a WordPress theme with a standard two column layout and all the basic WP features. What makes this theme special is it&#8217;s unique &#8220;grunge&#8221; design.</p>\n<p><a href=\"http://www.teckitech.com/07/wordpress-theme-web-20-blue/\"><strong>Web 2.0 Blue</strong></a></p>\n<p><a href=\"http://ourfather.co.cc/index.php?wptheme=Web+2.0+Blue\"><img alt=\"web2-0blue\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/web2-0blue.jpg\" height=\"125\" width=\"175\" /></a></p>\n<p>Great web 2.0 looking, Great use of colors, Simple, Plain, Minimalist theme, Hemingway Style, Can be customized easily through detailed CSS. Support all plugins that Hemingway supports, Huge space for Widget lovers as it is 3 Column theme, Can customize images for blockquotes and so on&#8230;</p>\n<p><a href=\"http://www.fikirbozan.net/if-wordpress-theme/\"><strong>iF</strong></a></p>\n<p><img alt=\"iF\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/if.png\" height=\"116\" width=\"200\" /></p>\n<p>iF is a clean, simple theme with a left sidebar.</p>\n<p><a href=\"http://www.wpthemedesigner.com/2008/07/13/funky-zine-theme/\"><strong>FunkyZine</strong></a></p>\n<p><a href=\"http://ythv.info/funkyzine/\"><img alt=\"FunkyZine\" src=\"http://weblogtoolscollection.com/b2-img/2008/07/funkyzine.jpg\" height=\"114\" width=\"200\" /></a></p>\n<p>2 Columns, Wood Design, Rounded Corners, Dark, Banner Ad Ready, Tested on WordPress 2.5, Fixed Width, PSD logo Attached.</p>\";}i:49;a:7:{s:5:\"title\";s:27:\"Matt: WordPress for iPhone!\";s:4:\"guid\";s:42:\"http://ma.tt/2008/07/wordpress-for-iphone/\";s:4:\"link\";s:42:\"http://ma.tt/2008/07/wordpress-for-iphone/\";s:11:\"description\";s:306:\"<p><a href=\"http://iphone.wordpress.org/2008/07/22/wordpress-for-iphone-available-now/\">WordPress for iPhone Available Now</a>. :)! Reviews coming in: <a href=\"http://news.cnet.com/8301-17939_109-9996382-2.html?part=rss&amp;tag=feed&amp;subj=Webware\">WordPress app hits the iPhone fashionably late</a>.</p>\";s:7:\"pubdate\";s:31:\"Tue, 22 Jul 2008 19:30:34 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:306:\"<p><a href=\"http://iphone.wordpress.org/2008/07/22/wordpress-for-iphone-available-now/\">WordPress for iPhone Available Now</a>. :)! Reviews coming in: <a href=\"http://news.cnet.com/8301-17939_109-9996382-2.html?part=rss&amp;tag=feed&amp;subj=Webware\">WordPress app hits the iPhone fashionably late</a>.</p>\";}}s:7:\"channel\";a:5:{s:5:\"title\";s:16:\"WordPress Planet\";s:4:\"link\";s:28:\"http://planet.wordpress.org/\";s:8:\"language\";s:2:\"en\";s:11:\"description\";s:47:\"WordPress Planet - http://planet.wordpress.org/\";s:7:\"tagline\";s:47:\"WordPress Planet - http://planet.wordpress.org/\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:4:\"etag\";s:26:\"\"1f846-4898596d-1174596\"\r\n\";s:13:\"last_modified\";s:31:\"Tue, 05 Aug 2008 13:45:17 GMT\r\n\";}','no'),(88,0,'rss_867bd5c64f85878d03a060509cd2f92c_ts','1217943919','no'),(89,0,'rss_a5420c83891a9c88ad2a4f04584a5efc','O:9:\"magpierss\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:15:{i:0;a:7:{s:5:\"title\";s:33:\"uberdose on \"All in One SEO Pack\"\";s:4:\"link\";s:65:\"http://wordpress.org/extend/plugins/all-in-one-seo-pack/#post-753\";s:7:\"pubdate\";s:31:\"Fri, 30 Mar 2007 20:08:18 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"uberdose\";}s:4:\"guid\";s:40:\"753@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:92:\"Automatically optimizes your Wordpress blog for Search Engines (Search Engine Optimization).\";s:7:\"summary\";s:92:\"Automatically optimizes your Wordpress blog for Search Engines (Search Engine Optimization).\";}i:1;a:7:{s:5:\"title\";s:29:\"andy on \"WordPress.com Stats\"\";s:4:\"link\";s:52:\"http://wordpress.org/extend/plugins/stats/#post-1355\";s:7:\"pubdate\";s:31:\"Sun, 06 May 2007 02:15:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"andy\";}s:4:\"guid\";s:41:\"1355@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:119:\"You can have simple, concise stats with no additional load on your server by plugging into WordPress.com\'s stat system.\";s:7:\"summary\";s:119:\"You can have simple, concise stats with no additional load on your server by plugging into WordPress.com\'s stat system.\";}i:2;a:7:{s:5:\"title\";s:36:\"olivers on \"cformsII - contact form\"\";s:4:\"link\";s:52:\"http://wordpress.org/extend/plugins/cforms/#post-925\";s:7:\"pubdate\";s:31:\"Mon, 09 Apr 2007 22:12:26 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"olivers\";}s:4:\"guid\";s:40:\"925@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:119:\"cforms II is the most customizable, flexible &#038; powerful ajax supporting contact form plugin (&#038; comment form)!\";s:7:\"summary\";s:119:\"cforms II is the most customizable, flexible &#038; powerful ajax supporting contact form plugin (&#038; comment form)!\";}i:3;a:7:{s:5:\"title\";s:17:\"matt on \"Akismet\"\";s:4:\"link\";s:52:\"http://wordpress.org/extend/plugins/akismet/#post-15\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 22:11:30 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"matt\";}s:4:\"guid\";s:39:\"15@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:98:\"Akismet checks your comments against the Akismet web service to see if they look like spam or not.\";s:7:\"summary\";s:98:\"Akismet checks your comments against the Akismet web service to see if they look like spam or not.\";}i:4;a:7:{s:5:\"title\";s:30:\"arnee on \"Google XML Sitemaps\"\";s:4:\"link\";s:70:\"http://wordpress.org/extend/plugins/google-sitemap-generator/#post-132\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 22:31:32 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:5:\"arnee\";}s:4:\"guid\";s:40:\"132@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:87:\"This plugin will create a Google sitemaps compliant XML-Sitemap of your WordPress blog.\";s:7:\"summary\";s:87:\"This plugin will create a Google sitemaps compliant XML-Sitemap of your WordPress blog.\";}i:5;a:7:{s:5:\"title\";s:44:\"keithdsouza on \"Wordpress Automatic upgrade\"\";s:4:\"link\";s:74:\"http://wordpress.org/extend/plugins/wordpress-automatic-upgrade/#post-2560\";s:7:\"pubdate\";s:31:\"Sat, 27 Oct 2007 20:55:05 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"keithdsouza\";}s:4:\"guid\";s:41:\"2560@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:112:\"Wordpress automatic upgrade allows a user to automatically upgrade the wordpress installation to the latest one.\";s:7:\"summary\";s:112:\"Wordpress automatic upgrade allows a user to automatically upgrade the wordpress installation to the latest one.\";}i:6;a:7:{s:5:\"title\";s:27:\"donncha on \"WP Super Cache\"\";s:4:\"link\";s:61:\"http://wordpress.org/extend/plugins/wp-super-cache/#post-2572\";s:7:\"pubdate\";s:31:\"Mon, 05 Nov 2007 11:40:04 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"donncha\";}s:4:\"guid\";s:41:\"2572@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:73:\"A very fast caching engine for WordPress that produces static html files.\";s:7:\"summary\";s:73:\"A very fast caching engine for WordPress that produces static html files.\";}i:7;a:7:{s:5:\"title\";s:29:\"alexrabe on \"NextGEN Gallery\"\";s:4:\"link\";s:62:\"http://wordpress.org/extend/plugins/nextgen-gallery/#post-1169\";s:7:\"pubdate\";s:31:\"Mon, 23 Apr 2007 20:08:06 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"alexrabe\";}s:4:\"guid\";s:41:\"1169@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:102:\"NextGEN Gallery is a full integrated Image Gallery plugin for WordPress with a Flash slideshow option.\";s:7:\"summary\";s:102:\"NextGEN Gallery is a full integrated Image Gallery plugin for WordPress with a Flash slideshow option.\";}i:8;a:7:{s:5:\"title\";s:26:\"alexkingorg on \"ShareThis\"\";s:4:\"link\";s:56:\"http://wordpress.org/extend/plugins/share-this/#post-274\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 23:05:42 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"alexkingorg\";}s:4:\"guid\";s:40:\"274@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"ShareThis plugin allows users to add your post to many social bookmarking sites, or to send your post link via email, AIM, Facebook, MySpace and more.\";s:7:\"summary\";s:150:\"ShareThis plugin allows users to add your post to many social bookmarking sites, or to send your post link via email, AIM, Facebook, MySpace and more.\";}i:9;a:7:{s:5:\"title\";s:23:\"GamerZ on \"WP-PageNavi\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/wp-pagenavi/#post-363\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 23:17:57 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"GamerZ\";}s:4:\"guid\";s:40:\"363@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:59:\"Adds a more advanced paging navigation your WordPress blog.\";s:7:\"summary\";s:59:\"Adds a more advanced paging navigation your WordPress blog.\";}i:10;a:7:{s:5:\"title\";s:20:\"GamerZ on \"WP-Polls\"\";s:4:\"link\";s:54:\"http://wordpress.org/extend/plugins/wp-polls/#post-368\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 23:18:28 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"GamerZ\";}s:4:\"guid\";s:40:\"368@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:117:\"Adds an AJAX poll system to your WordPress blog. You can also easily add a poll into your WordPress\'s blog post/page.\";s:7:\"summary\";s:117:\"Adds an AJAX poll system to your WordPress blog. You can also easily add a poll into your WordPress\'s blog post/page.\";}i:11;a:7:{s:5:\"title\";s:25:\"joostdevalk on \"Sociable\"\";s:4:\"link\";s:55:\"http://wordpress.org/extend/plugins/sociable/#post-2865\";s:7:\"pubdate\";s:31:\"Thu, 31 Jan 2008 11:36:17 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"joostdevalk\";}s:4:\"guid\";s:41:\"2865@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:74:\"Automatically add links on your posts to popular social bookmarking sites.\";s:7:\"summary\";s:74:\"Automatically add links on your posts to popular social bookmarking sites.\";}i:12;a:7:{s:5:\"title\";s:23:\"stimuli on \"Lightbox 2\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/lightbox-2/#post-2451\";s:7:\"pubdate\";s:31:\"Wed, 10 Oct 2007 02:20:04 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"stimuli\";}s:4:\"guid\";s:41:\"2451@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:146:\"Used to overlay images on the current page. Lightbox JS v2.2 by Lokesh Dhakar. Mad props to Tanin and Eric Strathmeyer for improvements and fixes!\";s:7:\"summary\";s:146:\"Used to overlay images on the current page. Lightbox JS v2.2 by Lokesh Dhakar. Mad props to Tanin and Eric Strathmeyer for improvements and fixes!\";}i:13;a:7:{s:5:\"title\";s:34:\"ronalfy on \"WP Ajax Edit Comments\"\";s:4:\"link\";s:68:\"http://wordpress.org/extend/plugins/wp-ajax-edit-comments/#post-1706\";s:7:\"pubdate\";s:31:\"Sun, 20 May 2007 06:15:08 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"ronalfy\";}s:4:\"guid\";s:41:\"1706@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:141:\"Allows users and admins to edit comments on a post. Users can edit their own comments for a limited time, while admins can edit all comments.\";s:7:\"summary\";s:141:\"Allows users and admins to edit comments on a post. Users can edit their own comments for a limited time, while admins can edit all comments.\";}i:14;a:7:{s:5:\"title\";s:28:\"freediver on \"Smart YouTube\"\";s:4:\"link\";s:60:\"http://wordpress.org/extend/plugins/smart-youtube/#post-2935\";s:7:\"pubdate\";s:31:\"Tue, 12 Feb 2008 12:05:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:9:\"freediver\";}s:4:\"guid\";s:41:\"2935@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:112:\"Smart Youtube plugin allows you to insert full featured YouTube videos into your post, comments and in RSS feed.\";s:7:\"summary\";s:112:\"Smart Youtube plugin allows you to insert full featured YouTube videos into your post, comments and in RSS feed.\";}}s:7:\"channel\";a:6:{s:5:\"title\";s:12:\"Most Popular\";s:4:\"link\";s:36:\"http://wordpress.org/extend/plugins/\";s:11:\"description\";s:12:\"Most Popular\";s:8:\"language\";s:2:\"en\";s:7:\"pubdate\";s:31:\"Tue, 05 Aug 2008 13:45:23 +0000\";s:7:\"tagline\";s:12:\"Most Popular\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:13:\"last_modified\";s:21:\"2007-03-30 20:08:18\r\n\";s:4:\"etag\";s:36:\"\"a8894f3af502f182fd4e92b6405bade8\"\r\n\";}','no'),(90,0,'rss_a5420c83891a9c88ad2a4f04584a5efc_ts','1217943920','no'),(91,0,'rss_0ff4b43bd116a9d8720d689c80e7dfd4','O:9:\"magpierss\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:10:{i:0;a:12:{s:5:\"title\";s:15:\"Theme Directory\";s:4:\"link\";s:57:\"http://wordpress.org/development/2008/07/theme-directory/\";s:8:\"comments\";s:66:\"http://wordpress.org/development/2008/07/theme-directory/#comments\";s:7:\"pubdate\";s:31:\"Fri, 18 Jul 2008 06:11:13 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Joseph Scott\";}s:8:\"category\";s:10:\"Metathemes\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=266\";s:11:\"description\";s:353:\"It&#8217;s been a long time since themes.wordpress.net stopped accepting new themes.  Since then most theme authors have been distributing their themes from their own sites, without a good centralized place for people to browse, search, comment on, and rate themes.  With the success of the plugins directory, we&#8217;ve wanted to have those same [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:2453:\"<p>It&#8217;s been a long time since <a href=\"http://themes.wordpress.net/blog/4426/upcoming-changes/\">themes.wordpress.net</a> stopped accepting new themes.  Since then most theme authors have been distributing their themes from their own sites, without a good centralized place for people to browse, search, comment on, and rate themes.  With the success of the <a href=\"http://wordpress.org/extend/plugins/\">plugins directory</a>, we&#8217;ve wanted to have those same benefits in a theme directory.  Today is the day we start making that happen, with the introduction of <a href=\"http://wordpress.org/extend/themes/\">wordpress.org/extend/themes/</a>.</p>\n<p>Bringing the new theme directory under the WordPress &#8220;<a href=\"http://wordpress.org/extend/\">extend</a>&#8221; umbrella allowed us to take advantage of all the infrastructure that has already been built up to support WordPress.org.  If you&#8217;ve browsed through the plugin directory, you&#8217;ll feel right at home in the new theme directory.</p>\n<p>We&#8217;ve gone through great lengths to make this as painless as possible for theme authors.  You don&#8217;t need to know anything about Subversion (our back end magic takes care of all that for you), just login with your WordPress.org username and password and go to the <a href=\"http://wordpress.org/extend/themes/upload/\">upload page</a>.  From there you upload your regular theme zip file and we take care of the rest.</p>\n<p>Once you upload your new theme we do a few automated checks for some of the <a href=\"http://wordpress.org/extend/themes/about/#requirements\">requirements</a> for each theme.  If we find one that you missed we&#8217;ll provide you an error and description of what needs to be fixed.  When a theme upload has been accepted we&#8217;ll send you an email and put it in the queue to be reviewed, to make sure we didn&#8217;t miss anything.  After the theme has been approved you&#8217;ll get another email letting you know that the theme is now live.</p>\n<p>That catches you up to where we&#8217;re at today.  When you finish that theme you&#8217;ve been slaving over, upload it to the new directory and let us know what you think. Since so much has changed since the old theme directory we&#8217;re starting fresh from zero. If you&#8217;ve got specific questions or suggestions <a href=\"http://wordpress.org/extend/themes/contact/\">contact us</a> and we&#8217;ll do our best to get them answered.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:62:\"http://wordpress.org/development/2008/07/theme-directory/feed/\";}s:7:\"summary\";s:353:\"It&#8217;s been a long time since themes.wordpress.net stopped accepting new themes.  Since then most theme authors have been distributing their themes from their own sites, without a good centralized place for people to browse, search, comment on, and rate themes.  With the success of the plugins directory, we&#8217;ve wanted to have those same [...]\";s:12:\"atom_content\";s:2453:\"<p>It&#8217;s been a long time since <a href=\"http://themes.wordpress.net/blog/4426/upcoming-changes/\">themes.wordpress.net</a> stopped accepting new themes.  Since then most theme authors have been distributing their themes from their own sites, without a good centralized place for people to browse, search, comment on, and rate themes.  With the success of the <a href=\"http://wordpress.org/extend/plugins/\">plugins directory</a>, we&#8217;ve wanted to have those same benefits in a theme directory.  Today is the day we start making that happen, with the introduction of <a href=\"http://wordpress.org/extend/themes/\">wordpress.org/extend/themes/</a>.</p>\n<p>Bringing the new theme directory under the WordPress &#8220;<a href=\"http://wordpress.org/extend/\">extend</a>&#8221; umbrella allowed us to take advantage of all the infrastructure that has already been built up to support WordPress.org.  If you&#8217;ve browsed through the plugin directory, you&#8217;ll feel right at home in the new theme directory.</p>\n<p>We&#8217;ve gone through great lengths to make this as painless as possible for theme authors.  You don&#8217;t need to know anything about Subversion (our back end magic takes care of all that for you), just login with your WordPress.org username and password and go to the <a href=\"http://wordpress.org/extend/themes/upload/\">upload page</a>.  From there you upload your regular theme zip file and we take care of the rest.</p>\n<p>Once you upload your new theme we do a few automated checks for some of the <a href=\"http://wordpress.org/extend/themes/about/#requirements\">requirements</a> for each theme.  If we find one that you missed we&#8217;ll provide you an error and description of what needs to be fixed.  When a theme upload has been accepted we&#8217;ll send you an email and put it in the queue to be reviewed, to make sure we didn&#8217;t miss anything.  After the theme has been approved you&#8217;ll get another email letting you know that the theme is now live.</p>\n<p>That catches you up to where we&#8217;re at today.  When you finish that theme you&#8217;ve been slaving over, upload it to the new directory and let us know what you think. Since so much has changed since the old theme directory we&#8217;re starting fresh from zero. If you&#8217;ve got specific questions or suggestions <a href=\"http://wordpress.org/extend/themes/contact/\">contact us</a> and we&#8217;ll do our best to get them answered.</p>\n\";}i:1;a:12:{s:5:\"title\";s:13:\"WordPress 2.6\";s:4:\"link\";s:60:\"http://wordpress.org/development/2008/07/wordpress-26-tyner/\";s:8:\"comments\";s:69:\"http://wordpress.org/development/2008/07/wordpress-26-tyner/#comments\";s:7:\"pubdate\";s:31:\"Tue, 15 Jul 2008 03:38:42 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:8:\"Releases\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=246\";s:11:\"description\";s:340:\" I&#8217;m happy to announce that version 2.6 of WordPress.org is now available, almost a month ahead schedule. Version 2.6 &#8220;Tyner,&#8221; named for jazz pianist McCoy Tyner, contains a number of new features that make WordPress a more powerful CMS: you can now track changes to every post and page and easily post from wherever [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:10270:\"<p><iframe src=\'http://digg.com/api/diggthis.php?u=http%3A%2F%2Fdigg.com%2Fsoftware%2FWordPress_2_6_Tyner_Released\' height=\'82\' width=\'55\' frameborder=\'0\' scrolling=\'no\' style=\'float: right; margin-left: 10px; margin-bottom: 5px; padding: 4px 0 2px 4px; background: #fff;\'></iframe> I&#8217;m happy to announce that version 2.6 of WordPress.org <a href=\"http://wordpress.org/download/\">is now available</a>, almost a month ahead schedule. Version 2.6 &#8220;Tyner,&#8221; named for jazz pianist <a href=\"http://en.wikipedia.org/wiki/McCoy_Tyner\">McCoy Tyner</a>, contains a number of new features that make WordPress a more powerful CMS: you can now track changes to every post and page and easily post from wherever you are on the web, plus there are dozens of incremental improvements to the features introduced in <a href=\"http://wordpress.org/development/2008/03/wordpress-25-brecker/\">version 2.5</a>.</p>\n<p>We&#8217;ve prepared a brief video tour of 2.6, if you have 3 minutes and 29 seconds to spare, it&#8217;s worth a watch:</p>\n<p><embed src=\"http://v.wordpress.com/mARhRBcT/fmt_dvd\" type=\"application/x-shockwave-flash\" width=\"640\" height=\"385\" flashvars=\"blog_domain=http://wordpress.org/development/2008/07/wordpress-26/&#038;width=640&#038;height=385\"> </embed></p>\n<p>If you&#8217;d like to embed the tour video in your blog, copy and paste this code for the high quality version:</p>\n<pre name=\"code\" class=\"html\">\n&lt;embed src=&quot;http://v.wordpress.com/mARhRBcT/fmt_dvd&quot; type=&quot;application/x-shockwave-flash&quot; width=&quot;640&quot; height=&quot;385&quot; flashvars=&quot;blog_domain=http://wordpress.org/development/2008/07/wordpress-26/&amp;width=640&amp;height=385&quot;&gt; &lt;/embed&gt;\n</pre>\n<p>And here&#8217;s a smaller version, 400 pixels wide:</p>\n<pre name=\"code\" class=\"html\">\n&lt;embed src=&quot;http://v.wordpress.com/mARhRBcT/fmt_std&quot; type=&quot;application/x-shockwave-flash&quot; width=&quot;400&quot; height=&quot;250&quot; flashvars=&quot;blog_domain=http://wordpress.org/development/2008/07/wordpress-26/&amp;width=400&amp;height=250&quot;&gt; &lt;/embed&gt;\n</pre>\n<p>Here&#8217;s a more textual overview of what&#8217;s hawt in 2.6:</p>\n<h3>Post Revisions: Wiki-like tracking of edits</h3>\n<p>With the power of modern computers, it&#8217;s silly that we still use save and editing metaphors from the time when the most common method of storage was floppy disks. WordPress has always respected the importance of your writing with auto-save, and now we&#8217;re taking that to another level by allowing you to view who made what changes when to any post or page through a super-easy interface, much like <a href=\"http://en.wikipedia.org/w/index.php?title=Wikipedia&#038;action=history\">Wikipedia</a> or a <a href=\"http://codex.wordpress.org/Using_Subversion\">version control system</a>.</p>\n<p><img src=\"http://wpdotorg.wordpress.com/files/2008/07/diff.gif\" alt=\"Differences between two versions of posts.\" class=\"alignnone size-full wp-image-3\" /></p>\n<p>This is handy on any blog in case you make a mistake and want to go back to an older version of a post, and it&#8217;s super handy for multi-author blogs where you can see every change tracked by person.</p>\n<h3>Press This!: Post from wherever you are on the web</h3>\n<p>A few months ago on my blog we <a href=\"http://ma.tt/2008/05/favorite-posting-bookmarklet/\">started a conversation about the posting bookmarklet</a> in WordPress and which systems we should look to for inspiration, like Flock, FriendFeed, Facebook, Tumblr, and Delicious. From these suggestions and the Quick Post plugin by Josh Kenzer, we developed a Press This bookmark you can add to your toolbar that provides a fast and smart popup to do posts to your WordPress blog:</p>\n<p><img src=\"http://wpdotorg.wordpress.com/files/2008/07/press-this.gif\" alt=\"Screenshot of Press This interface.\" title=\"press-this\" class=\"alignnone size-full wp-image-4\" /></p>\n<p>For example, if you click &#8220;Press This&#8221; from a <a href=\"http://youtube.com/\">Youtube</a> page it&#8217;ll magically extract the video embed code, and if you do it from a <a href=\"http://flickr.com/\">Flickr</a> page it&#8217;ll make it easy for you to put the image in your post. On my blog I&#8217;ve been experimenting with using different categories and the <code>in_category()</code> function &#8212; such as video, quote, aside, et cetera &#8212; to create a more tumblelog-like format.</p>\n<h3>Shift Gears: Turbo-speed your blogging</h3>\n<p><a href=\"http://gears.google.com/\">Gears</a> is an open source browser extension project started by Google that developers like us can use to give you features we wouldn&#8217;t normally be able to. There are a lot of things we can do with Gears in the future, but in this release we&#8217;ve stuck to using what&#8217;s called a &#8220;Local Server&#8221; to cache or keep a copy of commonly-used Javascript and CSS files on your computer, which can speed up the loading of some pages by several seconds (they just pop right up!). You can install Gears for Firefox or Internet Explorer, with support for Safari and Opera pending. WordPress works just fine without it, you just get a little extra juice when you have it installed.</p>\n<h3>Theme Previews: See it before your audience does</h3>\n<p>Now when you select a theme it pops up a window that shows the theme live with all your content, instead of immediately making it active on your site. This is great for just test driving themes before making a switch over publicly, and it is also helpful when you are developing a theme and need to test it but don&#8217;t want everybody to see your ongoing <del datetime=\"2008-07-14T21:23:41+00:00\">mistakes</del> development.</p>\n<p>Here are some of the smaller features and improvements in 2.6:</p>\n<ul>\n<li><strong>Word count!</strong> Never guess how many words are in your post anymore.</li>\n<li><strong>Image captions</strong>, so you can add sweet captions <a href=\"http://politicalticker.blogs.cnn.com/\">like Political Ticker does</a> under your images.</li>\n<li><strong>Bulk management</strong> of plugins.</li>\n<li>A completely <strong>revamped image control</strong> to allow for easier inserting, floating, and resizing. It&#8217;s now fully integrated with the WYSIWYG.</li>\n<li>Drag-and-drop <strong>reordering of Galleries</strong>.</li>\n<li><strong>Plugin update</strong> notification bubble.</li>\n<li>Customizable default avatars.</li>\n<li>You can now upload media when in full-screen mode.</li>\n<li>Remote publishing via XML-RPC and APP is now secure (off) by default, but you can turn it on easily through the options screen.</li>\n<li>Full <a href=\"http://en.wikipedia.org/wiki/Secure_Sockets_Layer\">SSL support</a> in the core, and the ability to <strong>force SSL</strong> for security.</li>\n<li>You can now have many thousands of pages or categories with no interface issues.</li>\n<li>Ability to move your <code>wp-config</code> file and <code>wp-content</code> directories to a custom location, for &#8220;clean&#8221; SVN checkouts.</li>\n<li>Select a range of checkboxes with &#8220;shift-click.&#8221;</li>\n<li>You can toggle between the Flash uploader and the classic one.</li>\n<li>A number of proactive security enhancements, including cookies and database interactions.</li>\n<li>Stronger better faster versions of TinyMCE, jQuery, and jQuery UI.</li>\n<li>Version 2.6 <a href=\"http://trac.wordpress.org/query?status=closed&#038;milestone=2.6\">fixes approximately 194 bugs</a>.</li>\n</ul>\n<h3>Developer Notes</h3>\n<p>WordPress.org had over 75 people contributing code to WordPress 2.6. In addition to the <a href=\"http://wordpress.org/about/\">core commit team</a> we had contributions from <a href=\"http://dd32.id.au/\">Dion Hulse</a>, <a href=\"http://www.ilfilosofo.com/\">Austin Matzko</a>, <a href=\"http://ottodestruct.com/blog/\">Otto42</a>, <a href=\"http://extralogical.net/\">Benedict Eastaugh</a>, and <a href=\"http://www.freecharity.org.uk/\">pishmishy</a>. <a href=\"http://xavisys.com/\">AaronCampbell</a> and <a href=\"http://www.marcozehe.de/\">Marco Zehe</a> provided more than a few patches. Back among the top code contributors is <a href=\"http://www.santosj.name/\">Jacob Santos</a>. <a href=\"http://www.buayacorp.com/\">Alex Concha</a> continues to have WordPress&#8217; back. Joining bug reporting and gardening elite are <a href=\"http://codex.wordpress.org/User:Hakre\">hakre</a>, <a href=\"http://www.simonwheatley.co.uk/\">Simon Wheatley</a>, mtekk, and <a href=\"http://www.prescriber.org.uk/\">Matty Rob</a>. Finally, congratulations to our <a href=\"http://westi.wordpress.com/\">Peter Westwood</a> on your recent wedding! I&#8217;m also proud to announce we&#8217;re adding a new core committer to the team: Andrew Ozz (azaozz) has been a huge help to the core team this year, particularly around TinyMCE and making the WYSIWYG something that works for you, not against you.</p>\n<p>Because of the new capabilities to make WordPress a clean SVN checkout, plugin and theme authors should do their best to handle forms and posts through WP rather than trying to post to their files directly, here&#8217;s a quick Codex article about <a href=\"http://codex.wordpress.org/Handling_POST_and_AJAX_Requests\">how to do it using our forward-compatible APIs</a>.</p>\n<h3>Upgrading</h3>\n<p>2.6 is pretty much identical to 2.5 from a plugin and theme compatibility point of view, so upgrades from 2.5 should be pretty painless. The 2.5 branch will no longer be maintain so everyone is encouraged to upgrade. <a href=\"http://codex.wordpress.org/Upgrading_WordPress\">Our standard 3-step upgrade instructions apply to this release</a>. There were at least 1,984,047 downloads of the 2.5 series, the fastest growing release we&#8217;ve ever had, and I think all of those people will find 2.6 adds a level of polish that really makes WP a pleasure to use every day. (At least I do. :))</p>\n<h3>Easter Egg</h3>\n<p>There have been rumors and allegations that there was a so-called &#8220;easter egg&#8221; added to 2.6 early in its development. These rumors and allegations are completely false!</p>\n<p>P.S. If you&#8217;re a fan of WordPress, <a href=\"http://www.facebook.com/pages/WordPress/6427302910\">consider joining our fan page on Facebook</a>.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:65:\"http://wordpress.org/development/2008/07/wordpress-26-tyner/feed/\";}s:7:\"summary\";s:340:\" I&#8217;m happy to announce that version 2.6 of WordPress.org is now available, almost a month ahead schedule. Version 2.6 &#8220;Tyner,&#8221; named for jazz pianist McCoy Tyner, contains a number of new features that make WordPress a more powerful CMS: you can now track changes to every post and page and easily post from wherever [...]\";s:12:\"atom_content\";s:10270:\"<p><iframe src=\'http://digg.com/api/diggthis.php?u=http%3A%2F%2Fdigg.com%2Fsoftware%2FWordPress_2_6_Tyner_Released\' height=\'82\' width=\'55\' frameborder=\'0\' scrolling=\'no\' style=\'float: right; margin-left: 10px; margin-bottom: 5px; padding: 4px 0 2px 4px; background: #fff;\'></iframe> I&#8217;m happy to announce that version 2.6 of WordPress.org <a href=\"http://wordpress.org/download/\">is now available</a>, almost a month ahead schedule. Version 2.6 &#8220;Tyner,&#8221; named for jazz pianist <a href=\"http://en.wikipedia.org/wiki/McCoy_Tyner\">McCoy Tyner</a>, contains a number of new features that make WordPress a more powerful CMS: you can now track changes to every post and page and easily post from wherever you are on the web, plus there are dozens of incremental improvements to the features introduced in <a href=\"http://wordpress.org/development/2008/03/wordpress-25-brecker/\">version 2.5</a>.</p>\n<p>We&#8217;ve prepared a brief video tour of 2.6, if you have 3 minutes and 29 seconds to spare, it&#8217;s worth a watch:</p>\n<p><embed src=\"http://v.wordpress.com/mARhRBcT/fmt_dvd\" type=\"application/x-shockwave-flash\" width=\"640\" height=\"385\" flashvars=\"blog_domain=http://wordpress.org/development/2008/07/wordpress-26/&#038;width=640&#038;height=385\"> </embed></p>\n<p>If you&#8217;d like to embed the tour video in your blog, copy and paste this code for the high quality version:</p>\n<pre name=\"code\" class=\"html\">\n&lt;embed src=&quot;http://v.wordpress.com/mARhRBcT/fmt_dvd&quot; type=&quot;application/x-shockwave-flash&quot; width=&quot;640&quot; height=&quot;385&quot; flashvars=&quot;blog_domain=http://wordpress.org/development/2008/07/wordpress-26/&amp;width=640&amp;height=385&quot;&gt; &lt;/embed&gt;\n</pre>\n<p>And here&#8217;s a smaller version, 400 pixels wide:</p>\n<pre name=\"code\" class=\"html\">\n&lt;embed src=&quot;http://v.wordpress.com/mARhRBcT/fmt_std&quot; type=&quot;application/x-shockwave-flash&quot; width=&quot;400&quot; height=&quot;250&quot; flashvars=&quot;blog_domain=http://wordpress.org/development/2008/07/wordpress-26/&amp;width=400&amp;height=250&quot;&gt; &lt;/embed&gt;\n</pre>\n<p>Here&#8217;s a more textual overview of what&#8217;s hawt in 2.6:</p>\n<h3>Post Revisions: Wiki-like tracking of edits</h3>\n<p>With the power of modern computers, it&#8217;s silly that we still use save and editing metaphors from the time when the most common method of storage was floppy disks. WordPress has always respected the importance of your writing with auto-save, and now we&#8217;re taking that to another level by allowing you to view who made what changes when to any post or page through a super-easy interface, much like <a href=\"http://en.wikipedia.org/w/index.php?title=Wikipedia&#038;action=history\">Wikipedia</a> or a <a href=\"http://codex.wordpress.org/Using_Subversion\">version control system</a>.</p>\n<p><img src=\"http://wpdotorg.wordpress.com/files/2008/07/diff.gif\" alt=\"Differences between two versions of posts.\" class=\"alignnone size-full wp-image-3\" /></p>\n<p>This is handy on any blog in case you make a mistake and want to go back to an older version of a post, and it&#8217;s super handy for multi-author blogs where you can see every change tracked by person.</p>\n<h3>Press This!: Post from wherever you are on the web</h3>\n<p>A few months ago on my blog we <a href=\"http://ma.tt/2008/05/favorite-posting-bookmarklet/\">started a conversation about the posting bookmarklet</a> in WordPress and which systems we should look to for inspiration, like Flock, FriendFeed, Facebook, Tumblr, and Delicious. From these suggestions and the Quick Post plugin by Josh Kenzer, we developed a Press This bookmark you can add to your toolbar that provides a fast and smart popup to do posts to your WordPress blog:</p>\n<p><img src=\"http://wpdotorg.wordpress.com/files/2008/07/press-this.gif\" alt=\"Screenshot of Press This interface.\" title=\"press-this\" class=\"alignnone size-full wp-image-4\" /></p>\n<p>For example, if you click &#8220;Press This&#8221; from a <a href=\"http://youtube.com/\">Youtube</a> page it&#8217;ll magically extract the video embed code, and if you do it from a <a href=\"http://flickr.com/\">Flickr</a> page it&#8217;ll make it easy for you to put the image in your post. On my blog I&#8217;ve been experimenting with using different categories and the <code>in_category()</code> function &#8212; such as video, quote, aside, et cetera &#8212; to create a more tumblelog-like format.</p>\n<h3>Shift Gears: Turbo-speed your blogging</h3>\n<p><a href=\"http://gears.google.com/\">Gears</a> is an open source browser extension project started by Google that developers like us can use to give you features we wouldn&#8217;t normally be able to. There are a lot of things we can do with Gears in the future, but in this release we&#8217;ve stuck to using what&#8217;s called a &#8220;Local Server&#8221; to cache or keep a copy of commonly-used Javascript and CSS files on your computer, which can speed up the loading of some pages by several seconds (they just pop right up!). You can install Gears for Firefox or Internet Explorer, with support for Safari and Opera pending. WordPress works just fine without it, you just get a little extra juice when you have it installed.</p>\n<h3>Theme Previews: See it before your audience does</h3>\n<p>Now when you select a theme it pops up a window that shows the theme live with all your content, instead of immediately making it active on your site. This is great for just test driving themes before making a switch over publicly, and it is also helpful when you are developing a theme and need to test it but don&#8217;t want everybody to see your ongoing <del datetime=\"2008-07-14T21:23:41+00:00\">mistakes</del> development.</p>\n<p>Here are some of the smaller features and improvements in 2.6:</p>\n<ul>\n<li><strong>Word count!</strong> Never guess how many words are in your post anymore.</li>\n<li><strong>Image captions</strong>, so you can add sweet captions <a href=\"http://politicalticker.blogs.cnn.com/\">like Political Ticker does</a> under your images.</li>\n<li><strong>Bulk management</strong> of plugins.</li>\n<li>A completely <strong>revamped image control</strong> to allow for easier inserting, floating, and resizing. It&#8217;s now fully integrated with the WYSIWYG.</li>\n<li>Drag-and-drop <strong>reordering of Galleries</strong>.</li>\n<li><strong>Plugin update</strong> notification bubble.</li>\n<li>Customizable default avatars.</li>\n<li>You can now upload media when in full-screen mode.</li>\n<li>Remote publishing via XML-RPC and APP is now secure (off) by default, but you can turn it on easily through the options screen.</li>\n<li>Full <a href=\"http://en.wikipedia.org/wiki/Secure_Sockets_Layer\">SSL support</a> in the core, and the ability to <strong>force SSL</strong> for security.</li>\n<li>You can now have many thousands of pages or categories with no interface issues.</li>\n<li>Ability to move your <code>wp-config</code> file and <code>wp-content</code> directories to a custom location, for &#8220;clean&#8221; SVN checkouts.</li>\n<li>Select a range of checkboxes with &#8220;shift-click.&#8221;</li>\n<li>You can toggle between the Flash uploader and the classic one.</li>\n<li>A number of proactive security enhancements, including cookies and database interactions.</li>\n<li>Stronger better faster versions of TinyMCE, jQuery, and jQuery UI.</li>\n<li>Version 2.6 <a href=\"http://trac.wordpress.org/query?status=closed&#038;milestone=2.6\">fixes approximately 194 bugs</a>.</li>\n</ul>\n<h3>Developer Notes</h3>\n<p>WordPress.org had over 75 people contributing code to WordPress 2.6. In addition to the <a href=\"http://wordpress.org/about/\">core commit team</a> we had contributions from <a href=\"http://dd32.id.au/\">Dion Hulse</a>, <a href=\"http://www.ilfilosofo.com/\">Austin Matzko</a>, <a href=\"http://ottodestruct.com/blog/\">Otto42</a>, <a href=\"http://extralogical.net/\">Benedict Eastaugh</a>, and <a href=\"http://www.freecharity.org.uk/\">pishmishy</a>. <a href=\"http://xavisys.com/\">AaronCampbell</a> and <a href=\"http://www.marcozehe.de/\">Marco Zehe</a> provided more than a few patches. Back among the top code contributors is <a href=\"http://www.santosj.name/\">Jacob Santos</a>. <a href=\"http://www.buayacorp.com/\">Alex Concha</a> continues to have WordPress&#8217; back. Joining bug reporting and gardening elite are <a href=\"http://codex.wordpress.org/User:Hakre\">hakre</a>, <a href=\"http://www.simonwheatley.co.uk/\">Simon Wheatley</a>, mtekk, and <a href=\"http://www.prescriber.org.uk/\">Matty Rob</a>. Finally, congratulations to our <a href=\"http://westi.wordpress.com/\">Peter Westwood</a> on your recent wedding! I&#8217;m also proud to announce we&#8217;re adding a new core committer to the team: Andrew Ozz (azaozz) has been a huge help to the core team this year, particularly around TinyMCE and making the WYSIWYG something that works for you, not against you.</p>\n<p>Because of the new capabilities to make WordPress a clean SVN checkout, plugin and theme authors should do their best to handle forms and posts through WP rather than trying to post to their files directly, here&#8217;s a quick Codex article about <a href=\"http://codex.wordpress.org/Handling_POST_and_AJAX_Requests\">how to do it using our forward-compatible APIs</a>.</p>\n<h3>Upgrading</h3>\n<p>2.6 is pretty much identical to 2.5 from a plugin and theme compatibility point of view, so upgrades from 2.5 should be pretty painless. The 2.5 branch will no longer be maintain so everyone is encouraged to upgrade. <a href=\"http://codex.wordpress.org/Upgrading_WordPress\">Our standard 3-step upgrade instructions apply to this release</a>. There were at least 1,984,047 downloads of the 2.5 series, the fastest growing release we&#8217;ve ever had, and I think all of those people will find 2.6 adds a level of polish that really makes WP a pleasure to use every day. (At least I do. :))</p>\n<h3>Easter Egg</h3>\n<p>There have been rumors and allegations that there was a so-called &#8220;easter egg&#8221; added to 2.6 early in its development. These rumors and allegations are completely false!</p>\n<p>P.S. If you&#8217;re a fan of WordPress, <a href=\"http://www.facebook.com/pages/WordPress/6427302910\">consider joining our fan page on Facebook</a>.</p>\n\";}i:2;a:12:{s:5:\"title\";s:24:\"WordPress Birthday Party\";s:4:\"link\";s:56:\"http://wordpress.org/development/2008/05/birthday-party/\";s:8:\"comments\";s:65:\"http://wordpress.org/development/2008/05/birthday-party/#comments\";s:7:\"pubdate\";s:31:\"Sun, 25 May 2008 05:51:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:6:\"Events\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=243\";s:11:\"description\";s:321:\"On Tuesday, May 27th, WordPress will turn 5 years old. We&#8217;ve come a long way from that original 0.7 release.\nTo celebrate we&#8217;re throwing a party in San Francisco at 111 Minna, starting at 9PM. You can get the full details and RSVP on Upcoming.org or on Facebook.\nI hope you see some of you there, should [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:874:\"<p>On Tuesday, May 27th, WordPress will turn 5 years old. We&#8217;ve come a long way from <a href=\"http://wordpress.org/development/2003/05/wordpress-now-available/\">that original 0.7 release</a>.</p>\n<p>To celebrate we&#8217;re throwing a party in San Francisco at 111 Minna, starting at 9PM. You can get the full details and RSVP <a href=\"http://upcoming.yahoo.com/event/714298\">on Upcoming.org</a> or <a href=\"http://www.facebook.com/event.php?eid=44552835108\">on Facebook</a>.</p>\n<p>I hope you see some of you there, should be a fun time.</p>\n<p>If you host a party in your area for WordPress&#8217; 5th, let us know and we&#8217;ll post it here.</p>\n<p><strong>Update:</strong> Party in Sydney! <a href=\"http://bethesignal.org/blog/2008/05/25/wordpress-party-sydney-style/\">Blog post</a>, <a href=\"http://www.facebook.com/event.php?eid=13815594694\">Facebook</a>.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:61:\"http://wordpress.org/development/2008/05/birthday-party/feed/\";}s:7:\"summary\";s:321:\"On Tuesday, May 27th, WordPress will turn 5 years old. We&#8217;ve come a long way from that original 0.7 release.\nTo celebrate we&#8217;re throwing a party in San Francisco at 111 Minna, starting at 9PM. You can get the full details and RSVP on Upcoming.org or on Facebook.\nI hope you see some of you there, should [...]\";s:12:\"atom_content\";s:874:\"<p>On Tuesday, May 27th, WordPress will turn 5 years old. We&#8217;ve come a long way from <a href=\"http://wordpress.org/development/2003/05/wordpress-now-available/\">that original 0.7 release</a>.</p>\n<p>To celebrate we&#8217;re throwing a party in San Francisco at 111 Minna, starting at 9PM. You can get the full details and RSVP <a href=\"http://upcoming.yahoo.com/event/714298\">on Upcoming.org</a> or <a href=\"http://www.facebook.com/event.php?eid=44552835108\">on Facebook</a>.</p>\n<p>I hope you see some of you there, should be a fun time.</p>\n<p>If you host a party in your area for WordPress&#8217; 5th, let us know and we&#8217;ll post it here.</p>\n<p><strong>Update:</strong> Party in Sydney! <a href=\"http://bethesignal.org/blog/2008/05/25/wordpress-party-sydney-style/\">Blog post</a>, <a href=\"http://www.facebook.com/event.php?eid=13815594694\">Facebook</a>.</p>\n\";}i:3;a:12:{s:5:\"title\";s:29:\"Usability Testing in New York\";s:4:\"link\";s:71:\"http://wordpress.org/development/2008/05/usability-testing-in-new-york/\";s:8:\"comments\";s:80:\"http://wordpress.org/development/2008/05/usability-testing-in-new-york/#comments\";s:7:\"pubdate\";s:31:\"Tue, 20 May 2008 22:13:49 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ryan\";}s:8:\"category\";s:11:\"Development\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=241\";s:11:\"description\";s:98:\"We&#8217;re doing some usability testing in New York City.  Join in if you&#8217;re in the area.\n\";s:7:\"content\";a:1:{s:7:\"encoded\";s:187:\"<p><a href=\"http://en.blog.wordpress.com/2008/05/20/new-york-usability-testing/\">We&#8217;re doing some usability testing in New York City</a>.  Join in if you&#8217;re in the area.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:76:\"http://wordpress.org/development/2008/05/usability-testing-in-new-york/feed/\";}s:7:\"summary\";s:98:\"We&#8217;re doing some usability testing in New York City.  Join in if you&#8217;re in the area.\n\";s:12:\"atom_content\";s:187:\"<p><a href=\"http://en.blog.wordpress.com/2008/05/20/new-york-usability-testing/\">We&#8217;re doing some usability testing in New York City</a>.  Join in if you&#8217;re in the area.</p>\n\";}i:4;a:12:{s:5:\"title\";s:18:\"Upcoming WordCamps\";s:4:\"link\";s:60:\"http://wordpress.org/development/2008/04/upcoming-wordcamps/\";s:8:\"comments\";s:69:\"http://wordpress.org/development/2008/04/upcoming-wordcamps/#comments\";s:7:\"pubdate\";s:31:\"Wed, 30 Apr 2008 09:54:48 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:6:\"Events\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=240\";s:11:\"description\";s:343:\"WordCamps are my favorite events to go to because there&#8217;s something about the core WordPress community that attracts smart folks with good philosophies that are fun to hang out with. In this post I&#8217;ve collated the upcoming WordCamps we know about, including the one in San Francisco. Hopefully there will be one nearby so you [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:1446:\"<p>WordCamps are my favorite events to go to because there&#8217;s something about the core WordPress community that attracts smart folks with good philosophies that are fun to hang out with. In this post I&#8217;ve collated the upcoming WordCamps we know about, including the one in San Francisco. Hopefully there will be one nearby so you can meet other WordPressers in your area.</p>\n<p><strong>WordCamp San Francisco</strong> will be August 16 at the Mission Bay Conference Center.</p>\n<p><a href=\"http://barcamp.org/WordCampParis\">WordCamp Paris will be on May 3rd</a>. <a href=\"http://wordcamp.fr/\">Here&#8217;s their official site</a>.</p>\n<p><a href=\"http://iwordpresscamp.wordpress.com/2008/02/20/european-wordcamp-beta/\">WordCamp Italy in Milan will be May 10th</a>. (And I believe I&#8217;ll be there.)</p>\n<p><a href=\"http://wiki.wordcampuk.tonyscott.org.uk/Main_Page\">WordCamp Birmingham UK</a> will be July 19-20.</p>\n<p><a href=\"http://wordcamptoronto.eventbrite.com/\">WordCamp Toronto will be October 4th</a>.</p>\n<p>There are people in the planning stages in <a href=\"http://australia.wordcamp.org/\">Australia</a>, Philippines, Beijing, Utah, Hawaii, UK, NYC, and possibly others, so if you live in one of those areas and would like to help set up a WordCamp in your area Google around or connect with bloggers in your area.</p>\n<p>You can always find out more at <a href=\"http://central.wordcamp.org/\">WordCamp Central</a>.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:65:\"http://wordpress.org/development/2008/04/upcoming-wordcamps/feed/\";}s:7:\"summary\";s:343:\"WordCamps are my favorite events to go to because there&#8217;s something about the core WordPress community that attracts smart folks with good philosophies that are fun to hang out with. In this post I&#8217;ve collated the upcoming WordCamps we know about, including the one in San Francisco. Hopefully there will be one nearby so you [...]\";s:12:\"atom_content\";s:1446:\"<p>WordCamps are my favorite events to go to because there&#8217;s something about the core WordPress community that attracts smart folks with good philosophies that are fun to hang out with. In this post I&#8217;ve collated the upcoming WordCamps we know about, including the one in San Francisco. Hopefully there will be one nearby so you can meet other WordPressers in your area.</p>\n<p><strong>WordCamp San Francisco</strong> will be August 16 at the Mission Bay Conference Center.</p>\n<p><a href=\"http://barcamp.org/WordCampParis\">WordCamp Paris will be on May 3rd</a>. <a href=\"http://wordcamp.fr/\">Here&#8217;s their official site</a>.</p>\n<p><a href=\"http://iwordpresscamp.wordpress.com/2008/02/20/european-wordcamp-beta/\">WordCamp Italy in Milan will be May 10th</a>. (And I believe I&#8217;ll be there.)</p>\n<p><a href=\"http://wiki.wordcampuk.tonyscott.org.uk/Main_Page\">WordCamp Birmingham UK</a> will be July 19-20.</p>\n<p><a href=\"http://wordcamptoronto.eventbrite.com/\">WordCamp Toronto will be October 4th</a>.</p>\n<p>There are people in the planning stages in <a href=\"http://australia.wordcamp.org/\">Australia</a>, Philippines, Beijing, Utah, Hawaii, UK, NYC, and possibly others, so if you live in one of those areas and would like to help set up a WordCamp in your area Google around or connect with bloggers in your area.</p>\n<p>You can always find out more at <a href=\"http://central.wordcamp.org/\">WordCamp Central</a>.</p>\n\";}i:5;a:12:{s:5:\"title\";s:15:\"WordPress 2.5.1\";s:4:\"link\";s:55:\"http://wordpress.org/development/2008/04/wordpress-251/\";s:8:\"comments\";s:64:\"http://wordpress.org/development/2008/04/wordpress-251/#comments\";s:7:\"pubdate\";s:31:\"Fri, 25 Apr 2008 15:54:19 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ryan\";}s:8:\"category\";s:8:\"Releases\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=231\";s:11:\"description\";s:360:\"Version 2.5.1 of WordPress is now available. It includes a number of bug fixes, performance enhancements, and one very important security fix. We recommend everyone update immediately, particularly if your blog has open registration. The vulnerability is not public but it will be shortly.\nIn addition to the security fix, 2.5.1 contains many bug fixes.  [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:2251:\"<p>Version 2.5.1 of WordPress is now available. It includes a number of bug fixes, performance enhancements, and one very important security fix. We recommend everyone update immediately, particularly if your blog has open registration. The vulnerability is not public but it will be shortly.</p>\n<p>In addition to the security fix, 2.5.1 contains many bug fixes.  If you are interested only in the security fixes, you can download these corrected copies of <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-includes/pluggable.php?format=raw\">wp-includes/pluggable.php</a>, <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-admin/includes/media.php?format=raw\">wp-admin/includes/media.php</a>, and <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-admin/media.php?format=raw\">wp-admin/media.php</a>.  Replace your existing copies of these files with these new copies.</p>\n<p>If you download the entire 2.5.1 release, you will be getting <a href=\"http://trac.wordpress.org/query?status=closed&amp;milestone=2.5.1&amp;resolution=fixed&amp;order=priority\">over 70 other fixes</a>.  2.5.1 focuses on fixing the most annoying bugs and improving performance.  Here are some highlights:</p>\n<ul>\n<li>Performance improvements for the Dashboard, Write Post, and Edit Comments pages.</li>\n<li>Better performance for those who have many categories</li>\n<li>Media Uploader fixes</li>\n<li>An upgrade to TinyMCE 3.0.7</li>\n<li>Widget Administration fixes</li>\n<li>Various usability improvements</li>\n<li>Layout fixes for IE</li>\n</ul>\n<h3>Secret lives of blogs</h3>\n<p>Since 2.5 your <code>wp-config.php</code> file allows a new constant called <code>SECRET_KEY</code> which basically is meant to introduce a little permanent randomness into the cryptographic functions used for cookies in WordPress. You can <a href=\"http://api.wordpress.org/secret-key/1.0/\">visit this link we set up to get a unique secret key</a> for your config file. (It&#8217;s unique and random on every page load.) Having this line in your config file helps secure your blog.</p>\n<p>Many thanks to <a href=\"http://www.cl.cam.ac.uk/~sjm217/\">Steven Murdoch</a> for responsibly reporting the security issue (CVE-2008-1930) and Alex Concha for reporting an XSS issue.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:60:\"http://wordpress.org/development/2008/04/wordpress-251/feed/\";}s:7:\"summary\";s:360:\"Version 2.5.1 of WordPress is now available. It includes a number of bug fixes, performance enhancements, and one very important security fix. We recommend everyone update immediately, particularly if your blog has open registration. The vulnerability is not public but it will be shortly.\nIn addition to the security fix, 2.5.1 contains many bug fixes.  [...]\";s:12:\"atom_content\";s:2251:\"<p>Version 2.5.1 of WordPress is now available. It includes a number of bug fixes, performance enhancements, and one very important security fix. We recommend everyone update immediately, particularly if your blog has open registration. The vulnerability is not public but it will be shortly.</p>\n<p>In addition to the security fix, 2.5.1 contains many bug fixes.  If you are interested only in the security fixes, you can download these corrected copies of <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-includes/pluggable.php?format=raw\">wp-includes/pluggable.php</a>, <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-admin/includes/media.php?format=raw\">wp-admin/includes/media.php</a>, and <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-admin/media.php?format=raw\">wp-admin/media.php</a>.  Replace your existing copies of these files with these new copies.</p>\n<p>If you download the entire 2.5.1 release, you will be getting <a href=\"http://trac.wordpress.org/query?status=closed&amp;milestone=2.5.1&amp;resolution=fixed&amp;order=priority\">over 70 other fixes</a>.  2.5.1 focuses on fixing the most annoying bugs and improving performance.  Here are some highlights:</p>\n<ul>\n<li>Performance improvements for the Dashboard, Write Post, and Edit Comments pages.</li>\n<li>Better performance for those who have many categories</li>\n<li>Media Uploader fixes</li>\n<li>An upgrade to TinyMCE 3.0.7</li>\n<li>Widget Administration fixes</li>\n<li>Various usability improvements</li>\n<li>Layout fixes for IE</li>\n</ul>\n<h3>Secret lives of blogs</h3>\n<p>Since 2.5 your <code>wp-config.php</code> file allows a new constant called <code>SECRET_KEY</code> which basically is meant to introduce a little permanent randomness into the cryptographic functions used for cookies in WordPress. You can <a href=\"http://api.wordpress.org/secret-key/1.0/\">visit this link we set up to get a unique secret key</a> for your config file. (It&#8217;s unique and random on every page load.) Having this line in your config file helps secure your blog.</p>\n<p>Many thanks to <a href=\"http://www.cl.cam.ac.uk/~sjm217/\">Steven Murdoch</a> for responsibly reporting the security issue (CVE-2008-1930) and Alex Concha for reporting an XSS issue.</p>\n\";}i:6;a:12:{s:5:\"title\";s:23:\"An Event Apart Discount\";s:4:\"link\";s:65:\"http://wordpress.org/development/2008/04/an-event-apart-discount/\";s:8:\"comments\";s:74:\"http://wordpress.org/development/2008/04/an-event-apart-discount/#comments\";s:7:\"pubdate\";s:31:\"Tue, 22 Apr 2008 22:33:19 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:36:\"Eventsan event aparthappy cogzeldman\";s:4:\"guid\";s:65:\"http://wordpress.org/development/2008/04/an-event-apart-discount/\";s:11:\"description\";s:321:\"An Event Apart is a web design and development conference which features some of the same fine folks who helped out with WordPress 2.5. (And many others.) I attended the one in Chicago a while back and was engaged the whole day in interesting talks on design, writing copy as interface, advanced CSS, and creativity [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:1194:\"<p><a href=\"http://aneventapart.com/\">An Event Apart</a> is a web design and development conference which features some of the same fine folks who helped out with WordPress 2.5. (And many others.) I attended the one in Chicago a while back and was engaged the whole day in interesting talks on design, writing copy as interface, advanced CSS, and creativity &#8212; each topic presented by the leading folks in the field.</p>\n<p>The conference normally costs just under a thousand dollars to attend, which is well worth it, but because of our association with the folks they&#8217;ve set up a discount for WordPress users.</p>\n<p>If you enter AEAWP on checkout the price drops to $795, or $200 below the regular registration fee. The coupon is unlimited, and can be used for one ticket or five. They have events coming up in New Orleans, Boston, San Francisco, and Chicago. <a href=\"http://aneventapart.com/\">You can learn more and register on their site at aneventapart.com</a>.</p>\n<p>We&#8217;ll also have some final dates for WordCamp San Francisco coming up, I&#8217;ll be posting those soonish so people can start making vacation and travel plans. (Nothing like a blogging vacation.)</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:70:\"http://wordpress.org/development/2008/04/an-event-apart-discount/feed/\";}s:7:\"summary\";s:321:\"An Event Apart is a web design and development conference which features some of the same fine folks who helped out with WordPress 2.5. (And many others.) I attended the one in Chicago a while back and was engaged the whole day in interesting talks on design, writing copy as interface, advanced CSS, and creativity [...]\";s:12:\"atom_content\";s:1194:\"<p><a href=\"http://aneventapart.com/\">An Event Apart</a> is a web design and development conference which features some of the same fine folks who helped out with WordPress 2.5. (And many others.) I attended the one in Chicago a while back and was engaged the whole day in interesting talks on design, writing copy as interface, advanced CSS, and creativity &#8212; each topic presented by the leading folks in the field.</p>\n<p>The conference normally costs just under a thousand dollars to attend, which is well worth it, but because of our association with the folks they&#8217;ve set up a discount for WordPress users.</p>\n<p>If you enter AEAWP on checkout the price drops to $795, or $200 below the regular registration fee. The coupon is unlimited, and can be used for one ticket or five. They have events coming up in New Orleans, Boston, San Francisco, and Chicago. <a href=\"http://aneventapart.com/\">You can learn more and register on their site at aneventapart.com</a>.</p>\n<p>We&#8217;ll also have some final dates for WordCamp San Francisco coming up, I&#8217;ll be posting those soonish so people can start making vacation and travel plans. (Nothing like a blogging vacation.)</p>\n\";}i:7;a:12:{s:5:\"title\";s:13:\"WordPress 2.5\";s:4:\"link\";s:62:\"http://wordpress.org/development/2008/03/wordpress-25-brecker/\";s:8:\"comments\";s:71:\"http://wordpress.org/development/2008/03/wordpress-25-brecker/#comments\";s:7:\"pubdate\";s:31:\"Sat, 29 Mar 2008 16:49:45 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:8:\"Releases\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=228\";s:11:\"description\";s:391:\"WordPress 2.5, the culmination of six months of work by the WordPress community, people just like you. The improvements in 2.5 are numerous, and almost entirely a result of your feedback: multi-file uploading, one-click plugin upgrades, built-in galleries, customizable dashboard, salted passwords and cookie encryption, media library, a WYSIWYG that doesn&#8217;t mess with your code, [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:15112:\"<p>WordPress 2.5, the culmination of six months of work by the WordPress community, people just like you. The improvements in 2.5 are numerous, and almost entirely a result of your feedback: multi-file uploading, one-click plugin upgrades, built-in galleries, customizable dashboard, salted passwords and cookie encryption, media library, a WYSIWYG that doesn&#8217;t mess with your code, concurrent post editing protection, full-screen writing, and search that covers posts <em>and</em> pages.</p>\n<p>For a short overview of the features with screenshots, it&#8217;d be best to <a href=\"http://wordpress.org/development/2008/03/25-sneak-peek/\">visit our sneak peek announcement for RC1</a>. Or check out <a href=\"http://wordpress.org/development/2008/03/wordpress-25-rc2/\">a 4-minute screencast of the new interface in action</a>. If you just want to jump straight to the good stuff <a href=\"http://wordpress.org/download/\">here&#8217;s where you can find 2.5 upgrade and download information</a>.</p>\n<p>If you want to see everything I would grab a cup of coffee or a mojito, because this post is epic.</p>\n<h3>User Features</h3>\n<p><strong>Cleaner, faster, less cluttered dashboard</strong> &#8212; we&#8217;ve worked hard to take your feedback about what&#8217;s most important in the dashboard and organize things to allow you to focus on what&#8217;s important &#8212; your blog &#8212; and get out of your way. In collaboration with <a href=\"http://happycog.com/\">Happy Cog</a> and the community  we&#8217;ve taken the first major step forward in the WordPress interface since version 1.5.</p>\n<p><strong>Dashboard Widgets</strong> &#8212; the dashboard home page is now a series of widgets, including ones to show you fun stats about your posting, latest comments, people linking to you, new and popular plugins, and of course WordPress news. You can customize any of the dashboard widgets to show, for example, news from your local paper instead of WP news. Plugins can also hook in, for example the <a href=\"http://wordpress.org/extend/plugins/stats/\">WordPress.com stats widget</a> adds a handy double-wide stats box.</p>\n<p><img src=\"http://s.wordpress.org/screenshots/2.5/stats-widget.png\" alt=\"\" width=\"690\" /></p>\n<p><strong>Multi-file upload with progress bar</strong> &#8212; before when you would upload a large file you&#8217;d wait forever, never knowing how far along it was. And uploading more than one photo was an exercise in patience, as you could only do one at a time. Now you can select a whole of folder images or music or videos at once and it&#8217;ll show you the progress of each upload.</p>\n<p><strong>Bonus: EXIF extraction</strong> &#8212; if you upload JPEG files with EXIF metadata like camera make and model, aperture, shutter speed, ISO, et al. WordPress will extract all the data into custom fields you can use in your template. If you use the EXIF title fields or similar those will be put into their equivalent fields in WP. Most modern digital cameras generate EXIF data.</p>\n<p><strong>Search posts and pages</strong> &#8212; search used to cover just posts, now it includes pages too, a great boon for those using WordPress as a CMS. New themes can style or sort pages differently in results.</p>\n<p><strong>Tag management</strong> &#8212; you can now add, rename, delete, and do whatever else you like to tags from inside WordPress, no plugins needed.</p>\n<p><strong>Password strength meter</strong> &#8212; when you change your password on your profile it&#8217;ll tell you how strong your password is to help you pick a good one.</p>\n<p><strong>Concurrent editing protection</strong> &#8212; for those of you on multi-author blogs, have you ever opened a post while someone was already editing it, and your auto-saves kept overwriting each other, irrecoverably losing hours of work? I bet that added a few words to your vocabulary. Now if you open a post that someone else is editing, WordPress magically locks it and prevents you from saving until the other person is done. You&#8217;ll see a message like below.</p>\n<p><img src=\"http://s.wordpress.org/screenshots/2.5/write-currently-editing.png\" alt=\"\" /></p>\n<p><strong>Few-click plugin upgrades</strong> &#8212; if the plugins you use are part of the plugin directory since 2.3 we&#8217;ve told you when they have an update available. Now we take that to the next logical step &#8212; downloading and installing the upgrade for you. This is dependent a little bit on your host setup, and it may ask you for your FTP password much like OS X or Windows will ask you for a password, but it works well on majority of hosts we were able to test, your mileage may very, plugins in mirror may be larger than they appear.</p>\n<p><strong>Friendlier visual post editor</strong> &#8212; I&#8217;m not sure how to articulate this improvement except to say &#8220;it doesn&#8217;t mess with your code anymore.&#8221; We&#8217;re now using version 3.0 of TinyMCE, which means better compatibility with Safari, and we&#8217;ve paid particular attention this release to its integration and interaction with complex HTML. It also now has a &#8220;no-distractions&#8221; mode which is like Writeroom for your browser.</p>\n<p><strong>Built-in galleries</strong> &#8212; when you take advantage of multi-file upload to upload a bunch of photos, we have a new shortcode that lets you to easily embed galleries by just putting [ gallery] (without the space) in your post. It&#8217;ll display all your thumbnails and captions and each will link each to a page where people can comment on the individual photos. I&#8217;ve been using this feature on my blog and have already uploaded over <a href=\"http://ma.tt/category/gallery/\">1,200 pictures into 23 galleries</a>. The shortcode has some hidden options too, <a href=\"http://codex.wordpress.org/Using_the_gallery_shortcode\">check out this documentation</a>.</p>\n<h3>Developer Features</h3>\n<p>Now for the geeky stuff. While we&#8217;re excited about the above features, each one represents a new opportunity or API for other developers to take to another level. (The best of which we&#8217;ll someday integrate back into WP.)</p>\n<p><strong>Salted passwords</strong> &#8212; we now use the <a href=\"http://www.openwall.com/phpass/\">phpass</a> library to stretch and salt all passwords stored in the database, which makes brute-forcing them impractical. If you use something like <a href=\"http://modauthmysql.sourceforge.net/\">mod_auth_mysql</a> we&#8217;ve <a href=\"http://wordpress.org/extend/plugins/md5-password-hashes/\">created a plugin that will allow you to use legacy MD5 hashing</a>. (The hashing is completely pluggable.) Users will automatically switch to the more secure passwords next time they log in.</p>\n<p><strong>Secure cookies</strong> &#8212; cookies are now encrypted based on the protocol described <a href=\"http://www.cse.msu.edu/~alexliu/publications/Cookie/cookie.pdf\">in this PDF paper</a>. which is something like <code>user name|expiration time|HMAC( user name|expiration time, k)</code> where <code>k = HMAC(user name|expiration time, sk)</code> and where <code>sk</code> is a secret key, which you can define in your config.</p>\n<p><strong>Easy taxonomy and URL creation</strong> &#8212; probably best illustrated with an example: I can call <code>register_taxonomy()</code> with <a href=\"http://trac.wordpress.org/attachment/ticket/6357/taxes.php\">a few arguments</a> to register a &#8220;people&#8221; taxonomy and whenever I edit an image I&#8217;ll see a UI like tags has for identifying the people in a photo, and these will be URL addressable with <code>/person/firstname-lastname/</code>. All with a single function call.</p>\n<p><strong>Inline documentation</strong> &#8212; the vast majority of the new code going into WordPress include inline documentation that explains the functions and documents their arguments.</p>\n<p><strong>Database optimization</strong> &#8212; we haven&#8217;t changed the table layout in this release, which is one of the reasons so many plugins work fine with 2.5. We have added a few new indicies and made a few default fields more flexible based on some bottlenecks we found on WordPress.com, which now hosts 2.7 million WordPress blogs. It should be invisible to the application, just a bit faster on the database side.</p>\n<p><strong>$wpdb->prepare()</strong> &#8212; now almost all of the SQL in WordPress is prepared first, and the same functions are available to your plugins. This should prevent elementary SQL escaping issues.</p>\n<p><strong>Media buttons</strong> &#8212; the add media buttons above the post are both expandable, so you could have an &#8220;Add Google Map&#8221; button if you like, They can be overridden, so if you think you can do the video or audio tab better than we have you can replace the default.</p>\n<p><strong>Shortcode API</strong> &#8212; the new gallery functionality is powered by the new shortcode API. Shortcodes are little bracket-delineated strings that can be magically expanded at runtime to something more interesting. They give users a short, easy to type and copy/paste string they can move around their post without worrying about messing up complex HTML or embed codes. <a href=\"http://codex.wordpress.org/Shortcode_API\">The Shortcode API is fully documented</a>.</p>\n<p>Now you see why 2.5 took a little extra time. <img src=\'http://wordpress.org/development/wp-includes/images/smilies/icon_smile.gif\' alt=\':)\' class=\'wp-smiley\' /> </p>\n<h3>Upgrade Notes</h3>\n<p>2.5 does include security fixes so it is recommended for all users, the 2.3 branch will no longer be updated. <a href=\"http://codex.wordpress.org/Upgrading_WordPress\">The upgrade instructions for this version are pretty much the same as any other version</a>. The most important thing to check is your plugins, so if for example everything works except the new uploader, a legacy plugin might be causing a javascript error on the page and breaking it. If something goes wrong, the safest thing to do is turn your plugins off (we have a button to do them all at once, now) and turn them back on one-by-one, testing the problem along the way. This has solved almost everybody&#8217;s problems in testing, and it also lets you know which plugin author to show some love to so they&#8217;ll update their plugin, and which plugin authors already have so you can shower them with praises on your blog.</p>\n<p>One brief note about some of the new upload and plugin upgrade features, there are some edge-case hosting platforms, like versions of Lighttpd before 1.5 or over-agressive mod_security rules, which can break. If something isn&#8217;t working like it was looked in the screenshot, ask your host if there&#8217;s something on the server side which may be interfering. Hosts, feel free to join and post to <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">our wp-testers mailing list</a> if you have an environment that requires some extra code to work around. We&#8217;d be happy to include it in the next update.</p>\n<p><em>Quick tip:</em> in 2.5 you click the name of things to edit them, like your username to edit your profile or the title of a post to edit it.</p>\n<h3>The Community is Growing</h3>\n<p>More than growing, it&#8217;s on fire. We always talk about things like downloads, and the 2.3 branch has already had <strong>1.92 million downloads</strong> as I write this post, but this time we have some far more interesting information I&#8217;d like to share.</p>\n<p>There were over 1,200 commits to our repository since 2.3.0 and <strong>over 90 people were credited</strong> in them. This means in our core code, not plugins, there were at least 90 individual folks that contributed something high-quality enough that it made the cut to be part of the download you guys get today. I had no idea this group of people was so large.</p>\n<p>Outside of the <a href=\"http://wordpress.org/about/\">core commit team</a>, there was particular help from these people, in rough order of number of credits and tickets: <a href=\"http://blogwaffe.com/\">mdawaffe (Michael Adams)</a>, <a href=\"http://www.laptoptips.ca/\">azaozz (Andrew Ozz)</a>, <a href=\"http://nb.niichavo.org/\">nbachiyski (Nikolay Bachiyski)</a>, <a href=\"http://skeltoac.com/\">andy (Andy Skelton)</a>, <a href=\"http://iammattthomas.com/\">iammattthomas (Matt Thomas)</a>, <a href=\"http://thresholdstate.com/\">tellyworth (Alex Shiels)</a>, <a href=\"http://joseph.randomnetworks.com/\">josephscott (Joseph Scott)</a>, <a href=\"http://foolswisdom.com/\">lloydbudd (Lloyd Budd)</a>, <a href=\"http://dd32.id.au/\">DD32 (Dion)</a>, <a href=\"http://www.ilfilosofo.com/\">filosofo (Austin Matzko)</a>, <a href=\"http://engel.uk.to/\">hansengel (Hans Engel)</a>, <a href=\"http://www.freecharity.org.uk/\">pishmishy</a>, ffemtcj, <a href=\"http://www.viper007bond.com/\">Viper007Bond</a>, <a href=\"http://extralogical.net/\">ionfish (Benedict Eastaugh)</a>, <a href=\"http://www.poplarware.com/\">jhodgdon (Jennifer Hodgdon)</a>, <a href=\"http://ottodestruct.com/blog/\">Otto42</a>, <a href=\"http://www.cefm.ca/\">thee17 (Charles E. Free-Melvin)</a>, and <a href=\"http://www.buayacorp.com/\">xknown</a>. Also want to thank MichaelH and Lorelle on the documentation side, and moshu, Kafkaesqui, whooami, MichaelH, Otto42, and jeremyclark13 for helping with support.</p>\n<p>The 2.5 branch is nicknamed &#8220;Brecker&#8221; in honor of <a href=\"http://en.wikipedia.org/wiki/Michael_Brecker\">Michael Brecker</a>, an exceptionally talented saxophonist who could cross styles effortlessly and never stopped experimenting and pushing himself until he passed away last year.</p>\n<h3>New WordPress.org</h3>\n<p>All of this wasn&#8217;t enough, so in our copious spare time we decided to redesign WordPress.org to better match the aesthetics of the new dashboard and also to spruce up a few areas that needed lovin&#8217;. Some parts of the site, like the Codex, might show the old style for a day or two. We know, just give us a bit of time. Thanks to <a href=\"http://iammattthomas.com/\">Matt Thomas</a> for his epic effort in designing and coding the new site.</p>\n<h3>What&#8217;s Next</h3>\n<p>As always with WordPress, we don&#8217;t claim any of these features to be perfect, or to be better than everyone else in the world, but they are done by and for the people and the one thing we do promise is that with every release we listen and do our best to improve. </p>\n<p>2.5 is a major milestone for WordPress not because it added dozens of user-requested features, but because it reaffirms that we&#8217;re as passionate about blogging as the day we started. Our community is too fierce to rest on its laurels &#8212; contrary to what pundits claim, blogging is far from &#8220;finished&#8221; and every improvement just whets our appetite for more. And more is coming.</p>\n<p>It&#8217;s a good thing WordPress doesn&#8217;t limit the length of posts, because this one would have hit it. If you made it this far, thanks for sharing a bit of your day with us. I sincerely hope this new version of WordPress helps you do what you love to do.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:67:\"http://wordpress.org/development/2008/03/wordpress-25-brecker/feed/\";}s:7:\"summary\";s:391:\"WordPress 2.5, the culmination of six months of work by the WordPress community, people just like you. The improvements in 2.5 are numerous, and almost entirely a result of your feedback: multi-file uploading, one-click plugin upgrades, built-in galleries, customizable dashboard, salted passwords and cookie encryption, media library, a WYSIWYG that doesn&#8217;t mess with your code, [...]\";s:12:\"atom_content\";s:15112:\"<p>WordPress 2.5, the culmination of six months of work by the WordPress community, people just like you. The improvements in 2.5 are numerous, and almost entirely a result of your feedback: multi-file uploading, one-click plugin upgrades, built-in galleries, customizable dashboard, salted passwords and cookie encryption, media library, a WYSIWYG that doesn&#8217;t mess with your code, concurrent post editing protection, full-screen writing, and search that covers posts <em>and</em> pages.</p>\n<p>For a short overview of the features with screenshots, it&#8217;d be best to <a href=\"http://wordpress.org/development/2008/03/25-sneak-peek/\">visit our sneak peek announcement for RC1</a>. Or check out <a href=\"http://wordpress.org/development/2008/03/wordpress-25-rc2/\">a 4-minute screencast of the new interface in action</a>. If you just want to jump straight to the good stuff <a href=\"http://wordpress.org/download/\">here&#8217;s where you can find 2.5 upgrade and download information</a>.</p>\n<p>If you want to see everything I would grab a cup of coffee or a mojito, because this post is epic.</p>\n<h3>User Features</h3>\n<p><strong>Cleaner, faster, less cluttered dashboard</strong> &#8212; we&#8217;ve worked hard to take your feedback about what&#8217;s most important in the dashboard and organize things to allow you to focus on what&#8217;s important &#8212; your blog &#8212; and get out of your way. In collaboration with <a href=\"http://happycog.com/\">Happy Cog</a> and the community  we&#8217;ve taken the first major step forward in the WordPress interface since version 1.5.</p>\n<p><strong>Dashboard Widgets</strong> &#8212; the dashboard home page is now a series of widgets, including ones to show you fun stats about your posting, latest comments, people linking to you, new and popular plugins, and of course WordPress news. You can customize any of the dashboard widgets to show, for example, news from your local paper instead of WP news. Plugins can also hook in, for example the <a href=\"http://wordpress.org/extend/plugins/stats/\">WordPress.com stats widget</a> adds a handy double-wide stats box.</p>\n<p><img src=\"http://s.wordpress.org/screenshots/2.5/stats-widget.png\" alt=\"\" width=\"690\" /></p>\n<p><strong>Multi-file upload with progress bar</strong> &#8212; before when you would upload a large file you&#8217;d wait forever, never knowing how far along it was. And uploading more than one photo was an exercise in patience, as you could only do one at a time. Now you can select a whole of folder images or music or videos at once and it&#8217;ll show you the progress of each upload.</p>\n<p><strong>Bonus: EXIF extraction</strong> &#8212; if you upload JPEG files with EXIF metadata like camera make and model, aperture, shutter speed, ISO, et al. WordPress will extract all the data into custom fields you can use in your template. If you use the EXIF title fields or similar those will be put into their equivalent fields in WP. Most modern digital cameras generate EXIF data.</p>\n<p><strong>Search posts and pages</strong> &#8212; search used to cover just posts, now it includes pages too, a great boon for those using WordPress as a CMS. New themes can style or sort pages differently in results.</p>\n<p><strong>Tag management</strong> &#8212; you can now add, rename, delete, and do whatever else you like to tags from inside WordPress, no plugins needed.</p>\n<p><strong>Password strength meter</strong> &#8212; when you change your password on your profile it&#8217;ll tell you how strong your password is to help you pick a good one.</p>\n<p><strong>Concurrent editing protection</strong> &#8212; for those of you on multi-author blogs, have you ever opened a post while someone was already editing it, and your auto-saves kept overwriting each other, irrecoverably losing hours of work? I bet that added a few words to your vocabulary. Now if you open a post that someone else is editing, WordPress magically locks it and prevents you from saving until the other person is done. You&#8217;ll see a message like below.</p>\n<p><img src=\"http://s.wordpress.org/screenshots/2.5/write-currently-editing.png\" alt=\"\" /></p>\n<p><strong>Few-click plugin upgrades</strong> &#8212; if the plugins you use are part of the plugin directory since 2.3 we&#8217;ve told you when they have an update available. Now we take that to the next logical step &#8212; downloading and installing the upgrade for you. This is dependent a little bit on your host setup, and it may ask you for your FTP password much like OS X or Windows will ask you for a password, but it works well on majority of hosts we were able to test, your mileage may very, plugins in mirror may be larger than they appear.</p>\n<p><strong>Friendlier visual post editor</strong> &#8212; I&#8217;m not sure how to articulate this improvement except to say &#8220;it doesn&#8217;t mess with your code anymore.&#8221; We&#8217;re now using version 3.0 of TinyMCE, which means better compatibility with Safari, and we&#8217;ve paid particular attention this release to its integration and interaction with complex HTML. It also now has a &#8220;no-distractions&#8221; mode which is like Writeroom for your browser.</p>\n<p><strong>Built-in galleries</strong> &#8212; when you take advantage of multi-file upload to upload a bunch of photos, we have a new shortcode that lets you to easily embed galleries by just putting [ gallery] (without the space) in your post. It&#8217;ll display all your thumbnails and captions and each will link each to a page where people can comment on the individual photos. I&#8217;ve been using this feature on my blog and have already uploaded over <a href=\"http://ma.tt/category/gallery/\">1,200 pictures into 23 galleries</a>. The shortcode has some hidden options too, <a href=\"http://codex.wordpress.org/Using_the_gallery_shortcode\">check out this documentation</a>.</p>\n<h3>Developer Features</h3>\n<p>Now for the geeky stuff. While we&#8217;re excited about the above features, each one represents a new opportunity or API for other developers to take to another level. (The best of which we&#8217;ll someday integrate back into WP.)</p>\n<p><strong>Salted passwords</strong> &#8212; we now use the <a href=\"http://www.openwall.com/phpass/\">phpass</a> library to stretch and salt all passwords stored in the database, which makes brute-forcing them impractical. If you use something like <a href=\"http://modauthmysql.sourceforge.net/\">mod_auth_mysql</a> we&#8217;ve <a href=\"http://wordpress.org/extend/plugins/md5-password-hashes/\">created a plugin that will allow you to use legacy MD5 hashing</a>. (The hashing is completely pluggable.) Users will automatically switch to the more secure passwords next time they log in.</p>\n<p><strong>Secure cookies</strong> &#8212; cookies are now encrypted based on the protocol described <a href=\"http://www.cse.msu.edu/~alexliu/publications/Cookie/cookie.pdf\">in this PDF paper</a>. which is something like <code>user name|expiration time|HMAC( user name|expiration time, k)</code> where <code>k = HMAC(user name|expiration time, sk)</code> and where <code>sk</code> is a secret key, which you can define in your config.</p>\n<p><strong>Easy taxonomy and URL creation</strong> &#8212; probably best illustrated with an example: I can call <code>register_taxonomy()</code> with <a href=\"http://trac.wordpress.org/attachment/ticket/6357/taxes.php\">a few arguments</a> to register a &#8220;people&#8221; taxonomy and whenever I edit an image I&#8217;ll see a UI like tags has for identifying the people in a photo, and these will be URL addressable with <code>/person/firstname-lastname/</code>. All with a single function call.</p>\n<p><strong>Inline documentation</strong> &#8212; the vast majority of the new code going into WordPress include inline documentation that explains the functions and documents their arguments.</p>\n<p><strong>Database optimization</strong> &#8212; we haven&#8217;t changed the table layout in this release, which is one of the reasons so many plugins work fine with 2.5. We have added a few new indicies and made a few default fields more flexible based on some bottlenecks we found on WordPress.com, which now hosts 2.7 million WordPress blogs. It should be invisible to the application, just a bit faster on the database side.</p>\n<p><strong>$wpdb->prepare()</strong> &#8212; now almost all of the SQL in WordPress is prepared first, and the same functions are available to your plugins. This should prevent elementary SQL escaping issues.</p>\n<p><strong>Media buttons</strong> &#8212; the add media buttons above the post are both expandable, so you could have an &#8220;Add Google Map&#8221; button if you like, They can be overridden, so if you think you can do the video or audio tab better than we have you can replace the default.</p>\n<p><strong>Shortcode API</strong> &#8212; the new gallery functionality is powered by the new shortcode API. Shortcodes are little bracket-delineated strings that can be magically expanded at runtime to something more interesting. They give users a short, easy to type and copy/paste string they can move around their post without worrying about messing up complex HTML or embed codes. <a href=\"http://codex.wordpress.org/Shortcode_API\">The Shortcode API is fully documented</a>.</p>\n<p>Now you see why 2.5 took a little extra time. <img src=\'http://wordpress.org/development/wp-includes/images/smilies/icon_smile.gif\' alt=\':)\' class=\'wp-smiley\' /> </p>\n<h3>Upgrade Notes</h3>\n<p>2.5 does include security fixes so it is recommended for all users, the 2.3 branch will no longer be updated. <a href=\"http://codex.wordpress.org/Upgrading_WordPress\">The upgrade instructions for this version are pretty much the same as any other version</a>. The most important thing to check is your plugins, so if for example everything works except the new uploader, a legacy plugin might be causing a javascript error on the page and breaking it. If something goes wrong, the safest thing to do is turn your plugins off (we have a button to do them all at once, now) and turn them back on one-by-one, testing the problem along the way. This has solved almost everybody&#8217;s problems in testing, and it also lets you know which plugin author to show some love to so they&#8217;ll update their plugin, and which plugin authors already have so you can shower them with praises on your blog.</p>\n<p>One brief note about some of the new upload and plugin upgrade features, there are some edge-case hosting platforms, like versions of Lighttpd before 1.5 or over-agressive mod_security rules, which can break. If something isn&#8217;t working like it was looked in the screenshot, ask your host if there&#8217;s something on the server side which may be interfering. Hosts, feel free to join and post to <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">our wp-testers mailing list</a> if you have an environment that requires some extra code to work around. We&#8217;d be happy to include it in the next update.</p>\n<p><em>Quick tip:</em> in 2.5 you click the name of things to edit them, like your username to edit your profile or the title of a post to edit it.</p>\n<h3>The Community is Growing</h3>\n<p>More than growing, it&#8217;s on fire. We always talk about things like downloads, and the 2.3 branch has already had <strong>1.92 million downloads</strong> as I write this post, but this time we have some far more interesting information I&#8217;d like to share.</p>\n<p>There were over 1,200 commits to our repository since 2.3.0 and <strong>over 90 people were credited</strong> in them. This means in our core code, not plugins, there were at least 90 individual folks that contributed something high-quality enough that it made the cut to be part of the download you guys get today. I had no idea this group of people was so large.</p>\n<p>Outside of the <a href=\"http://wordpress.org/about/\">core commit team</a>, there was particular help from these people, in rough order of number of credits and tickets: <a href=\"http://blogwaffe.com/\">mdawaffe (Michael Adams)</a>, <a href=\"http://www.laptoptips.ca/\">azaozz (Andrew Ozz)</a>, <a href=\"http://nb.niichavo.org/\">nbachiyski (Nikolay Bachiyski)</a>, <a href=\"http://skeltoac.com/\">andy (Andy Skelton)</a>, <a href=\"http://iammattthomas.com/\">iammattthomas (Matt Thomas)</a>, <a href=\"http://thresholdstate.com/\">tellyworth (Alex Shiels)</a>, <a href=\"http://joseph.randomnetworks.com/\">josephscott (Joseph Scott)</a>, <a href=\"http://foolswisdom.com/\">lloydbudd (Lloyd Budd)</a>, <a href=\"http://dd32.id.au/\">DD32 (Dion)</a>, <a href=\"http://www.ilfilosofo.com/\">filosofo (Austin Matzko)</a>, <a href=\"http://engel.uk.to/\">hansengel (Hans Engel)</a>, <a href=\"http://www.freecharity.org.uk/\">pishmishy</a>, ffemtcj, <a href=\"http://www.viper007bond.com/\">Viper007Bond</a>, <a href=\"http://extralogical.net/\">ionfish (Benedict Eastaugh)</a>, <a href=\"http://www.poplarware.com/\">jhodgdon (Jennifer Hodgdon)</a>, <a href=\"http://ottodestruct.com/blog/\">Otto42</a>, <a href=\"http://www.cefm.ca/\">thee17 (Charles E. Free-Melvin)</a>, and <a href=\"http://www.buayacorp.com/\">xknown</a>. Also want to thank MichaelH and Lorelle on the documentation side, and moshu, Kafkaesqui, whooami, MichaelH, Otto42, and jeremyclark13 for helping with support.</p>\n<p>The 2.5 branch is nicknamed &#8220;Brecker&#8221; in honor of <a href=\"http://en.wikipedia.org/wiki/Michael_Brecker\">Michael Brecker</a>, an exceptionally talented saxophonist who could cross styles effortlessly and never stopped experimenting and pushing himself until he passed away last year.</p>\n<h3>New WordPress.org</h3>\n<p>All of this wasn&#8217;t enough, so in our copious spare time we decided to redesign WordPress.org to better match the aesthetics of the new dashboard and also to spruce up a few areas that needed lovin&#8217;. Some parts of the site, like the Codex, might show the old style for a day or two. We know, just give us a bit of time. Thanks to <a href=\"http://iammattthomas.com/\">Matt Thomas</a> for his epic effort in designing and coding the new site.</p>\n<h3>What&#8217;s Next</h3>\n<p>As always with WordPress, we don&#8217;t claim any of these features to be perfect, or to be better than everyone else in the world, but they are done by and for the people and the one thing we do promise is that with every release we listen and do our best to improve. </p>\n<p>2.5 is a major milestone for WordPress not because it added dozens of user-requested features, but because it reaffirms that we&#8217;re as passionate about blogging as the day we started. Our community is too fierce to rest on its laurels &#8212; contrary to what pundits claim, blogging is far from &#8220;finished&#8221; and every improvement just whets our appetite for more. And more is coming.</p>\n<p>It&#8217;s a good thing WordPress doesn&#8217;t limit the length of posts, because this one would have hit it. If you made it this far, thanks for sharing a bit of your day with us. I sincerely hope this new version of WordPress helps you do what you love to do.</p>\n\";}i:8;a:12:{s:5:\"title\";s:32:\"Screencast and WordPress 2.5 RC2\";s:4:\"link\";s:58:\"http://wordpress.org/development/2008/03/wordpress-25-rc2/\";s:8:\"comments\";s:67:\"http://wordpress.org/development/2008/03/wordpress-25-rc2/#comments\";s:7:\"pubdate\";s:31:\"Wed, 26 Mar 2008 11:36:25 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:11:\"Development\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=227\";s:11:\"description\";s:334:\"2.5 is coming along thanks to the fantastic feedback you guys provided on RC1 (over 580 pingbacks and counting), and we&#8217;re now ready to show you a bit more of a peek with a short screencast covering the new dashboard and uploader and Release Candidate 2. First here&#8217;s the screencast, which is also available embedded [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:2991:\"<p>2.5 is coming along thanks to the <a href=\"http://wordpress.org/development/2008/03/25-sneak-peek/\">fantastic feedback you guys provided on RC1</a> (over 580 pingbacks and counting), and we&#8217;re now ready to show you a bit more of a peek with a short screencast covering the new dashboard and uploader and Release Candidate 2. First here&#8217;s the screencast, which is also available embedded below, <a href=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.swf\">as a Flash movie</a>, or as a <a href=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.avi\">17mb AVI download</a>:</p>\n<p>            <object id=\"csSWF\" classid=\"clsid:d27cdb6e-ae6d-11cf-96b8-444553540000\" width=\"640\" height=\"355\" codebase=\"http://active.macromedia.com/flash7/cabs/ swflash.cab#version=9,0,28,0\"><param name=\"src\" value=\"\"/><param name=\"bgcolor\" value=\"#1a1a1a\"/><param name=\"quality\" value=\"best\"/><param name=\"allowScriptAccess\" value=\"always\"/><param name=\"allowFullScreen\" value=\"true\"/><param name=\"scale\" value=\"showall\"/><param name=\"flashVars\" value=\"autostart=false\"/><embed name=\"csSWF\" src=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.swf\" width=\"549\" height=\"304\" bgcolor=\"#1a1a1a\" quality=\"best\" allowScriptAccess=\"always\" allowFullScreen=\"true\" scale=\"showall\" flashVars=\"autostart=false\" pluginspage=\"http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash\"></embed></object></p>\n<p>I&#8217;ve <a href=\"http://ma.tt/category/gallery/\">uploaded more than a thousand photos already</a> into the new gallery system &#8212; it works.</p>\n<p>(This was my first screencast, but I hope we can have more on WordPress.org and our documentation in the future.)</p>\n<p>If you make frequent backups and you’re interested in helping us out with development by testing the very latest, <a href=\"http://wordpress.org/wordpress-2.5-RC2.zip\">download</a> and install <a href=\"http://wordpress.org/wordpress-2.5-RC2.zip\">Release Candidate 2</a> of WordPress 2.5, and <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">join our testers mailing list</a> to report any bugs you find in the code.</p>\n<p>Finally with regards to theme and plugin compatibility, we&#8217;ve had no reports of any broken themes in this upgrade, which makes sense because we didn&#8217;t really change anything core about themes, just added new optional capabilities like Gravatars. Plugins that work with the admin may require updating to take advantage of the new, cleaner UI in WordPress 2.5.</p>\n<p>The community has started to <a href=\"http://codex.wordpress.org/Plugins/Plugin_Compatibility/2.5\">keep a list here of which plugins work great and which don&#8217;t</a>. It&#8217;s worth looking at, or even better just deactivate your plugins before upgrading for 2.5 and let the built-in updater notify and give you one-click upgrades to plugins you have installed, assuming the developer is cool and has updated their code for 2.5 already.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:63:\"http://wordpress.org/development/2008/03/wordpress-25-rc2/feed/\";}s:7:\"summary\";s:334:\"2.5 is coming along thanks to the fantastic feedback you guys provided on RC1 (over 580 pingbacks and counting), and we&#8217;re now ready to show you a bit more of a peek with a short screencast covering the new dashboard and uploader and Release Candidate 2. First here&#8217;s the screencast, which is also available embedded [...]\";s:12:\"atom_content\";s:2991:\"<p>2.5 is coming along thanks to the <a href=\"http://wordpress.org/development/2008/03/25-sneak-peek/\">fantastic feedback you guys provided on RC1</a> (over 580 pingbacks and counting), and we&#8217;re now ready to show you a bit more of a peek with a short screencast covering the new dashboard and uploader and Release Candidate 2. First here&#8217;s the screencast, which is also available embedded below, <a href=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.swf\">as a Flash movie</a>, or as a <a href=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.avi\">17mb AVI download</a>:</p>\n<p>            <object id=\"csSWF\" classid=\"clsid:d27cdb6e-ae6d-11cf-96b8-444553540000\" width=\"640\" height=\"355\" codebase=\"http://active.macromedia.com/flash7/cabs/ swflash.cab#version=9,0,28,0\"><param name=\"src\" value=\"\"/><param name=\"bgcolor\" value=\"#1a1a1a\"/><param name=\"quality\" value=\"best\"/><param name=\"allowScriptAccess\" value=\"always\"/><param name=\"allowFullScreen\" value=\"true\"/><param name=\"scale\" value=\"showall\"/><param name=\"flashVars\" value=\"autostart=false\"/><embed name=\"csSWF\" src=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.swf\" width=\"549\" height=\"304\" bgcolor=\"#1a1a1a\" quality=\"best\" allowScriptAccess=\"always\" allowFullScreen=\"true\" scale=\"showall\" flashVars=\"autostart=false\" pluginspage=\"http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash\"></embed></object></p>\n<p>I&#8217;ve <a href=\"http://ma.tt/category/gallery/\">uploaded more than a thousand photos already</a> into the new gallery system &#8212; it works.</p>\n<p>(This was my first screencast, but I hope we can have more on WordPress.org and our documentation in the future.)</p>\n<p>If you make frequent backups and you’re interested in helping us out with development by testing the very latest, <a href=\"http://wordpress.org/wordpress-2.5-RC2.zip\">download</a> and install <a href=\"http://wordpress.org/wordpress-2.5-RC2.zip\">Release Candidate 2</a> of WordPress 2.5, and <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">join our testers mailing list</a> to report any bugs you find in the code.</p>\n<p>Finally with regards to theme and plugin compatibility, we&#8217;ve had no reports of any broken themes in this upgrade, which makes sense because we didn&#8217;t really change anything core about themes, just added new optional capabilities like Gravatars. Plugins that work with the admin may require updating to take advantage of the new, cleaner UI in WordPress 2.5.</p>\n<p>The community has started to <a href=\"http://codex.wordpress.org/Plugins/Plugin_Compatibility/2.5\">keep a list here of which plugins work great and which don&#8217;t</a>. It&#8217;s worth looking at, or even better just deactivate your plugins before upgrading for 2.5 and let the built-in updater notify and give you one-click upgrades to plugins you have installed, assuming the developer is cool and has updated their code for 2.5 already.</p>\n\";}i:9;a:12:{s:5:\"title\";s:14:\"2.5 Sneak Peek\";s:4:\"link\";s:55:\"http://wordpress.org/development/2008/03/25-sneak-peek/\";s:8:\"comments\";s:64:\"http://wordpress.org/development/2008/03/25-sneak-peek/#comments\";s:7:\"pubdate\";s:31:\"Tue, 18 Mar 2008 07:08:57 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:11:\"Development\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=226\";s:11:\"description\";s:360:\"A customizable dashboard, multi-file upload, built-in galleries, one-click plugin upgrades, tag management, built-in Gravatars, full text feeds, and faster load times sound interesting? Then WordPress 2.5 might be the release for you. It&#8217;s been in the oven for a while, and we&#8217;re finally ready to open the doors a bit to give you a taste.\nFor [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:5448:\"<p>A customizable dashboard, multi-file upload, built-in galleries, one-click plugin upgrades, tag management, built-in Gravatars, full text feeds, and faster load times sound interesting? Then WordPress 2.5 might be the release for you. It&#8217;s been in the oven for a while, and we&#8217;re finally ready to open the doors a bit to give you a taste.</p>\n<p>For the past few months, we’ve been working with our friends at Happy Cog &#8212; <a href=\"http://zeldman.com/\">Jeffrey Zeldman</a>, <a href=\"http://jasonsantamaria.com/\">Jason Santa Maria</a>, and <a href=\"http://bobulate.com/\">Liz Danzico</a> &#8212; to redesign WordPress from the ground-up. The result is a new way of interacting with WordPress that will remain familiar to seasoned users while improving the experience for everyone. This isn’t just a fresh coat of paint &#8212; we’ve re-thought the look of WordPress, as well as how it’s organized so that you can forget about the software and focus on your own creative pursuits.</p>\n<p>Here are a few vignettes of what&#8217;s in store.</p>\n<h3>The Dashboard</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/dashboard-wide.png\" alt=\"dashboard-wide.png\" /></p>\n<p>The Dashboard’s most important role is to inform quickly and get you to where you’re headed in the admin. In interviewing users, we found that most of you ignore the Dashboard entirely &#8212; its useful information being mostly hidden in an overly complex design. The new Dashboard is focused on the most relevant tasks at hand: a quick summary of what’s published and scheduled for publication, the latest comments and incoming links, blog stats, and WordPress updates and news. You can add your own RSS feeds and edit the way information is presented so that the new Dashboard conforms to the way you use WordPress.</p>\n<h3>Navigation</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/nav-wide.png\" alt=\"nav-wide.png\" /></p>\n<p>The WordPress navigation has confounded even sophisticated users. With the new design, we’ve cut the number of navigation options in half, separating the primary functions (writing, managing posts and pages, editing the blog’s design, and managing comments) from secondary functions. This presents information at a more comfortable pace, revealing only the information that’s necessary. Everything you need is still there &#8212; just better organized. (Especially for people new to WP.)</p>\n<h3>Write</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/write-wide.png\" alt=\"write-wide.png\" /></p>\n<p><img class=\"right\" src=\"http://wpcom.wordpress.com/files/2008/03/write3.png\" alt=\"write2.png\" /></p>\n<p>By far, the most frequently accessed part of WordPress is the Write screen. It gets the job done, but its myriad options can be overwhelming. The new write screen only displays the information that you’ll use most often. It displays the most common fields in a way that makes posting incredibly easy. Additional options are hidden away until you need them. The new Write screen anticipates the natural flow of the way you write, and is smart enough to remember the way you left it so that your preferred writing environment is always quickly available. The new visual editor even has a handy full-screen mode to help block out distractions while composing your newest post. (My personal favorite new feature.)</p>\n<h3>Manage</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/manage-wide.png\" alt=\"\" /></p>\n<p><img class=\"right\" src=\"http://wpcom.wordpress.com/files/2008/03/manage.png\" alt=\"\" /></p>\n<p>The Manage screens have been redesigned and unified so that now, managing your pages, posts, media, and comments all use similar, consistent interfaces. We’ve omitted superfluous information and made what’s important faster to find. We believe these changes will make you a faster, more proficient blogger.</p>\n<p>You might also notice there are some new colors, the dashboard feels much fresher and lighter. If you&#8217;re jonesing for the old look under your user options you can now select the &#8220;classic&#8221; colors and get those old blues back. (It&#8217;s also pluggable so people can easily add or share their own color schemes.)</p>\n<p>If you make frequent backups and you&#8217;re interested in helping us out with development by testing the new code, <a href=\"http://wordpress.org/wordpress-2.5-RC1.zip\">download</a> and install <a href=\"http://wordpress.org/wordpress-2.5-RC1.zip\">Release Candidate 1</a> of WordPress 2.5, and <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">join our testers mailing list</a> to report any bugs you find in the code.</p>\n<p>We&#8217;re also interested in feedback on the new interface and would love to hear your opinions, thoughts, rants, raves, and anything in between. We created a special email address just for the occasion: <a href=\"mailto:2.5-feedback@wordpress.org\">2.5-feedback@wordpress.org</a>.</p>\n<p>The software is basically done and stable, and could be released today, but we&#8217;d like to incorporate feedback from a wider audience before making it available to the general public. After a few days of your feedback we&#8217;ll set a final release date. Personally, I can&#8217;t wait. <img src=\'http://wordpress.org/development/wp-includes/images/smilies/icon_smile.gif\' alt=\':)\' class=\'wp-smiley\' /></p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:60:\"http://wordpress.org/development/2008/03/25-sneak-peek/feed/\";}s:7:\"summary\";s:360:\"A customizable dashboard, multi-file upload, built-in galleries, one-click plugin upgrades, tag management, built-in Gravatars, full text feeds, and faster load times sound interesting? Then WordPress 2.5 might be the release for you. It&#8217;s been in the oven for a while, and we&#8217;re finally ready to open the doors a bit to give you a taste.\nFor [...]\";s:12:\"atom_content\";s:5448:\"<p>A customizable dashboard, multi-file upload, built-in galleries, one-click plugin upgrades, tag management, built-in Gravatars, full text feeds, and faster load times sound interesting? Then WordPress 2.5 might be the release for you. It&#8217;s been in the oven for a while, and we&#8217;re finally ready to open the doors a bit to give you a taste.</p>\n<p>For the past few months, we’ve been working with our friends at Happy Cog &#8212; <a href=\"http://zeldman.com/\">Jeffrey Zeldman</a>, <a href=\"http://jasonsantamaria.com/\">Jason Santa Maria</a>, and <a href=\"http://bobulate.com/\">Liz Danzico</a> &#8212; to redesign WordPress from the ground-up. The result is a new way of interacting with WordPress that will remain familiar to seasoned users while improving the experience for everyone. This isn’t just a fresh coat of paint &#8212; we’ve re-thought the look of WordPress, as well as how it’s organized so that you can forget about the software and focus on your own creative pursuits.</p>\n<p>Here are a few vignettes of what&#8217;s in store.</p>\n<h3>The Dashboard</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/dashboard-wide.png\" alt=\"dashboard-wide.png\" /></p>\n<p>The Dashboard’s most important role is to inform quickly and get you to where you’re headed in the admin. In interviewing users, we found that most of you ignore the Dashboard entirely &#8212; its useful information being mostly hidden in an overly complex design. The new Dashboard is focused on the most relevant tasks at hand: a quick summary of what’s published and scheduled for publication, the latest comments and incoming links, blog stats, and WordPress updates and news. You can add your own RSS feeds and edit the way information is presented so that the new Dashboard conforms to the way you use WordPress.</p>\n<h3>Navigation</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/nav-wide.png\" alt=\"nav-wide.png\" /></p>\n<p>The WordPress navigation has confounded even sophisticated users. With the new design, we’ve cut the number of navigation options in half, separating the primary functions (writing, managing posts and pages, editing the blog’s design, and managing comments) from secondary functions. This presents information at a more comfortable pace, revealing only the information that’s necessary. Everything you need is still there &#8212; just better organized. (Especially for people new to WP.)</p>\n<h3>Write</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/write-wide.png\" alt=\"write-wide.png\" /></p>\n<p><img class=\"right\" src=\"http://wpcom.wordpress.com/files/2008/03/write3.png\" alt=\"write2.png\" /></p>\n<p>By far, the most frequently accessed part of WordPress is the Write screen. It gets the job done, but its myriad options can be overwhelming. The new write screen only displays the information that you’ll use most often. It displays the most common fields in a way that makes posting incredibly easy. Additional options are hidden away until you need them. The new Write screen anticipates the natural flow of the way you write, and is smart enough to remember the way you left it so that your preferred writing environment is always quickly available. The new visual editor even has a handy full-screen mode to help block out distractions while composing your newest post. (My personal favorite new feature.)</p>\n<h3>Manage</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/manage-wide.png\" alt=\"\" /></p>\n<p><img class=\"right\" src=\"http://wpcom.wordpress.com/files/2008/03/manage.png\" alt=\"\" /></p>\n<p>The Manage screens have been redesigned and unified so that now, managing your pages, posts, media, and comments all use similar, consistent interfaces. We’ve omitted superfluous information and made what’s important faster to find. We believe these changes will make you a faster, more proficient blogger.</p>\n<p>You might also notice there are some new colors, the dashboard feels much fresher and lighter. If you&#8217;re jonesing for the old look under your user options you can now select the &#8220;classic&#8221; colors and get those old blues back. (It&#8217;s also pluggable so people can easily add or share their own color schemes.)</p>\n<p>If you make frequent backups and you&#8217;re interested in helping us out with development by testing the new code, <a href=\"http://wordpress.org/wordpress-2.5-RC1.zip\">download</a> and install <a href=\"http://wordpress.org/wordpress-2.5-RC1.zip\">Release Candidate 1</a> of WordPress 2.5, and <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">join our testers mailing list</a> to report any bugs you find in the code.</p>\n<p>We&#8217;re also interested in feedback on the new interface and would love to hear your opinions, thoughts, rants, raves, and anything in between. We created a special email address just for the occasion: <a href=\"mailto:2.5-feedback@wordpress.org\">2.5-feedback@wordpress.org</a>.</p>\n<p>The software is basically done and stable, and could be released today, but we&#8217;d like to incorporate feedback from a wider audience before making it available to the general public. After a few days of your feedback we&#8217;ll set a final release date. Personally, I can&#8217;t wait. <img src=\'http://wordpress.org/development/wp-includes/images/smilies/icon_smile.gif\' alt=\':)\' class=\'wp-smiley\' /></p>\n\";}}s:7:\"channel\";a:7:{s:5:\"title\";s:26:\"WordPress Development Blog\";s:4:\"link\";s:32:\"http://wordpress.org/development\";s:11:\"description\";s:33:\"WordPress development and updates\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 22:38:37 +0000\";s:9:\"generator\";s:36:\"http://wordpress.org/?v=2.7-bleeding\";s:8:\"language\";s:2:\"en\";s:7:\"tagline\";s:33:\"WordPress development and updates\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:13:\"last_modified\";s:31:\"Thu, 31 Jul 2008 22:38:37 GMT\r\n\";s:4:\"etag\";s:36:\"\"1992f063a21d60f93b6bc15a6a72ae93\"\r\n\";}','no'),(92,0,'rss_0ff4b43bd116a9d8720d689c80e7dfd4_ts','1217943920','no'),(93,0,'rss_57bc725ad6568758915363af670fd8bc','O:9:\"magpierss\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:15:{i:0;a:7:{s:5:\"title\";s:30:\"nemo.vrusso on \"Wp-UnitPNGfix\"\";s:4:\"link\";s:60:\"http://wordpress.org/extend/plugins/wp-unitpngfix/#post-6529\";s:7:\"pubdate\";s:31:\"Sat, 02 Aug 2008 10:02:16 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"nemo.vrusso\";}s:4:\"guid\";s:41:\"6529@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"This plugin includes the \'unitpngfix.js\' javascript file if the browser is IE6 or lower. In plain words, it implements the solution for the PNG traspa\";s:7:\"summary\";s:150:\"This plugin includes the \'unitpngfix.js\' javascript file if the browser is IE6 or lower. In plain words, it implements the solution for the PNG traspa\";}i:1;a:7:{s:5:\"title\";s:52:\"katzwebdesign on \"Constant Contact WordPress Widget\"\";s:4:\"link\";s:82:\"http://wordpress.org/extend/plugins/constant-contact-signup-form-widget/#post-6527\";s:7:\"pubdate\";s:31:\"Sat, 02 Aug 2008 09:12:48 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"katzwebdesign\";}s:4:\"guid\";s:41:\"6527@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:104:\"This widget adds a Constant Contact signup form to your sidebar and allows\nyou to configure the display.\";s:7:\"summary\";s:104:\"This widget adds a Constant Contact signup form to your sidebar and allows\nyou to configure the display.\";}i:2;a:7:{s:5:\"title\";s:30:\"bigdawggi on \"Weather Postin\'\"\";s:4:\"link\";s:61:\"http://wordpress.org/extend/plugins/weather-postin/#post-6524\";s:7:\"pubdate\";s:31:\"Fri, 01 Aug 2008 18:49:13 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:9:\"bigdawggi\";}s:4:\"guid\";s:41:\"6524@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:125:\"Saves current weather information when a post is published.  Weather information is then displayed after the post\'s contents.\";s:7:\"summary\";s:125:\"Saves current weather information when a post is published.  Weather information is then displayed after the post\'s contents.\";}i:3;a:7:{s:5:\"title\";s:20:\"e-sense on \"Statrix\"\";s:4:\"link\";s:54:\"http://wordpress.org/extend/plugins/statrix/#post-6522\";s:7:\"pubdate\";s:31:\"Fri, 01 Aug 2008 11:42:52 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"e-sense\";}s:4:\"guid\";s:41:\"6522@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:144:\"Generates real-time statistics for your blog. You can browse, filter and export detailed data about visitors, spiders, search terms and engines.\";s:7:\"summary\";s:144:\"Generates real-time statistics for your blog. You can browse, filter and export detailed data about visitors, spiders, search terms and engines.\";}i:4;a:7:{s:5:\"title\";s:25:\"katemag on \"Private Only\"\";s:4:\"link\";s:59:\"http://wordpress.org/extend/plugins/private-only/#post-6521\";s:7:\"pubdate\";s:31:\"Fri, 01 Aug 2008 06:05:07 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"katemag\";}s:4:\"guid\";s:41:\"6521@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:47:\"Redirects all non-logged in users to login form\";s:7:\"summary\";s:47:\"Redirects all non-logged in users to login form\";}i:5;a:7:{s:5:\"title\";s:54:\"Viper007Bond on \"jQuery Lightbox For Native Galleries\"\";s:4:\"link\";s:83:\"http://wordpress.org/extend/plugins/jquery-lightbox-for-native-galleries/#post-6519\";s:7:\"pubdate\";s:31:\"Fri, 01 Aug 2008 04:49:44 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Viper007Bond\";}s:4:\"guid\";s:41:\"6519@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:128:\"Makes the native WordPress galleries introduced in WordPress 2.5 use jQuery Lightbox by balupton to display the fullsize images.\";s:7:\"summary\";s:128:\"Makes the native WordPress galleries introduced in WordPress 2.5 use jQuery Lightbox by balupton to display the fullsize images.\";}i:6;a:7:{s:5:\"title\";s:28:\"thegary on \"Last.fm Artists\"\";s:4:\"link\";s:61:\"http://wordpress.org/extend/plugins/lastfm-artists/#post-6518\";s:7:\"pubdate\";s:31:\"Fri, 01 Aug 2008 03:11:12 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"thegary\";}s:4:\"guid\";s:41:\"6518@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:124:\"Last.fm Artists is a Wordpress widget that allows for user to display images of their top listened to artists on their blog.\";s:7:\"summary\";s:124:\"Last.fm Artists is a Wordpress widget that allows for user to display images of their top listened to artists on their blog.\";}i:7;a:7:{s:5:\"title\";s:22:\"leandrow on \"iLast.Fm\"\";s:4:\"link\";s:54:\"http://wordpress.org/extend/plugins/ilastfm/#post-6516\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 21:26:04 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"leandrow\";}s:4:\"guid\";s:41:\"6516@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:137:\"iLast.Fm provides a complete integration between your blog and your Last.fm account, show what musics you are listening to and much more!\";s:7:\"summary\";s:137:\"iLast.Fm provides a complete integration between your blog and your Last.fm account, show what musics you are listening to and much more!\";}i:8;a:7:{s:5:\"title\";s:33:\"GSaraiva on \"Developer Formatter\"\";s:4:\"link\";s:59:\"http://wordpress.org/extend/plugins/devformatter/#post-6512\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 19:25:04 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"GSaraiva\";}s:4:\"guid\";s:41:\"6512@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:87:\"Developer Formatter system to WordPress. Powered by GeSHi server-side highlight system.\";s:7:\"summary\";s:87:\"Developer Formatter system to WordPress. Powered by GeSHi server-side highlight system.\";}i:9;a:7:{s:5:\"title\";s:23:\"digsite on \"Live+Press\"\";s:4:\"link\";s:56:\"http://wordpress.org/extend/plugins/livepress/#post-6511\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 18:43:10 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"digsite\";}s:4:\"guid\";s:41:\"6511@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"Crosspost blog entries to LiveJournal automatically or on a post by post basis. Crosspostable options include user pic, current music, current mood, s\";s:7:\"summary\";s:150:\"Crosspost blog entries to LiveJournal automatically or on a post by post basis. Crosspostable options include user pic, current music, current mood, s\";}i:10;a:7:{s:5:\"title\";s:25:\"joostdevalk on \"Upcoming\"\";s:4:\"link\";s:55:\"http://wordpress.org/extend/plugins/upcoming/#post-6508\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 14:05:29 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"joostdevalk\";}s:4:\"guid\";s:41:\"6508@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:106:\"Easily create a list of your upcoming events on your blog. Use the widget or include it in a post or page.\";s:7:\"summary\";s:106:\"Easily create a list of your upcoming events on your blog. Use the widget or include it in a post or page.\";}i:11;a:7:{s:5:\"title\";s:30:\"weefselkweekje on \"WP-Cumulus\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/wp-cumulus/#post-6499\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 10:00:57 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:14:\"weefselkweekje\";}s:4:\"guid\";s:41:\"6499@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:91:\"WP-Cumulus displays your tags and/or categories in 3D by placing them on a rotating sphere.\";s:7:\"summary\";s:91:\"WP-Cumulus displays your tags and/or categories in 3D by placing them on a rotating sphere.\";}i:12;a:7:{s:5:\"title\";s:45:\"shane.pearlman on \"S&P Wavewatch Surf Widget\"\";s:4:\"link\";s:68:\"http://wordpress.org/extend/plugins/wavewatch-surf-widget/#post-6498\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 05:51:59 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:14:\"shane.pearlman\";}s:4:\"guid\";s:41:\"6498@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:121:\"Widget that displays either current condition or 3 day animated swell forecast. Select from over 60 regions of the world.\";s:7:\"summary\";s:121:\"Widget that displays either current condition or 3 day animated swell forecast. Select from over 60 regions of the world.\";}i:13;a:7:{s:5:\"title\";s:39:\"joostdevalk on \"Minimum Comment Length\"\";s:4:\"link\";s:69:\"http://wordpress.org/extend/plugins/minimum-comment-length/#post-6497\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 05:42:53 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"joostdevalk\";}s:4:\"guid\";s:41:\"6497@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:79:\"Check the comment for a set minimum length and disapprove it if it\'s too short.\";s:7:\"summary\";s:79:\"Check the comment for a set minimum length and disapprove it if it\'s too short.\";}i:14;a:7:{s:5:\"title\";s:48:\"ecooper on \"Possibly Related Classroom Projects\"\";s:4:\"link\";s:82:\"http://wordpress.org/extend/plugins/possibly-related-classroom-projects/#post-6495\";s:7:\"pubdate\";s:31:\"Thu, 31 Jul 2008 03:27:29 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"ecooper\";}s:4:\"guid\";s:41:\"6495@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:144:\"\"Possibly Related Classroom Projects\" enables you to share relevant classroom projects from DonorsChoose.org based on the content of your posts.\";s:7:\"summary\";s:144:\"\"Possibly Related Classroom Projects\" enables you to share relevant classroom projects from DonorsChoose.org based on the content of your posts.\";}}s:7:\"channel\";a:6:{s:5:\"title\";s:6:\"Newest\";s:4:\"link\";s:36:\"http://wordpress.org/extend/plugins/\";s:11:\"description\";s:6:\"Newest\";s:8:\"language\";s:2:\"en\";s:7:\"pubdate\";s:31:\"Tue, 05 Aug 2008 13:45:23 +0000\";s:7:\"tagline\";s:6:\"Newest\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:13:\"last_modified\";s:21:\"2008-08-02 10:02:16\r\n\";s:4:\"etag\";s:36:\"\"2d1fb76724401ef681fb70973d76d7c1\"\r\n\";}','no'),(94,0,'rss_57bc725ad6568758915363af670fd8bc_ts','1217943920','no'),(95,0,'rss_1a5f760f2e2b48827d4974a60857e7c2','O:9:\"magpierss\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:15:{i:0;a:7:{s:5:\"title\";s:29:\"plugin-master on \"d13gallery\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/d13gallery/#post-5534\";s:7:\"pubdate\";s:31:\"Wed, 07 May 2008 13:52:12 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"plugin-master\";}s:4:\"guid\";s:41:\"5534@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:97:\"Simply and quickly add thumbnail image galleries to your posts and pages using a html-like tag...\";s:7:\"summary\";s:97:\"Simply and quickly add thumbnail image galleries to your posts and pages using a html-like tag...\";}i:1;a:7:{s:5:\"title\";s:27:\"scoopMedia on \"twitterDash\"\";s:4:\"link\";s:58:\"http://wordpress.org/extend/plugins/twitterdash/#post-6463\";s:7:\"pubdate\";s:31:\"Mon, 28 Jul 2008 16:27:38 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"scoopMedia\";}s:4:\"guid\";s:41:\"6463@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:53:\"twitterDash adds twitter to your WordPress-dashboard.\";s:7:\"summary\";s:53:\"twitterDash adds twitter to your WordPress-dashboard.\";}i:2;a:7:{s:5:\"title\";s:21:\"BjornW on \"bbWP2UTF8\"\";s:4:\"link\";s:56:\"http://wordpress.org/extend/plugins/bbwp2utf8/#post-6154\";s:7:\"pubdate\";s:31:\"Tue, 01 Jul 2008 14:06:16 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"BjornW\";}s:4:\"guid\";s:41:\"6154@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:119:\"This plugin will attempt to convert a wordpress or wordpress mu database with content in whatever character set to utf8\";s:7:\"summary\";s:119:\"This plugin will attempt to convert a wordpress or wordpress mu database with content in whatever character set to utf8\";}i:3;a:7:{s:5:\"title\";s:35:\"pwayner on \"LinkShare RSS DealFeed\"\";s:4:\"link\";s:63:\"http://wordpress.org/extend/plugins/linksharerss-ads/#post-6491\";s:7:\"pubdate\";s:31:\"Wed, 30 Jul 2008 18:58:09 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"pwayner\";}s:4:\"guid\";s:41:\"6491@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"LinkShare RSS DealFeed uses RSS feeds from LinkShare Advertisers to automatically populate your Wordpress blog with deals and other promotional conten\";s:7:\"summary\";s:150:\"LinkShare RSS DealFeed uses RSS feeds from LinkShare Advertisers to automatically populate your Wordpress blog with deals and other promotional conten\";}i:4;a:7:{s:5:\"title\";s:45:\"Flowdrops on \"iF AJAX Comments For WordPress\"\";s:4:\"link\";s:77:\"http://wordpress.org/extend/plugins/if-ajax-comments-for-wordpress/#post-6041\";s:7:\"pubdate\";s:31:\"Thu, 19 Jun 2008 11:49:53 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:9:\"Flowdrops\";}s:4:\"guid\";s:41:\"6041@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:63:\"This plugin will enable AJAX commenting in your WordPress blog.\";s:7:\"summary\";s:63:\"This plugin will enable AJAX commenting in your WordPress blog.\";}i:5;a:7:{s:5:\"title\";s:24:\"aesqe on \"Decategorizer\"\";s:4:\"link\";s:60:\"http://wordpress.org/extend/plugins/decategorizer/#post-6135\";s:7:\"pubdate\";s:31:\"Sun, 29 Jun 2008 17:03:59 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:5:\"aesqe\";}s:4:\"guid\";s:41:\"6135@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:146:\"This plugin adds a filter (which removes category_base text from \npermalinks) on get_pagenum_link, wp_list_categories and the_category \nfunctions.\";s:7:\"summary\";s:146:\"This plugin adds a filter (which removes category_base text from \npermalinks) on get_pagenum_link, wp_list_categories and the_category \nfunctions.\";}i:6;a:7:{s:5:\"title\";s:17:\"Latz on \"pagebar\"\";s:4:\"link\";s:53:\"http://wordpress.org/extend/plugins/pagebar/#post-218\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 22:54:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Latz\";}s:4:\"guid\";s:40:\"218@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:92:\"Adds a nice page bar for selecting indivual pages instead of just the previous or next page.\";s:7:\"summary\";s:92:\"Adds a nice page bar for selecting indivual pages instead of just the previous or next page.\";}i:7;a:7:{s:5:\"title\";s:26:\"charlestang on \"WP Kit CN\"\";s:4:\"link\";s:56:\"http://wordpress.org/extend/plugins/wp-kit-cn/#post-6457\";s:7:\"pubdate\";s:31:\"Mon, 28 Jul 2008 07:46:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"charlestang\";}s:4:\"guid\";s:41:\"6457@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:79:\"This plugin extends wordpress by a series of template tags and sidebar widgets.\";s:7:\"summary\";s:79:\"This plugin extends wordpress by a series of template tags and sidebar widgets.\";}i:8;a:7:{s:5:\"title\";s:23:\"error on \"Bad Behavior\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/bad-behavior/#post-30\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 22:16:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:5:\"error\";}s:4:\"guid\";s:39:\"30@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"Welcome to a whole new way of keeping your blog, forum, guestbook, wiki or\ncontent management system free of link spam. Bad Behavior is a PHP-based\nso\";s:7:\"summary\";s:150:\"Welcome to a whole new way of keeping your blog, forum, guestbook, wiki or\ncontent management system free of link spam. Bad Behavior is a PHP-based\nso\";}i:9;a:7:{s:5:\"title\";s:22:\"neop on \"Postalicious\"\";s:4:\"link\";s:59:\"http://wordpress.org/extend/plugins/postalicious/#post-2507\";s:7:\"pubdate\";s:31:\"Mon, 15 Oct 2007 02:15:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"neop\";}s:4:\"guid\";s:41:\"2507@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"Postalicious is a WordPress plugin that automatically posts your delicious, ma.gnolia, or Google Reader, Reddit or Yahoo Pipes bookmarks to your blog.\";s:7:\"summary\";s:150:\"Postalicious is a WordPress plugin that automatically posts your delicious, ma.gnolia, or Google Reader, Reddit or Yahoo Pipes bookmarks to your blog.\";}i:10;a:7:{s:5:\"title\";s:17:\"dor on \"re.place\"\";s:4:\"link\";s:54:\"http://wordpress.org/extend/plugins/replace/#post-6260\";s:7:\"pubdate\";s:31:\"Fri, 11 Jul 2008 08:45:51 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:3:\"dor\";}s:4:\"guid\";s:41:\"6260@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:147:\"re.place adds content filter wich searches in posts for specified patterns\n(regular expressions) and replaces matches found with specified strings.\";s:7:\"summary\";s:147:\"re.place adds content filter wich searches in posts for specified patterns\n(regular expressions) and replaces matches found with specified strings.\";}i:11;a:7:{s:5:\"title\";s:30:\"markjaquith on \"Page Links To\"\";s:4:\"link\";s:59:\"http://wordpress.org/extend/plugins/page-links-to/#post-216\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 22:53:52 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"markjaquith\";}s:4:\"guid\";s:40:\"216@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:138:\"Page Links To allows you to make certain WordPress pages or posts link to a URL of your choosing, instead of their WordPress post or page.\";s:7:\"summary\";s:138:\"Page Links To allows you to make certain WordPress pages or posts link to a URL of your choosing, instead of their WordPress post or page.\";}i:12;a:7:{s:5:\"title\";s:22:\"katzgrau on \"wpSearch\"\";s:4:\"link\";s:55:\"http://wordpress.org/extend/plugins/wpsearch/#post-6149\";s:7:\"pubdate\";s:31:\"Mon, 30 Jun 2008 23:21:19 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"katzgrau\";}s:4:\"guid\";s:41:\"6149@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:145:\"wpSearch is a very powerful search plugin for Wordpress. It is based on the open source search engine \"Lucene\" which means its fast and relevant.\";s:7:\"summary\";s:145:\"wpSearch is a very powerful search plugin for Wordpress. It is based on the open source search engine \"Lucene\" which means its fast and relevant.\";}i:13;a:7:{s:5:\"title\";s:27:\"vhauri on \"Tagline Rotator\"\";s:4:\"link\";s:62:\"http://wordpress.org/extend/plugins/tagline-rotator/#post-6284\";s:7:\"pubdate\";s:31:\"Sun, 13 Jul 2008 22:05:32 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"vhauri\";}s:4:\"guid\";s:41:\"6284@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:87:\"Tagline Rotator plugin randomly selects a tagline from a list of user-entered taglines.\";s:7:\"summary\";s:87:\"Tagline Rotator plugin randomly selects a tagline from a list of user-entered taglines.\";}i:14;a:7:{s:5:\"title\";s:20:\"zeeg on \"Lifestream\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/lifestream/#post-6479\";s:7:\"pubdate\";s:31:\"Tue, 29 Jul 2008 22:10:24 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"zeeg\";}s:4:\"guid\";s:41:\"6479@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:115:\"Lifestream displays your social feeds and photos much like you would see it on many of the social networking sites.\";s:7:\"summary\";s:115:\"Lifestream displays your social feeds and photos much like you would see it on many of the social networking sites.\";}}s:7:\"channel\";a:6:{s:5:\"title\";s:16:\"Recently Updated\";s:4:\"link\";s:36:\"http://wordpress.org/extend/plugins/\";s:11:\"description\";s:16:\"Recently Updated\";s:8:\"language\";s:2:\"en\";s:7:\"pubdate\";s:31:\"Tue, 05 Aug 2008 13:45:23 +0000\";s:7:\"tagline\";s:16:\"Recently Updated\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:13:\"last_modified\";s:21:\"2008-05-07 13:52:12\r\n\";s:4:\"etag\";s:36:\"\"7b95c7e16d71c6db20dc6c8ac5cff014\"\r\n\";}','no'),(96,0,'rss_1a5f760f2e2b48827d4974a60857e7c2_ts','1217943920','no'),(97,0,'category_children','a:0:{}','yes'),(98,0,'cron','a:2:{i:1235770270;a:1:{s:17:\"wp_update_plugins\";a:1:{s:32:\"40cd750bba9870f18aada2478b24840a\";a:3:{s:8:\"schedule\";s:10:\"twicedaily\";s:4:\"args\";a:0:{}s:8:\"interval\";i:43200;}}}s:7:\"version\";i:2;}','yes'),(99,0,'doing_cron','0','yes');
/*!40000 ALTER TABLE `options` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `postmeta`
--

DROP TABLE IF EXISTS `postmeta`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `postmeta` (
  `meta_id` bigint(20) NOT NULL auto_increment,
  `post_id` bigint(20) NOT NULL default '0',
  `meta_key` varchar(255) default NULL,
  `meta_value` longtext,
  PRIMARY KEY  (`meta_id`),
  KEY `post_id` (`post_id`),
  KEY `meta_key` (`meta_key`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `postmeta`
--

LOCK TABLES `postmeta` WRITE;
/*!40000 ALTER TABLE `postmeta` DISABLE KEYS */;
/*!40000 ALTER TABLE `postmeta` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `posts`
--

DROP TABLE IF EXISTS `posts`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `posts` (
  `ID` bigint(20) unsigned NOT NULL auto_increment,
  `post_author` bigint(20) NOT NULL default '0',
  `post_date` datetime NOT NULL default '0000-00-00 00:00:00',
  `post_date_gmt` datetime NOT NULL default '0000-00-00 00:00:00',
  `post_content` longtext NOT NULL,
  `post_title` text NOT NULL,
  `post_category` int(4) NOT NULL default '0',
  `post_excerpt` text NOT NULL,
  `post_status` varchar(20) NOT NULL default 'publish',
  `comment_status` varchar(20) NOT NULL default 'open',
  `ping_status` varchar(20) NOT NULL default 'open',
  `post_password` varchar(20) NOT NULL default '',
  `post_name` varchar(200) NOT NULL default '',
  `to_ping` text NOT NULL,
  `pinged` text NOT NULL,
  `post_modified` datetime NOT NULL default '0000-00-00 00:00:00',
  `post_modified_gmt` datetime NOT NULL default '0000-00-00 00:00:00',
  `post_content_filtered` text NOT NULL,
  `post_parent` bigint(20) NOT NULL default '0',
  `guid` varchar(255) NOT NULL default '',
  `menu_order` int(11) NOT NULL default '0',
  `post_type` varchar(20) NOT NULL default 'post',
  `post_mime_type` varchar(100) NOT NULL default '',
  `comment_count` bigint(20) NOT NULL default '0',
  PRIMARY KEY  (`ID`),
  KEY `post_name` (`post_name`),
  KEY `type_status_date` (`post_type`,`post_status`,`post_date`,`ID`)
) ENGINE=MyISAM AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `posts`
--

LOCK TABLES `posts` WRITE;
/*!40000 ALTER TABLE `posts` DISABLE KEYS */;
INSERT INTO `posts` VALUES (1,1,'2008-07-14 22:33:23','2008-07-15 04:33:23','Welcome to WordPress. This is your first post. Edit or delete it, then start blogging!','Hello world!',0,'','publish','open','open','','hello-world','','','2008-07-14 22:33:23','2008-07-15 04:33:23','',0,'http://www.garff.org//?p=1',0,'post','',1),(2,1,'2008-07-14 22:33:23','2008-07-15 04:33:23','This is an example of a WordPress page, you could edit this to put information about yourself or your site so readers know where you are coming from. You can create as many pages like this one or sub-pages as you like and manage all of your content inside of WordPress.','About',0,'','publish','open','open','','about','','','2008-07-14 22:33:23','2008-07-15 04:33:23','',0,'http://www.garff.org//?page_id=2',0,'page','',0);
/*!40000 ALTER TABLE `posts` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `term_relationships`
--

DROP TABLE IF EXISTS `term_relationships`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `term_relationships` (
  `object_id` bigint(20) NOT NULL default '0',
  `term_taxonomy_id` bigint(20) NOT NULL default '0',
  `term_order` int(11) NOT NULL default '0',
  PRIMARY KEY  (`object_id`,`term_taxonomy_id`),
  KEY `term_taxonomy_id` (`term_taxonomy_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `term_relationships`
--

LOCK TABLES `term_relationships` WRITE;
/*!40000 ALTER TABLE `term_relationships` DISABLE KEYS */;
INSERT INTO `term_relationships` VALUES (1,2,0),(2,2,0),(3,2,0),(4,2,0),(5,2,0),(6,2,0),(7,2,0),(1,1,0);
/*!40000 ALTER TABLE `term_relationships` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `term_taxonomy`
--

DROP TABLE IF EXISTS `term_taxonomy`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `term_taxonomy` (
  `term_taxonomy_id` bigint(20) NOT NULL auto_increment,
  `term_id` bigint(20) NOT NULL default '0',
  `taxonomy` varchar(32) NOT NULL default '',
  `description` longtext NOT NULL,
  `parent` bigint(20) NOT NULL default '0',
  `count` bigint(20) NOT NULL default '0',
  PRIMARY KEY  (`term_taxonomy_id`),
  UNIQUE KEY `term_id_taxonomy` (`term_id`,`taxonomy`)
) ENGINE=MyISAM AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `term_taxonomy`
--

LOCK TABLES `term_taxonomy` WRITE;
/*!40000 ALTER TABLE `term_taxonomy` DISABLE KEYS */;
INSERT INTO `term_taxonomy` VALUES (1,1,'category','',0,1),(2,2,'link_category','',0,7);
/*!40000 ALTER TABLE `term_taxonomy` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `terms`
--

DROP TABLE IF EXISTS `terms`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `terms` (
  `term_id` bigint(20) NOT NULL auto_increment,
  `name` varchar(200) NOT NULL default '',
  `slug` varchar(200) NOT NULL default '',
  `term_group` bigint(10) NOT NULL default '0',
  PRIMARY KEY  (`term_id`),
  UNIQUE KEY `slug` (`slug`),
  KEY `name` (`name`)
) ENGINE=MyISAM AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `terms`
--

LOCK TABLES `terms` WRITE;
/*!40000 ALTER TABLE `terms` DISABLE KEYS */;
INSERT INTO `terms` VALUES (1,'Uncategorized','uncategorized',0),(2,'Blogroll','blogroll',0);
/*!40000 ALTER TABLE `terms` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `usermeta`
--

DROP TABLE IF EXISTS `usermeta`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `usermeta` (
  `umeta_id` bigint(20) NOT NULL auto_increment,
  `user_id` bigint(20) NOT NULL default '0',
  `meta_key` varchar(255) default NULL,
  `meta_value` longtext,
  PRIMARY KEY  (`umeta_id`),
  KEY `user_id` (`user_id`),
  KEY `meta_key` (`meta_key`)
) ENGINE=MyISAM AUTO_INCREMENT=8 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `usermeta`
--

LOCK TABLES `usermeta` WRITE;
/*!40000 ALTER TABLE `usermeta` DISABLE KEYS */;
INSERT INTO `usermeta` VALUES (1,1,'nickname','egarff'),(2,1,'rich_editing','true'),(3,1,'admin_color','fresh'),(4,1,'capabilities','a:1:{s:13:\"administrator\";b:1;}'),(5,1,'user_level','10'),(6,1,'first_name','Eric'),(7,1,'last_name','Garff');
/*!40000 ALTER TABLE `usermeta` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `users`
--

DROP TABLE IF EXISTS `users`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `users` (
  `ID` bigint(20) unsigned NOT NULL auto_increment,
  `user_login` varchar(60) NOT NULL default '',
  `user_pass` varchar(64) NOT NULL default '',
  `user_nicename` varchar(50) NOT NULL default '',
  `user_email` varchar(100) NOT NULL default '',
  `user_url` varchar(100) NOT NULL default '',
  `user_registered` datetime NOT NULL default '0000-00-00 00:00:00',
  `user_activation_key` varchar(60) NOT NULL default '',
  `user_status` int(11) NOT NULL default '0',
  `display_name` varchar(250) NOT NULL default '',
  PRIMARY KEY  (`ID`),
  KEY `user_login_key` (`user_login`),
  KEY `user_nicename` (`user_nicename`)
) ENGINE=MyISAM AUTO_INCREMENT=2 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `users`
--

LOCK TABLES `users` WRITE;
/*!40000 ALTER TABLE `users` DISABLE KEYS */;
INSERT INTO `users` VALUES (1,'admin','$P$9Xb48dRya9cDga/vKl0eb0HBG6smZn/','admin','eric@baldfury.net','http://www.garff.org/','2008-07-15 04:33:23','',0,'admin');
/*!40000 ALTER TABLE `users` ENABLE KEYS */;
UNLOCK TABLES;
/*!40103 SET TIME_ZONE=@OLD_TIME_ZONE */;

/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;

-- Dump completed on 2009-02-27 18:26:04
